/*--------------------------------------------------------------------
 * $Id: uvspec_lex.l,v 1.267 2008/01/08 19:51:40 arve Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

/* the "incl" state is used for picking up the name
 * of an include file
 */
%x include

%{

#include <getopt.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sun.h>
#include <string.h>
      
#include "uvspec.h"
#include "ascii.h"
#include "atmosphere.h"
#include "ancillary.h"
#include "netCDF_functions.h"

#if HAVE_LIBGSL 
  #include <gsl/gsl_rng.h>
#else
  #include "r250.h"
#endif


#if HAVE_OPENGL
  extern int GLmystic_hiddenline; /* hidden line removal */
#endif


static float  yytext2float(char* yytext, int which_token, int max_token);
static float* yytext2floats(char* yytext, int nary);
static double yytext2double(char* yytext, int which_token, int max_token);
static int    yytext2int(char* yytext, int which_token, int max_token);
static int*   yytext2integers(char* yytext, int nary);
static char*  yytext2string(char* yytext, int which_token, int max_token);
static int    yytext2ntokens(char* yytext);
static int    yytextCstring(char *dest, char* yytext, int which_token, int max_token);
static void free_output(input_struct input, output_struct *output);
#ifndef UVSPEC_FUNCTION
static int print_output(input_struct input, output_struct *output);
#else
static int print_output(input_struct input, output_struct *output, char *outfilename);
#endif
void write_flexstor_output     (input_struct Input, output_struct *output, FILE *f);
void write_polradtran_output   (input_struct Input, output_struct *output, FILE *f);
void write_header              (input_struct Input, output_struct *output, FILE *f);
void write_spectral_heat       (input_struct Input, output_struct *output, FILE *f);
void write_user_defined_output (input_struct Input, output_struct *output, FILE *f);
int check_if_result_exists     (int *output_user, int n_output_user, int solver, int **valid_result, int quiet);
int write_netCDF_output        (input_struct Input, output_struct *output);

char *s, *t;
int ntokens;
int mol_id;
float tmpfloat;
int   tmpint;
float* tmp;
int*   tmpi;
int ierror=0;
int line_number;
input_struct Input;
output_struct Output;
int i=0, j=0;
int token_number=0;
int position=0;
float epsilon=1E-7;

#define EPSILON 1.E-6        /*relative rounding error for floats */
#define MAX_INCLUDE_DEPTH 10
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

#ifndef PI
#define PI 3.14159265358979323846264338327
#endif

%}

%option noyywrap
%option nounput

DIGIT    [0-9]
SPACE    [ \t]
CCHAR    [#]
COMMENT  ({CCHAR}.*)
INTEGER  {DIGIT}+
SPACE_INTEGER {SPACE}+{INTEGER}
INTEGERS {SPACE_INTEGER}+ 
EXPON   ([DdEe][+-]?{DIGIT}+)
FLOAT    ((({DIGIT}*+"."{DIGIT}*)|({DIGIT}*+"."{DIGIT}*{EXPON}))|({INTEGER}))
SPACE_FLOAT {SPACE}+{FLOAT}
FLOATS   {SPACE_FLOAT}+ 
SIGN     [-+]
SIGNED_FLOAT {SIGN}?{FLOAT}
SPACE_SIGNED_FLOAT {SPACE}+{SIGNED_FLOAT}
SIGNED_FLOATS {SPACE_SIGNED_FLOAT}+
SIGNED_INTEGER {SIGN}?{INTEGER}
WORD     [_\.\/A-Za-z][_\.\/A-Za-z0-9\-]*
SPACE_WORD {SPACE}+{WORD}
WORDS {SPACE_WORD}*
FLOAT_OR_WORD ({SPACE_FLOAT}|{SPACE_WORD})
FLOATS_OR_WORDS {FLOAT_OR_WORD}*
SIGNED_FLOAT_OR_WORD ({SPACE_SIGNED_FLOAT}|{SPACE_WORD})
SIGNED_FLOATS_OR_WORDS {SIGNED_FLOAT_OR_WORD}*

%%

  /* These are the allowed input lines in the uvspec input file 
     The input line specifications must come before the eating of
     blanks, otherwise things get messed up 
     Obviously this whole lot could have been handled more properly
     using bison etc., but then rather few would know what was going 
     on??? 
   */


absorption{SPACE}+{WORD}{SPACE}+{WORD} { s = (char *) calloc (strlen(yytext)+1,sizeof(char));
            strcpy (s, yytext);
            ntokens = yytext2ntokens(s);
            free(s);
            fflush (stderr);
            s = yytext2string(yytext,2,ntokens+1);
            t = yytext2string(yytext,3,ntokens+1);
	    if (strncasecmp(t,"on",2)==0) {
	      if (     strncasecmp("O4",  s,2)==0) {Input.ck_o4abs   = 1;}
	      else if (strncasecmp("N2",  s,2)==0) {Input.ck_n2abs   = 1;}
	      else if (strncasecmp("CO" , s,2)==0) {Input.ck_coabs   = 1;}
	      else if (strncasecmp("SO2", s,3)==0) {Input.ck_so2abs  = 1;}
	      else if (strncasecmp("NH3", s,3)==0) {Input.ck_nh3abs  = 1;}
	      else if (strncasecmp("NO",  s,2)==0) {Input.ck_noabs   = 1;}
	      else if (strncasecmp("HNO3",s,4)==0) {Input.ck_hno3abs = 1;}
	      else {fprintf(stderr,"Unknown 'absorption %s' on line %d\n", s, line_number+1); ierror++;}
            }
	    else if (strncasecmp(t,"off",3)==0) {
	      if (     strncasecmp("O4",  s,2)==0) {Input.ck_o4abs   = 0;}
	      else if (strncasecmp("N2",  s,2)==0) {Input.ck_n2abs   = 0;}
	      else if (strncasecmp("CO" , s,2)==0) {Input.ck_coabs   = 0;}
	      else if (strncasecmp("SO2", s,3)==0) {Input.ck_so2abs  = 0;}
	      else if (strncasecmp("NH3", s,3)==0) {Input.ck_nh3abs  = 0;}
	      else if (strncasecmp("NO",  s,2)==0) {Input.ck_noabs   = 0;}
	      else if (strncasecmp("HNO3",s,4)==0) {Input.ck_hno3abs = 0;}
	      else {fprintf(stderr,"Unknown 'absorption %s' on line %d\n", s, line_number+1); ierror++;}
           }
	   free(s); free(t);}

aerosol_angstrom{SIGNED_FLOATS}     {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);
                                     Input.aer.spec++;}
aerosol_default                     {Input.aer.haze = 1;
                                     Input.aer.vulcan = 1; 
				     Input.aer.seasn = 1;
                                     Input.aer.visibility = 50.0;}
aerosol_files{SPACE}+{WORD}         {yytextCstring (Input.aer.filename[FN_AER_FILES],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_gg_file{SPACE}+{WORD}       {yytextCstring (Input.aer.filename[FN_AER_GG],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_haze{SPACE}+{INTEGER}       {Input.aer.haze     = yytext2int(yytext,2,2); }

aerosol_species_file{WORDS}         {ntokens = yytext2ntokens(yytext);
                                     if ( ntokens < 1 ) {
                                       fprintf (stderr, "Error, 'aerosol_species_file' on line %d needs at least one argument\n",line_number+1); 
                                       ierror++;
                                     }
                                     yytextCstring (Input.aer.filename[FN_AER_SPECIES], yytext, 1+1, ntokens+1);
                                     if ( ntokens > 1 ) {
                                       Input.aer.n_species = ntokens - 1; /* one token for file name */
                                       Input.aer.species_names = (char **) calloc (Input.aer.n_species, sizeof(char *));
                                       for (i=3;i<=ntokens+1; i++) {
				         Input.aer.species_names[i-3] = yytext2string(yytext, i, ntokens+1);
                                       }
                                     } 
                                     Input.aer.spec++; }
aerosol_species_library{SPACE}+{WORD} { 
                                     yytextCstring (Input.aer.filename[FN_AER_SPECIES_LIB], yytext, 2, 2);
                                    }
aerosol_moments_file{SPACE}+{WORD}  {yytextCstring (Input.aer.filename[FN_AER_MOM],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_refrac_file{SPACE}+{WORD}   {yytextCstring (Input.aer.filename[FN_AER_REF],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_refrac_index{SPACE_FLOAT}{SPACE_FLOAT} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmp   = yytext2floats(yytext, 2); 
                                     Input.aer.re = tmp[0];
				     Input.aer.im = tmp[1];
				     free(tmp); free(s); 
                                     Input.aer.spec++;}
aerosol_scale_ssa{SPACE}+{FLOAT}    {Input.aer.scale_ssa= yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_scale_tau{SPACE}+{FLOAT}    {Input.aer.scale_tau= yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_gg{SPACE}+{SIGNED_FLOAT} {Input.aer.set_gg   = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_ssa{SPACE}+{FLOAT}      {Input.aer.set_ssa  = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_tau{SPACE}+{FLOAT}      {Input.aer.set_tau  = yytext2float(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_set_tau550{SPACE}+{FLOAT}   {Input.aer.tau550 = yytext2float(yytext,2,2); }
aerosol_season{SPACE}+{INTEGER}     {Input.aer.seasn    = yytext2int(yytext,2,2); 
                                     Input.aer.spec++;}
aerosol_sizedist_file{SPACE}+{WORD} {yytextCstring (Input.aer.filename[FN_AER_SIZ], 
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_ssa_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_SSA], 
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_tau_file{SPACE}+{WORD}      {yytextCstring (Input.aer.filename[FN_AER_TAU],
						    yytext, 2, 2);
                                     Input.aer.spec++;}
aerosol_visibility{SPACE}+{FLOAT}   {Input.aer.visibility    = yytext2float(yytext,2,2); }
aerosol_vulcan{SPACE}+{INTEGER}     {Input.aer.vulcan = yytext2int(yytext,2,2); }
albedo{SPACE}+{FLOAT}               {Input.alb.albedo = yytext2float(yytext,2,2); 
                                     Input.alb.source = ALBEDO_CONSTANT; 
                                    }
albedo_file{SPACE}+{WORD}           {if (Input.alb.source != NOT_DEFINED_INTEGER) {
                                       fprintf (stderr, " *** Warning, 'albedo_file' on line %d\n",line_number+1);
                                       fprintf (stderr, "     another albedo was already defined in the input file!\n"); 
                                     }
                                     yytextCstring (Input.filename[FN_ALBEDO], yytext, 2, 2);
                                     Input.alb.source = ALBEDO_FROM_ALBEDO_FILE;
                                     Input.alb.surf_type_map = FALSE;
                                     Input.alb.albedo = -1.0;
                                    }
albedo_library{SPACE}+{WORD}        {if (Input.alb.source != NOT_DEFINED_INTEGER) {
                                       fprintf (stderr, " *** Warning, 'albedo_library' on line %d\n",line_number+1);
                                       fprintf (stderr, "     another albedo was already defined in the input file!\n"); 
                                     }
                                     ntokens = yytext2ntokens(yytext);
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if (strncasecmp("IGBP",s, 4)==0) {Input.alb.source = ALBEDO_IGBP_LIBRARY;}
                                     else {
                                       Input.alb.source = ALBEDO_USER_LIBRARY;
                                       yytextCstring (Input.filename[FN_ALBEDO_LIB_PATH],yytext, 1+1, ntokens+1);
                                       /* append slash */
                                       if (Input.filename[FN_ALBEDO_LIB_PATH][strlen(Input.filename[FN_ALBEDO_LIB_PATH])-1]!= '/')
				         strcat (Input.filename[FN_ALBEDO_LIB_PATH], "/");
                                     }
                                    }
albedo_map{WORDS}{SPACE_FLOAT}*    { if (Input.alb.source != NOT_DEFINED_INTEGER) {
                                       fprintf (stderr, " *** Warning, 'albedo_library' on line %d\n",line_number+1);
                                       fprintf (stderr, "     another albedo was already defined in the input file!\n"); 
                                     }
                                     ntokens = yytext2ntokens(yytext);
                                     if ( ntokens < 1 ||  3 < ntokens ) {
                                       fprintf (stderr, "Error, 'albedo map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.alb.source = ALBEDO_FROM_ALBEDO_MAP;
                                     yytextCstring (Input.filename[FN_ALBEDO_MAP], yytext, 1+1, ntokens+1);

                                     if (ntokens > 1)
                                       yytextCstring (Input.alb.netCDF_alb_name,     yytext, 2+1, ntokens+1);
                                     if (ntokens > 2)
                                       Input.alb.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
altitude{SPACE}+{SIGNED_FLOAT}{SPACE_SIGNED_FLOAT}* {
                                    s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				    strcpy (s, yytext);
                                    ntokens = yytext2ntokens(s);
                                    free(s);            
				    s =  yytext2string(yytext,2,ntokens+1);
                                    Input.alt.altitude = yytext2float(yytext,2,ntokens+1);
                                    Input.alt.source   = ALT_DIRECT_INPUT;
                                    if (ntokens == 2) {
                                      Input.alt.altitude_dz = yytext2float(yytext,3,ntokens+1); 
				    }
                                   }
altitude_map{SPACE_WORD}+{SPACE_FLOAT}*  {ntokens = yytext2ntokens(yytext);
                                    if ( ntokens < 1 ||  3 < ntokens ) {
                                      fprintf (stderr, "Error, 'altitude map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                      ierror++;
                                    }
                                    Input.alt.source = ALT_FROM_MAP; 
                                    yytextCstring (Input.filename[FN_ALTITUDE_MAP], yytext, 1+1, ntokens+1);

                                    if (ntokens > 1)
                                      yytextCstring (Input.alt.netCDF_alt_name,     yytext, 2+1, ntokens+1);
                                    if (ntokens > 2)
                                      Input.alt.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
angstrom{SIGNED_FLOATS}             {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.aer.alpha    = yytext2float(s,2,3); 
                                     Input.aer.beta     = yytext2float(yytext,3,3);
				     free(s);
                                     Input.aer.spec++;}
atmosphere_file{SPACE}+{WORD}       {yytextCstring(Input.filename[FN_ATMOSPHERE],yytext,2,2); }
brightness                          {Input.calibration = OUTCAL_BRIGHTNESS;}
ch4_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CH4]   = yytext2float(yytext,2,2); }

cloud_fraction_file{SPACE}+{WORD}   {yytextCstring(Input.filename[FN_CLOUD_FRACTION],yytext,2,2); }
cloud_fraction_map{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_CLOUD_FRACTION_MAP],yytext,2,2); }
cloud_overlap{SPACE}+{WORD}         {s = yytext2string(yytext,2,2);
                                      if      (strlen(s)==4  && strncasecmp("rand", s, 4)==0) 
				                      {Input.cloud_overlap = CLOUD_OVERLAP_RAND;}
				      else if (strlen(s)==7  && strncasecmp("maxrand", s, 7)==0) 
				                      {Input.cloud_overlap = CLOUD_OVERLAP_MAX;}
				      else  {fprintf(stderr,"Unknown 'cloud overap scheme %s' on line %d\n", s, line_number+1); ierror++;}		      
				     free(s);
				     }
co2_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_CO2]   = yytext2float(yytext,2,2); }
correlated_k{SPACE}+{WORD}          {s = yytext2string(yytext,2,2);
				     if      (strlen(s)==8 && strncasecmp("kato2.96",s,8)==0) {Input.ck_scheme = CK_KATO2_96;}
				     else if (strlen(s)==5 && strncasecmp("kato2",s,5)==0)    {Input.ck_scheme = CK_KATO2;}
				     else if (strlen(s)==4 && strncasecmp("kato",s,4)==0)     {Input.ck_scheme = CK_KATO;}
				     else if (strncasecmp("fu",s,2)==0)                       {Input.ck_scheme = CK_FU;}
				     else if (strncasecmp("avhrr_kratz",s,11)==0)             {Input.ck_scheme = CK_AVHRR_KRATZ;}
				     else if (strncasecmp("sbdart",s,6)==0 || strncasecmp("lowtran",s,7)==0) {
				                                                               Input.ck_scheme = CK_LOWTRAN;}
				     else {Input.ck_scheme = CK_GENERIC; 
				           strcpy (Input.filename[FN_CK_GENERIC], s);}
				     free(s);}
cox_and_munk                        {Input.brdf = BRDF_CM;}
cox_and_munk_u10{SPACE}+{FLOAT}     {Input.cm.u10 = yytext2float(yytext,2,2); }
cox_and_munk_u10_map{SPACE}+{WORD}  {yytextCstring (Input.filename[FN_U10_MAP], yytext, 1+1, ntokens+1); Input.brdf = BRDF_CM; }
cox_and_munk_pcl{SPACE}+{FLOAT}     {Input.cm.pcl = yytext2float(yytext,2,2); }
cox_and_munk_pcl_map{WORDS}{SPACE_FLOAT}* {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'cox_and_munk_pcl_map' on line %d requires ONE or TWO arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.brdf = BRDF_CM;
                                     yytextCstring (Input.filename[FN_PIGMENTS_MAP], yytext, 1+1, ntokens+1); 
                                     if (ntokens > 1 )
                                       yytextCstring (Input.cm.pcl_netCDF_name,      yytext, 2+1, ntokens+1);
                                     if (ntokens > 2 )
                                       Input.cm.pcl_scale_factor = yytext2float    ( yytext, 3+1, ntokens+1); 
                                    }
cox_and_munk_sal{SPACE}+{FLOAT}     {Input.cm.sal = yytext2float(yytext,2,2); }
cox_and_munk_sal_map{WORDS}{SPACE_FLOAT}* {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'cox_and_munk_sal_map' on line %d requires ONE or TWO arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.brdf = BRDF_CM;
                                     yytextCstring (Input.filename[FN_SALINITY_MAP], yytext, 1+1, ntokens+1);
                                     if (ntokens > 1 )
                                       yytextCstring (Input.cm.sal_netCDF_name,      yytext, 2+1, ntokens+1);
                                     if (ntokens > 2 )
                                       Input.cm.sal_scale_factor = yytext2float    ( yytext, 3+1, ntokens+1); 
                                    }
crs_file{SPACE}+{WORD}{SPACE}+{WORD} {
                                     s = yytext2string(yytext,2,4);
				     if      (strncasecmp("O3",s,2)==0)   {yytextCstring(Output.crs.filename[MOL_O3],yytext,3,4);}
				     else if (strncasecmp("O2",s,2)==0)   {yytextCstring(Output.crs.filename[MOL_O2],yytext,3,4);}
				     else if (strncasecmp("H2O",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_H2O],yytext,3,4);}
				     else if (strncasecmp("CO2",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_CO2],yytext,3,4);}
				     else if (strncasecmp("NO2",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_NO2],yytext,3,4);}
				     else if (strncasecmp("BRO",s,3)==0)  {yytextCstring(Output.crs.filename[MOL_BRO],yytext,3,4);}
				     else if (strncasecmp("OCLO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_OCLO],yytext,3,4);}
				     else if (strncasecmp("HCHO",s,4)==0) {yytextCstring(Output.crs.filename[MOL_HCHO],yytext,3,4);}
                                     else {fprintf(stderr,"Unknown 'crs_file %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
                                     }
data_files_path{SPACE}+{WORD}       {yytextCstring (Input.filename[FN_PATH],
						    yytext, 2, 2); 
                                     /* append slash */
                                     if (Input.filename[FN_PATH][strlen(Input.filename[FN_PATH])-1]!= '/')
				       strcat (Input.filename[FN_PATH], "/");}
day_of_year{SPACE}+{INTEGER}        {Input.UTC.tm_yday  = yytext2int(yytext,2,2); }
deltam{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("on",s,2)==0) {Input.rte.deltam = 1;}
				     else if (strncasecmp(s,"off",3)==0) {Input.rte.deltam = 0;}
                                     else {fprintf(stderr,"Unknown 'deltam %s' on line %d\n", s, line_number+1); ierror++;}
				     free(s);}
dens_column{SPACE}+{WORD}{SPACE}+{FLOAT}{SPACE_WORD}? {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
				     s =  yytext2string(yytext,2,ntokens+1);
				     if      (strncasecmp("O3",s,2)==0)   {mol_id = MOL_O3;  }
                                     else if (strncasecmp("O2",s,2)==0)   {mol_id = MOL_O2;  }
                                     else if (strncasecmp("H2O",s,3)==0)  {mol_id = MOL_H2O; }
                                     else if (strncasecmp("CO2",s,3)==0)  {mol_id = MOL_CO2; }
                                     else if (strncasecmp("NO2",s,3)==0)  {mol_id = MOL_NO2; }
                                     else if (strncasecmp("BRO",s,3)==0)  {mol_id = MOL_BRO; }
                                     else if (strncasecmp("OCLO",s,4)==0) {mol_id = MOL_OCLO;}
                                     else if (strncasecmp("HCHO",s,4)==0) {mol_id = MOL_HCHO;}
                                     else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
                                     Input.atm.column[mol_id] = yytext2float(yytext, 3, ntokens+1);
				     if (ntokens == 3) {
   				        s =  yytext2string(yytext,4,ntokens+1);
				        if      (strncasecmp("DU"  ,s,2)==0)                               {Input.atm.unit_column[mol_id]= DU; }
				        else if (strncasecmp("CM_2",s,4)==0 || strncasecmp("CM-2",s,4)==0) {Input.atm.unit_column[mol_id]= CM_2; }
                                        else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                        free(s);
                                     }
                                     }
dens_file{SPACE}+{WORD}{SPACE}+{WORD}{SPACE_WORD}? { s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
                                     fflush (stderr);
                                     s = yytext2string(yytext,2,ntokens+1);
				     if      (strncasecmp("O3"  ,s,2)==0) {mol_id = MOL_O3;}
				     else if (strncasecmp("O2"  ,s,2)==0) {mol_id = MOL_O2;}
				     else if (strncasecmp("H2O" ,s,3)==0) {mol_id = MOL_H2O;}
				     else if (strncasecmp("CO2" ,s,3)==0) {mol_id = MOL_CO2;}
				     else if (strncasecmp("NO2" ,s,3)==0) {mol_id = MOL_NO2;}
				     else if (strncasecmp("BRO" ,s,3)==0) {mol_id = MOL_BRO;}
				     else if (strncasecmp("OCLO",s,4)==0) {mol_id = MOL_OCLO;}
				     else if (strncasecmp("HCHO",s,4)==0) {mol_id = MOL_HCHO;}
                                     else {fprintf(stderr,"Unknown 'dens_file %s' on line %d\n", s, line_number+1); ierror++;}
                                     free(s);
				     if (ntokens == 3) {
   				        s =  yytext2string(yytext,3,ntokens+1);
				        if      (strncasecmp("CM_3",s,4)==0 || strncasecmp("CM-3",s,4)==0) {Input.atm.unit_profile[mol_id]= CM_3; }   
				        else if (strncasecmp("M_3" ,s,3)==0 || strncasecmp("M-3" ,s,3)==0) {Input.atm.unit_profile[mol_id]= M_3; }
				        else if (strncasecmp("MMR" ,s,3)==0)                               {Input.atm.unit_profile[mol_id]= MMR; }
				        else if (strncasecmp("VMR" ,s,3)==0)                               {Input.atm.unit_profile[mol_id]= VMR; }
                                        else if (strncasecmp("RH"  ,s,3)==0) {
                                          if (mol_id != MOL_H2O) {
                                            fprintf(stderr, "Error, unit \"relative humidity\" only possible for H2O dens_files.\n");
                                            fprintf(stderr, "See 'dens_column ... %s' in line %d\n", s, line_number+1);
                                            ierror++;
                                          }
                                          else 
                                            Input.atm.unit_profile[mol_id]= RH; 
                                        }
                                        else {fprintf(stderr,"Unknown 'dens_column %s' on line %d\n", s, line_number+1); ierror++;}
                                        free(s);
                                     }
                                     if (ntokens == 2)
                                       position = 3;
                                     else if (ntokens == 3)
                                       position = 4;
                                     yytextCstring(Input.atm.filename[mol_id],  yytext, position, ntokens+1);
                                     }
ECHAM_atmosphere_file{SPACE}+{WORD} {Input.atm.rs_source=RS_FROM_ECHAM;
                                      yytextCstring(Input.filename[FN_ECHAM],yytext,2,2);
                                      Input.atm.n_rs_gas=0;
                                    Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O; Input.atm.n_rs_gas++; mol_id = MOL_H2O;
                                      Input.atm.rs_unit[mol_id] = MMR;
                                     }

ECHAM_clouds_file{SPACE}+{WORD}      {Input.wc.source=CLOUDS_FROM_ECHAM;
                                      Input.ic.source=CLOUDS_FROM_ECHAM;
				      Input.wc.layer=1;
				      Input.ic.layer=1;
                                      yytextCstring(Input.filename[FN_WCLOUD],
						    yytext,2,2);
				      yytextCstring(Input.filename[FN_ICLOUD],
						    yytext,2,2);
				      }

ECHAM_sza{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_SZA],yytext,2,2);
                                      Input.atm.sza = NOT_DEFINED_FLOAT;
                                      Input.atm.sza_source = SZA_ECHAM;}

ECMWF_atmosphere_file{SPACE}+{WORD} {Input.atm.rs_source=RS_FROM_ECMWF;
                                     yytextCstring(Input.filename[FN_ECMWF],yytext,2,2);
                                     Input.atm.n_rs_gas=0;
                                     Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O3;  Input.atm.n_rs_gas++; mol_id = MOL_O3;
                                     Input.atm.rs_unit[mol_id] = MMR;
                                     Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O; Input.atm.n_rs_gas++; mol_id = MOL_H2O;
                                     Input.atm.rs_unit[mol_id] = MMR;
                                    }
ECMWF_wc_file{SPACE}+{WORD}         {Input.wc.source=CLOUDS_FROM_ECMWF;
                                     yytextCstring(Input.filename[FN_WCLOUD],yytext,2,2);
                                    }
ECMWF_ic_file{SPACE}+{WORD}         {Input.ic.source=CLOUDS_FROM_ECMWF;
                                     yytextCstring(Input.filename[FN_ICLOUD],yytext,2,2);
                                    }
ECMWF_wind_file{WORDS}               {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 2) {
                                       fprintf (stderr, "Error, to many arguments for option 'ECMWF_wind map' on line %d \n",line_number+1); 
                                       ierror++;
                                     }
                                     /* Input.wind.source = WIND_FROM_WIND_MAP; */
                                     yytextCstring (Input.filename[FN_ECMWF_WIND_MAP], yytext, 1+1, ntokens+1);
                                    }
emissivity_map{WORDS}{SPACE_FLOAT}* {ntokens = yytext2ntokens(yytext);
                                     if (ntokens > 3) {
                                       fprintf (stderr, "Error, 'emissivity_map' on line %d requires ONE or TWO optional arguments,\n",line_number+1); 
                                       ierror++;
                                     }
                                     Input.alb.source = ALBEDO_FROM_EMISSIVITY_MAP;
                                     yytextCstring (Input.filename[FN_ALBEDO_MAP], yytext, 1+1, ntokens+1);

                                     if (ntokens > 1)
                                       yytextCstring (Input.alb.netCDF_alb_name,     yytext, 2+1, ntokens+1);
                                     if (ntokens > 2)
                                       Input.alb.scale_factor  =   yytext2float(     yytext, 3+1, ntokens+1); 
                                   }
f11_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F11]   = yytext2float(yytext,2,2); }
f12_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F12]   = yytext2float(yytext,2,2); }
f22_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_F22]   = yytext2float(yytext,2,2); }
filter_function_file{SPACE}+{WORD}  {yytextCstring(Input.filename[FN_FILTERFUNCTION],yytext,2,2); }
fisot                               {Input.rte.fisot = 1; }
flexstor                            {Input.output_format  = OUTPUT_FORMAT_FLEXSTOR; }
fu_h2o_continuum{FLOAT_OR_WORD}     {s = yytext2string(yytext,2,2);
                                     if      (strncasecmp(s,"off",3)==0)  Input.ck_h2ocont = 0;
				     else if (strncasecmp(s,"on", 2)==0)  Input.ck_h2ocont = 1;
				     else { 
                                       tmpfloat = yytext2float(yytext,2,2);
                                       fprintf(stderr,"%f %f\n", fabs(tmpfloat-2.1), fabs(tmpfloat-2.4));
                                       if      (fabs(tmpfloat-2.1) < epsilon)  Input.ck_h2ocont = 1;
				       else if (fabs(tmpfloat-2.4) < epsilon)  Input.ck_h2ocont = 2;
                                       else {
                                         fprintf(stderr,"Unknown 'h2o_continuum %s' on line %d\n", s, line_number+1); 
                                         ierror++;
                                       }
                                     }
                                     fprintf(stderr,"Input.ck_h2ocont = %d\n", Input.ck_h2ocont);
				     free(s);}
h2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_H2O]   = yytext2float(yytext,2,2); }
h2o_precip{SPACE}+{FLOAT}           {Input.precip_water   = yytext2float(yytext,2,2); }
heating_rate{WORDS}                 {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     token_number = yytext2ntokens(s); 
				     free(s);
                                     if (token_number == 0)
                                       Input.heating = HEAT_LAYER_CD;
                                     else if (token_number == 1) {
                                       t = yytext2string(yytext, 2,token_number+1);
				       if      (strncasecmp("none",t,4)==0)           Input.heating = NO_HEAT;
				       else if (strncasecmp("local",t,5)==0)          Input.heating = HEAT_LOCAL;
				       else if (strncasecmp("layer_fd",t,8)==0)       Input.heating = HEAT_LAYER_FD;
				       else if (strncasecmp("layer_cd",t,8)==0)       Input.heating = HEAT_LAYER_CD;
                                       else {fprintf(stderr,"Unknown 'heating %s' on line %d\n", s, line_number+1);
				             ierror++;}
                                     }
                                     else{
                                       fprintf(stderr,"Wrong number of arguments 'heating %s' on line %d\n", s, line_number+1);
				       ierror++;
                                     }
				    }
header                              {Input.header    = 1; }
ichap{SPACE}+{INTEGER}              {Input.rte.ichap = yytext2int(yytext,2,2);}
ic_cloudcover{SPACE}+{FLOAT}        {Input.ic_cloudcover= yytext2float(yytext,2,2); 
                                     Input.icipa=1;}
ic_file{SPACE}+{WORD}               {Input.ic.source=CLOUDS_FROM_FILE;
                                     yytextCstring (Input.filename[FN_ICLOUD], yytext, 2, 2);
                                    }
ic_files{SPACE}+{WORD}              {Input.ic.source=CLOUDS_LAYER_FILES;
                                     yytextCstring (Input.filename[FN_ICLOUD_FILES], yytext, 2, 2);
                                     Input.ic.properties = ICPROP_EXPLICIT;
                                    }
ic_fu_tau{SPACE}+{WORD}             {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("unscaled",s,8)==0)  Input.ic.unscaled = 1;
                                     else {fprintf(stderr,"Unknown 'ic_fu_tau %s' on line %d\n", s, line_number+1); ierror++;}
				    }
ic_fu_reff{SPACE}+{WORD}            {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("yang",s,4)==0) Input.ic.fu2yang = 1;
                                     else {fprintf(stderr,"Unknown 'ic_fu_reff %s' on line %d\n", s, line_number+1); ierror++;}
				    }
ic_habit{SPACE}+{WORD}              {Input.ic.habit = -999;
                                     s =  yytext2string(yytext,2,2);       /*agf*/
                                     if      (strncasecmp("solid-column",s,12)==0)    Input.ic.habit = IC_HABIT_SOLIDCOLUMN;
				     else if (strncasecmp("hollow-column",s,13)==0)   Input.ic.habit = IC_HABIT_HOLLOWCOLUMN;
				     else if (strncasecmp("rough-aggregate",s,15)==0) Input.ic.habit = IC_HABIT_AGGREGATE;
				     else if (strncasecmp("rosette-4",s,9)==0)        Input.ic.habit = IC_HABIT_ROSETTE4;
				     else if (strncasecmp("rosette-6",s,9)==0)        Input.ic.habit = IC_HABIT_ROSETTE6;
				     else if (strncasecmp("plate",s,5)==0)            Input.ic.habit = IC_HABIT_PLATE;
				     else {fprintf(stderr,"Unknown 'ic_habit %s' on line %d\n", s, line_number+1); ierror++;}
				    }
ic_ipa_files{SPACE}+{WORD}          {Input.ic.source=CLOUDS_IPA_FILES;
                                     Input.icipa=1;
                                     yytextCstring (Input.filename[FN_ICLOUD_IPA], yytext, 2, 2);
                                     Input.ic.properties = ICPROP_FU;}
ic_layer                            {Input.ic.layer=1;
                                    }
ic_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if      (strncasecmp("fu",s,2)==0)             Input.ic.properties = ICPROP_FU;
				     else if (strncasecmp("echam4",s,6)==0)         Input.ic.properties = ICPROP_ECHAM4;
				     else if (strncasecmp("mie",s,3)==0)            Input.ic.properties = ICPROP_MIE;
				     else if (strncasecmp("yang",s,4)==0)           Input.ic.properties = ICPROP_YANG;
				     else if (strncasecmp("key",s,3)==0)            Input.ic.properties = ICPROP_KEY;
				     else if (strncasecmp("baum_detailed",s,13)==0) Input.ic.properties = ICPROP_BAUMDETAILED;
				     else if (strncasecmp("baum",s,4)==0)           Input.ic.properties = ICPROP_BAUM;
				     else  { Input.ic.properties =                  ICPROP_FILE;
					     strcpy (Input.filename[FN_ICLOUD_PROP], s); }
                                    }
ic_properties_interpolate           {Input.ic.interpolate=1; }
ic_saturate{SPACE}+{WORD}+{SPACE_FLOAT}* { ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || 2 < ntokens ) {
                                       fprintf(stderr,"Wrong number of arguments for 'ic_saturate' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     s = yytext2string(yytext,2,ntokens+1);
				     if      (strcmp("off",s) == 0 || strcmp("OFF",s) == 0) {Input.ic.wv_saturation = SATURATION_NONE;}
                                     else if (strcmp("on" ,s) == 0 || strcmp("ON" ,s) == 0) {Input.ic.wv_saturation = SATURATION_CLOUD;}
                                     else if (strcmp("ipa",s) == 0 || strcmp("IPA",s) == 0) {Input.ic.wv_saturation = SATURATION_IPA;}
                                     else {
                                       fprintf(stderr,"Unknown 'saturate %s' on line %d\n", s, line_number+1); 
                                       ierror++;
                                     }
                                     free(s);
                                     if (ntokens == 2) 
                                       Input.ic.wv_sat_ratio = yytext2float(yytext,3,ntokens+1);
                                    }
ic_scale_gg{SPACE}+{FLOAT}          {Input.ic.scale_gg   = yytext2float(yytext,2,2); }
ic_scale_ssa{SPACE}+{FLOAT}         {Input.ic.scale_ssa  = yytext2float(yytext,2,2); }
ic_set_gg{SPACE}+{SIGNED_FLOAT}     {Input.ic.gg     = yytext2float(yytext,2,2); }
ic_set_ssa{SPACE}+{FLOAT}           {Input.ic.ssa    = yytext2float(yytext,2,2); }
ic_set_tau{SPACE}+{FLOAT}           {Input.ic.tau    = yytext2float(yytext,2,2); }
ic_set_tau550{SPACE}+{FLOAT}        {Input.ic.tau550 = yytext2float(yytext,2,2); }                       
include                             BEGIN(include);
latitude{SIGNED_FLOATS_OR_WORDS}    {ntokens = yytext2ntokens(yytext);                                     
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if      (strcmp("N",s) == 0 || strcmp("n",s) == 0) tmpint = 1;
                                     else if (strcmp("S",s) == 0 || strcmp("s",s) == 0) tmpint =-1;
                                     else                                               tmpint = 0;
                                     tmpfloat = yytext2float (yytext,1+abs(tmpint)+1,ntokens+1);
                                     Input.latitude = tmpfloat;               /*       already  in degrees */
                                     if (ntokens > 1+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,2+abs(tmpint)+1,ntokens+1);
                                       Input.latitude += tmpfloat / 60.0;     /*   60.0 minutes to degrees */
                                     }
                                     if (ntokens > 2+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,3+abs(tmpint)+1,ntokens+1);
                                       Input.latitude += tmpfloat / 3600.0;   /* 3600.0 seconds to degrees */
                                     }
                                     if (tmpint != 0) Input.latitude *= tmpint;
                                    }
longitude{SIGNED_FLOATS_OR_WORDS}   {ntokens = yytext2ntokens(yytext);                                     
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if      (strcmp("E",s) == 0 || strcmp("e",s) == 0) tmpint = 1;
                                     else if (strcmp("W",s) == 0 || strcmp("w",s) == 0) tmpint =-1;
                                     else                                               tmpint = 0;
                                     tmpfloat = yytext2float (yytext,1+abs(tmpint)+1,ntokens+1);
                                     Input.longitude = tmpfloat;              /*        already in degrees */
                                     if (ntokens > 1+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,2+abs(tmpint)+1,ntokens+1);
                                       Input.longitude += tmpfloat / 60.0;    /*   60.0 minutes to degrees */
                                     }
                                     if (ntokens > 2+abs(tmpint)) {
                                       tmpfloat = yytext2float(yytext,3+abs(tmpint)+1,ntokens+1);
                                       Input.longitude += tmpfloat / 3600.0;  /* 3600.0 seconds to degrees */
                                     }
                                     if (tmpint != 0) Input.longitude *= tmpint;
                                    }
mc_absorption                       {Input.rte.mc.absorption = MCABS_ABSORPTION; }
mc_actinic                          {Input.rte.mc.absorption = MCABS_ACTINIC; }
mc_albedo_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_ALBEDO], s);}
mc_ambrals_file{SPACE}+{WORD}       {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_AMBRALS], s);}
mc_azimuth_old                      {Input.rte.mc.azimuth = MCAZIMUTH_OLD; }
mc_backward{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_INTEGER}?{SPACE_INTEGER}? {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     Input.rte.mc.backward.yes = 1;
                                     if (ntokens != 2 && ntokens != 4) {
				       fprintf (stderr, "Error, need either 2 or 4 integers as arguments for mc_backward!\n");
				       exit (1);
                                     }
				     if (ntokens == 2) {
                                       tmpi   = yytext2integers(yytext, 2); 
                                       Input.rte.mc.backward.islower = tmpi[0];
                                       Input.rte.mc.backward.jslower = tmpi[1];
                                       Input.rte.mc.backward.isupper = Input.rte.mc.backward.islower;
                                       Input.rte.mc.backward.jsupper = Input.rte.mc.backward.jslower;
                                     }
                                     if (ntokens == 4) {
                                       tmpi   = yytext2integers(yytext, 4); 
                                       Input.rte.mc.backward.islower = tmpi[0];
                                       Input.rte.mc.backward.jslower = tmpi[1];
                                       Input.rte.mc.backward.isupper = tmpi[2];
                                       Input.rte.mc.backward.jsupper = tmpi[3];
                                     }
				     free(tmpi); free(s);}
mc_backward_writeallpixels          {Input.rte.mc.backward.writeallpixels=1;}

mc_backward_output{SPACE}+{WORD}     {s =  yytext2string(yytext,2,2);
                                      if(strncasecmp("edir",s,4)==0)
				      Input.rte.mc.backward.output = MCBACKWARD_EDIR;
				      else if(strncasecmp("edn",s,3)==0)
				      Input.rte.mc.backward.output = MCBACKWARD_EDN;
				      else if(strncasecmp("eup",s,3)==0)
				      Input.rte.mc.backward.output = MCBACKWARD_EUP;
				      else if(strncasecmp("f",s,1)==0)
				      Input.rte.mc.backward.output = MCBACKWARD_ACT;
				      else if(strncasecmp("heating",s,7)==0)
				      Input.rte.mc.backward.output = MCBACKWARD_ABS;
					}
					
mc_basename{SPACE}+{WORD}           {yytextCstring (Input.rte.mc.filename[FN_MC_BASENAME], 
                                                    yytext, 2, 2);}
mc_bcond{SPACE}+{WORD}              {s =  yytext2string(yytext,2,2);
				     if      (strncasecmp("periodic",s,8)==0) Input.rte.mc.bcond = MCBCOND_PERIODIC;
				     else if (strncasecmp("mirror",s,6)==0)   Input.rte.mc.bcond = MCBCOND_MIRROR;
				     else if ((strncasecmp("absorb",s,6)==0) || (strncasecmp("absorp",s,6)==0))   Input.rte.mc.bcond = MCBCOND_ABSORB;
                                     else {fprintf(stderr,"Unknown 'mc_bcond %s' on line %d\n", s, line_number+1); ierror++;}
				    }
mc_cloud_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_INTEGER} {
                                     fprintf (stderr, " ... option mc_cloud_grid is obsolete!\n");}
mc_elevation_file{SPACE}+{WORD}     {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_ELEVATION], s);}
mc_emission                         {Input.rte.mc.absorption = MCABS_EMISSION; }
mc_escape                           {Input.rte.mc.escape = 1; }
mc_heating                          {Input.rte.mc.absorption = MCABS_HEATING; }
mc_ipa                              {Input.rte.mc.ipa = 1; }
mc_hiddenline                       {
                                     #if HAVE_OPENGL
                                       GLmystic_hiddenline = 1;
                                     #endif
                                    }
mc_momcld_file{SPACE}+{WORD}        {fprintf (stderr, "Error, option mc_momcld_file does not exist anymore.\n");
                                     fprintf (stderr, "Use wc_properties instead!\n"); exit(-1);}
mc_radial_pathlength{SPACE_INTEGER}{SPACE_INTEGER} {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     tmpi   = yytext2integers(yytext, 2); 
                                     Input.rte.mc.Nr = tmpi[0];
				     Input.rte.mc.Nt = tmpi[1];
				     free(tmpi); free(s); }
mc_radial_pathlength_dt{SPACE}+{SIGNED_FLOAT}   {Input.rte.mc.dt = yytext2float(yytext,2,2); }
mc_pcld_file{SPACE}+{WORD}          {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_PCLD], s);
                                     Input.wc.properties=WCPROP_PHASE;
				     }
mc_photons{SPACE}+{INTEGER}         {Input.rte.mc.photons  = yytext2int(yytext,2,2); }
mc_photons_file{SPACE}+{WORD}       {yytextCstring(Input.rte.mc.filename[FN_MC_PHOTONS],yytext,2,2); }
mc_rpv_file{SPACE}+{WORD}           {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_RPV], s);}
mc_rpv_type{SPACE}+{WORD}           {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_RPVTYPE], s);}
mc_sample_grid{SPACE_INTEGER}{SPACE_INTEGER}{SPACE_FLOAT}*{SPACE_FLOAT}* {
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
				     if (ntokens !=2 && ntokens != 4) {
                                       fprintf(stderr,"Wrong number of arguments for 'mc_sample_grid' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     Input.rte.mc.Nx_sample = yytext2int(yytext,2,ntokens+1);
				     Input.rte.mc.Ny_sample = yytext2int(yytext,3,ntokens+1);
                                     if (ntokens == 4) {
					Input.rte.mc.dx_sample = yytext2double(yytext,4,ntokens+1);
					Input.rte.mc.dy_sample = yytext2double(yytext,5,ntokens+1);
				     }
				     free(s); }
mc_randomseed{SPACE}+{INTEGER}      {Input.rte.mc.randomseed = yytext2int(yytext,2,2); }
mc_spherical                        {Input.rte.mc.spherical=1;}
mc_std                              {Input.rte.mc.std=1;}
mc_surface_reflectalways            {Input.rte.mc.reflectalways = 1;}
mc_surfaceparallel                  {Input.rte.mc.surfaceparallel = 1;}
mc_truncate                         {Input.rte.mc.truncate=0.99;}
mc_truncate{SPACE}+{SIGNED_FLOAT}   {Input.rte.mc.truncate = yytext2float(yytext,2,2); }
mc_wcloud_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_WCLOUD], s);}
mc_icloud_file{SPACE}+{WORD}        {s = yytext2string(yytext,2,2);
				     strcpy (Input.rte.mc.filename[FN_MC_ICLOUD], s);}
molecular_tau_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_MOLABS],yytext,2,2);}
n2o_mixing_ratio{SPACE}+{FLOAT}     {Input.mixing_ratio[MX_N2O]   = yytext2float(yytext,2,2); }
no_absorption                       {Input.absorption=0; Input.molabs=MOLABS_NONE;}
no_molecular_absorption             {Input.molabs=MOLABS_NONE;}
no_rayleigh                         {Input.rayleigh=RAYLEIGH_NONE;}
no_scattering                       {Input.scattering=0;Input.rayleigh=RAYLEIGH_NONE;}
no2_column_du{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2); 
                                     Input.atm.unit_column[MOL_NO2] = DU;}
no2_column_moleccm-2{SPACE}+{SIGNED_FLOAT} {Input.atm.column[MOL_NO2] = yytext2float(yytext,2,2);
                                     Input.atm.unit_column[MOL_NO2] = CM_2;}
nrefrac{SPACE}+{INTEGER}            {Input.rte.nrefrac  = yytext2int(yytext,2,2);}
nstr{SPACE}+{INTEGER}               {Input.rte.nstr  = yytext2int(yytext,2,2);}
nscat{SPACE}+{INTEGER}              {Input.rte.nscat  = yytext2int(yytext,2,2); }
o2_mixing_ratio{SPACE}+{FLOAT}      {Input.mixing_ratio[MX_O2]   = yytext2float(yytext,2,2); }
o3_crs{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
				     if (strncasecmp("Bass_and_Paur",s,13)==0) {Input.ozone_spectrum = O3_BASSPAUR;}
				     else if (strncasecmp(s,"Molina",6)==0)    {Input.ozone_spectrum = O3_MOLINA;}
				     else if (strncasecmp(s,"Daumont",7)==0)   {Input.ozone_spectrum = O3_DAUMONT;}
				     else {fprintf(stderr,"Unknown 'o3_crs %s' on line %d\n", s, line_number+1);
				     ierror++;}
				     free(s);}
ozone_column{SPACE}+{SIGNED_FLOAT}  {Input.atm.column[MOL_O3] = yytext2float(yytext,2,2); 
                                     Input.atm.unit_column[MOL_O3] = DU;}
output{WORDS}                       {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     token_number = yytext2ntokens(s); 
				     free(s);
                                     t = yytext2string(yytext, 2,token_number+1);
				     if      (strncasecmp("integrate",t,9) == 0) Input.processing = PROCESS_INT;
				     else if (strncasecmp("sum",t,3)       == 0) Input.processing = PROCESS_SUM;
				     else if (strncasecmp("rgbnorm",t,7)   == 0) Input.processing = PROCESS_RGBNORM;
				     else if (strncasecmp("rgb_norm",t,8)  == 0) Input.processing = PROCESS_RGBNORM;
				     else if (strncasecmp("rgb",t,3)       == 0) Input.processing = PROCESS_RGB;
                                     else if (strncasecmp("per_nm",t,6)    == 0)    Input.output_unit= UNIT_PER_NM;
                                     else if (strncasecmp("per_cm-1",t,8)  == 0)    Input.output_unit= UNIT_PER_CMm1;
                                     else if (strncasecmp("per_ck_band",t,11) == 0) Input.output_unit= UNIT_PER_CK_BAND;
                                     else {fprintf(stderr,"Unknown 'output %s' on line %d\n", s, line_number+1);
				           ierror++;}
				    }
output_format{WORDS}                {token_number = yytext2ntokens(yytext);
                                     if (token_number == 1 || token_number == 2) {
                                       t = yytext2string(yytext, 1+1, token_number+1); 
                                       if      (strcmp("ascii"   ,t)==0) {Input.output_format  = OUTPUT_FORMAT_ASCII;   }
                                       else if (strcmp("flexstor",t)==0) {Input.output_format  = OUTPUT_FORMAT_FLEXSTOR; }
                                       else if (strcmp("netCDF"  ,t)==0) {
                                         Input.output_format  = OUTPUT_FORMAT_NETCDF;
                                         if (token_number == 2)
                                           yytextCstring(Input.filename[FN_NETCDF_OUTFILE],yytext,2+1,token_number+1);
                                         else {
                                           fprintf(stderr,"Wrong number of arguments for 'output_format' on line %d\n", line_number+1); 
                                           ierror++;
                                         }
                                       }
                                       else if (strcmp("sat_picture",t)==0) {
                                         Input.output_format  = OUTPUT_FORMAT_SATPIC;
                                         if (token_number == 2)
                                           yytextCstring(Input.filename[FN_NETCDF_OUTFILE],yytext,2+1,token_number+1);
                                         else {
                                           fprintf(stderr,"Wrong number of arguments for 'output_format' on line %d\n", line_number+1); 
                                           ierror++;
                                         }
                                       }
                                       else {
                                         fprintf(stderr,"Wrong number of arguments for 'output_format' on line %d\n", line_number+1); 
                                         ierror++;
                                       }
                                     }
                                    }
output_user{WORDS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
                                     Input.n_output_user = yytext2ntokens(s); 
				     free(s);
				     Input.output_user     = calloc (Input.n_output_user, sizeof (int));
				     Input.output_user_gas = calloc (Input.n_output_user, sizeof (int));
				     for (i=2;i<=Input.n_output_user+1; i++) {
				       t = yytext2string(yytext, i, Input.n_output_user+1);  
				       if      (strcmp("lambda",t)==0 ||  
                                                strcmp("wavelength",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE;}
				       else if (strcmp("lambda_min",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE_MIN;}
				       else if (strcmp("lambda_max",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVE_MAX;}
				       else if (strcmp("wavenumber",t)==0)    {Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER;}
				       else if (strcmp("wavenumber_min",t)==0){Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER_MIN;}
				       else if (strcmp("wavenumber_max",t)==0){Input.output_user[i-2] = OUTPUT_USER_WAVENUMBER_MAX;}
				       else if (strcmp("zout",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SUR;}
				       else if (strcmp("zout_sur",t)==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SUR;}
				       else if (strcmp("zout_sea",t)==0)      {Input.output_user[i-2] = OUTPUT_USER_ZOUT_SEA;}
				       else if (strcmp("p",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_P;}
				       else if (strcmp("T",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_T;}
				       else if (strcmp("T_d",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_T_D;}
				       else if (strcmp("theta",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_THETA;}
				       else if (strcmp("theta_e",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_THETA_E;}
				       else if (strcmp("edir",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_EDIR;}
				       else if (strcmp("eglo",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_EGLO;}
				       else if (strcmp("edn",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_EDN;}
				       else if (strcmp("eup",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_EUP;}
				       else if (strcmp("enet",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ENET;}
				       else if (strcmp("esum",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_ESUM;}
				       else if (strcmp("fdir",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_FDIR;}
				       else if (strcmp("fglo",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_FGLO;}
				       else if (strcmp("fdn",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_FDN;}
				       else if (strcmp("fup",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_FUP;}
				       else if (strcmp("f",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_F;}
				       else if (strcmp("uavgdir",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_UDIR;}
				       else if (strcmp("uavgglo",t) ==0)      {Input.output_user[i-2] = OUTPUT_USER_UGLO;}
				       else if (strcmp("uavgdn",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_UDN;}
				       else if (strcmp("uavgup",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_UUP;}
				       else if (strcmp("uavg",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_U;}
				       else if (strcmp("albedo",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_ALB;}
				       else if (strcmp("sza",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_SZA;}
				       else if (strcmp("c_p",t)     ==0)      {Input.output_user[i-2] = OUTPUT_USER_C_P;}
				       else if (strcmp("heat",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_HEAT;}
				       else if (strcmp("w_rad",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_W_RAD;}
				       else if (strcmp("m_rad",t)   ==0)      {Input.output_user[i-2] = OUTPUT_USER_M_RAD;}
                                       else if (strncasecmp("n_",  t,2)==0)   {Input.output_user[i-2] = OUTPUT_USER_N;}
                                       else if (strncasecmp("rho_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_RHO;}
                                       else if (strncasecmp("vmr_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_VMR;}
                                       else if (strncasecmp("mmr_",t,4)==0)   {Input.output_user[i-2] = OUTPUT_USER_MMR;}
                                       else if (strcmp("rh",t)      ==0)      {Input.output_user[i-2] = OUTPUT_USER_RH;}
                                       else if (strcmp("rh_ice",t)  ==0)      {Input.output_user[i-2] = OUTPUT_USER_RH_ICE;}
                                       else if (strcmp("clwc",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_CLWC;}
                                       else if (strcmp("ciwc",t)    ==0)      {Input.output_user[i-2] = OUTPUT_USER_CIWC;}
                                       else if (strcmp("u",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_WIND_U;}
                                       else if (strcmp("v",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_WIND_V;}
                                       else if (strcmp("w",t)       ==0)      {Input.output_user[i-2] = OUTPUT_USER_WIND_W;}
                                       else if (strcmp("dTdx",t)      ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDX;}
                                       else if (strcmp("dTdy",t)      ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDY;}
                                       else if (strcmp("dTdz",t)      ==0)    {Input.output_user[i-2] = OUTPUT_USER_DTDZ;}
                                       else if (strcmp("heat_ad_x",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_X;}
                                       else if (strcmp("heat_ad_y",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_Y;}
                                       else if (strcmp("heat_ad_z",t) ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD_Z;}
                                       else if (strcmp("heat_ad",t)   ==0)    {Input.output_user[i-2] = OUTPUT_USER_HEAT_AD;}
				       else {fprintf(stderr,"Unknown 'output_user %s' on line %d\n", t, line_number+1);
				       ierror++;}
                                       if (Input.output_user[i-2] == OUTPUT_USER_N || 
                                           Input.output_user[i-2] == OUTPUT_USER_RHO || 
                                           Input.output_user[i-2] == OUTPUT_USER_VMR || 
                                           Input.output_user[i-2] == OUTPUT_USER_MMR) {
                                         if      ((s = strstr(t,"air")) != NULL || (s = strstr(t,"AIR")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_AIR;
                                         else if ((s = strstr(t,"o3"))  != NULL || (s = strstr(t,"O3"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_O3;
                                         else if ((s = strstr(t,"h2o")) != NULL || (s = strstr(t,"H2O")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_H2O;
                                         else if ((s = strstr(t,"co2")) != NULL || (s = strstr(t,"CO2")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_CO2;
                                         else if ((s = strstr(t,"no2")) != NULL || (s = strstr(t,"NO2")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_NO2;
                                         else if ((s = strstr(t,"bro")) != NULL || (s = strstr(t,"BRO")) != NULL)
                                            Input.output_user_gas[i-2] = MOL_BRO;
                                         else if ((s = strstr(t,"oclo"))!= NULL || (s = strstr(t,"OCLO"))!= NULL)
                                            Input.output_user_gas[i-2] = MOL_OCLO;
                                         else if ((s = strstr(t,"hcho"))!= NULL || (s = strstr(t,"HCHO"))!= NULL)
                                            Input.output_user_gas[i-2] = MOL_HCHO;
                                         else if ((s = strstr(t,"o2"))  != NULL || (s = strstr(t,"O2"))  != NULL)
                                            Input.output_user_gas[i-2] = MOL_O2;
                                         else {
                                           fprintf(stderr,"Unknown 'output_user %s' on line %d\n", t, line_number+1);
				           ierror++;
                                         }
                                       }                                         
				     }
                                    }

phi0{SPACE}+{SIGNED_FLOAT}           {Input.atm.phi0     = yytext2float(yytext,2,2); }
polradtran_aziorder{SPACE}+{INTEGER} {Input.rte.pol_aziorder  = yytext2int(yytext,2,2); }
polradtran_max_delta_tau{SPACE}+{SIGNED_FLOAT} {Input.rte.pol_max_delta_tau = (double) yytext2float(yytext,2,2); }
polradtran_nstokes{SPACE}+{INTEGER}  {Input.rte.pol_nstokes  = yytext2int(yytext,2,2); }
polradtran_quad_type{SPACE}+{WORD}   {Input.rte.pol_quad_type
				       = yytext2string (yytext, 2, 2);}
polradtran_src_code{SPACE}+{INTEGER} {Input.rte.pol_src_code = yytext2int(yytext,2,2); }
pressure{SPACE}+{SIGNED_FLOAT}       {Input.pressure = yytext2float(yytext,2,2); }
pressure_out{SIGNED_FLOATS_OR_WORDS}|p_out{SIGNED_FLOATS_OR_WORDS} {
                                      Input.atm.nzout = yytext2ntokens(yytext);
                                      Input.atm.press_zout = calloc (Input.atm.nzout,sizeof(float));
                                      for (i=2;i<=Input.atm.nzout+1; i++) {
                                       Input.atm.zout_source = OUTLEVEL_PRESS;
				       t = yytext2string(yytext, i, Input.atm.nzout+1);
				       if      (strncasecmp("toa",t,2)==0) {Input.atm.press_zout[i-2] = ZOUT_TOA;}
				       else if (strncasecmp("sur",t,2)==0) {Input.atm.press_zout[i-2] = ZOUT_SURFACE;}
                                       else                                {Input.atm.press_zout[i-2] = yytext2float(yytext,i,Input.atm.nzout+1);}
				      }
				     }
prndis{INTEGERS}                     {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				      strcpy (s, yytext);
				      Input.rte.nprndis  = yytext2ntokens(s); 
				      Input.rte.prndis   = 
				        yytext2integers(yytext, Input.rte.nprndis); 
				      free(s); }
quiet                               {Input.quiet = 1;}
radiosonde{SPACE}+{WORD}{WORDS}     {Input.atm.rs_source=RS_FROM_FILE;
                                     s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     ntokens = yytext2ntokens(s);
                                     free(s);
				     s =  yytext2string(yytext,2,ntokens+1);
				     strcpy (Input.filename[FN_RADIOSONDE], s);
                                     Input.atm.n_rs_gas=0;

                                     for (i=3;i<=ntokens+1; i++) {
				       t = yytext2string(yytext, i, ntokens+1);  
				       if      (strcmp("O3",t)  ==0||strcmp("o3",t)  ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O3;  Input.atm.n_rs_gas++; mol_id = MOL_O3;  }
				       else if (strcmp("O2",t)  ==0||strcmp("o2",t)  ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_O2;  Input.atm.n_rs_gas++; mol_id = MOL_O2;  }
				       else if (strcmp("H2O",t) ==0||strcmp("h2o",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_H2O; Input.atm.n_rs_gas++; mol_id = MOL_H2O; }
				       else if (strcmp("CO2",t) ==0||strcmp("co2",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_CO2; Input.atm.n_rs_gas++; mol_id = MOL_CO2; }
				       else if (strcmp("NO2",t) ==0||strcmp("no2",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_NO2; Input.atm.n_rs_gas++; mol_id = MOL_NO2; }
				       else if (strcmp("BRO",t) ==0||strcmp("bro",t) ==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_BRO; Input.atm.n_rs_gas++; mol_id = MOL_BRO; }
				       else if (strcmp("OCLO",t)==0||strcmp("oclo",t)==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_OCLO;Input.atm.n_rs_gas++; mol_id = MOL_OCLO;}
				       else if (strcmp("HCHO",t)==0||strcmp("hcho",t)==0) {Input.atm.rs_gas[Input.atm.n_rs_gas]=MOL_HCHO;Input.atm.n_rs_gas++; mol_id = MOL_HCHO;}
				       else if (strcmp("CM_3",t)==0||strcmp("cm-3",t)==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++; 
                                                                                            else Input.atm.rs_unit[mol_id] = CM_3; }
				       else if (strcmp("M_3",t) ==0||strcmp("m-3",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = M_3;  }
				       else if (strcmp("MMR",t) ==0||strcmp("mmr",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = MMR;  }
				       else if (strcmp("VMR",t) ==0||strcmp("vmr",t) ==0) { if (mol_id==0 || mol_id>=MOL_NN) ierror++;
                                                                                            else Input.atm.rs_unit[mol_id] = VMR;  }
                                       else if (strcmp("RH" ,t) ==0||strcmp("rh",t) == 0) {
                                         if (mol_id==0 || mol_id>=MOL_NN) 
                                           ierror++;
                                         else if (mol_id != MOL_H2O) {
                                           fprintf(stderr,"Error, unit \"relative humidity\" only possible for H2O dens_files.\n");
                                           fprintf(stderr,"See 'dens_column ... %s' in line %d\n", s, line_number+1);
                                           ierror++;
                                         }
                                         else 
                                           Input.atm.rs_unit[mol_id]= RH; 
                                        }
				       else {fprintf(stderr,"Unknown 'radiosonde ... %s' on line %d\n", t, line_number+1);
				       ierror++;}
				     }
                                     if (Input.atm.n_rs_gas>2)
                                     for (i=0;i<Input.atm.n_rs_gas-1;i++) {
                                       for (j=i+1;j<Input.atm.n_rs_gas;j++)
                                         if (Input.atm.rs_gas[i] == Input.atm.rs_gas[j]) {
                                           fprintf(stderr, "Error in input file, 'radiosonde', on line %d\n", line_number+1);
                                           fprintf(stderr, "At least one species is specified more than once!\n");
                                           ierror++;
                                         }
                                     }  
                                     if (ierror!=0) {
                                       fprintf(stderr,"Error in input file, 'radiosonde', on line %d\n",line_number+1);  
                                       fprintf(stderr,"Please check the arguments and their order (first species, than unit)\n"); 
                                     }
                                    }
rayleigh_crs{SPACE}+{WORD}          {s =  yytext2string(yytext,2,2);
				     if      (strncasecmp(s,"Bodhaine",8)==0)  Input.rayleigh_crs = RAYLEIGH_BODHAINE;
				     else if (strncasecmp(s,"Nicolet",7)==0)   Input.rayleigh_crs = RAYLEIGH_NICOLET;
				     else {fprintf(stderr,"Unknown 'rayleigh_crs %s' on line %d\n", s, line_number+1);
				           ierror++;}
				     free(s);}
rayleigh_depol{SPACE}+{SIGNED_FLOAT} {Input.rayleigh_depol = yytext2float(yytext,2,2); }
rayleigh_tau_file{SPACE}+{WORD}     {yytextCstring(Input.filename[FN_RAYLEIGH],yytext,2,2); }
reflectivity                        {Input.calibration = OUTCAL_REFLECTIVITY;}
refractive_index_file{SPACE}+{WORD} {yytextCstring (Input.filename[FN_REFIND],
						    yytext, 2, 2);}
reverse                             {Input.rte.reverse    = 1; }
rh_file{SPACE}+{WORD}               {yytextCstring(Input.atm.filename[MOL_H2O],yytext,2,2);
                                     Input.atm.unit_profile[MOL_H2O]= RH;}
rpv_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_RPV],yytext,2,2);
                                     Input.rpv.source = RPV_FROM_RPV_FILE; }
rpv_k{SPACE}+{SIGNED_FLOAT}         {Input.rpv.k     = yytext2float(yytext,2,2); 
                                     if (Input.rpv.source != RPV_CONSTANT)
                                       fprintf(stderr," Warning 'rpv_k' on line %d is ignored as there is already another rpv-input\n", line_number+1);
                                    }
rpv_rho0{SPACE}+{SIGNED_FLOAT}      {Input.rpv.rho0  = yytext2float(yytext,2,2);
                                     if (Input.rpv.source != RPV_CONSTANT)
                                       fprintf(stderr," Warning 'rpv_rho0' on line %d is ignored as there is already another rpv-input\n", line_number+1); 
                                    }
rpv_theta{SPACE}+{SIGNED_FLOAT}     {Input.rpv.theta = yytext2float(yytext,2,2); 
                                     if (Input.rpv.source != RPV_CONSTANT)
                                       fprintf(stderr," Warning 'rpv_theta' on line %d is ignored as there is already another rpv-input\n", line_number+1);
                                    }
rpv_library{SPACE}+{WORD}           {ntokens = yytext2ntokens(yytext);
                                     s = yytext2string(yytext,1+1,ntokens+1);
                                     if (strncasecmp("IGBP",s, 4)==0) {Input.rpv.source = RPV_IGBP_LIBRARY;}
                                     else {
                                       Input.rpv.source = RPV_USER_LIBRARY;
                                       yytextCstring (Input.filename[FN_RPV_LIB_PATH],yytext, 1+1, ntokens+1);
                                       /* append slash */
                                       if (Input.filename[FN_RPV_LIB_PATH][strlen(Input.filename[FN_RPV_LIB_PATH])-1]!= '/')
				         strcat (Input.filename[FN_RPV_LIB_PATH], "/");
                                     }
                                    }
rpv_sigma{SPACE}+{SIGNED_FLOAT}     {Input.rpv.sigma = yytext2float(yytext,2,2); }
rpv_t1{SPACE}+{SIGNED_FLOAT}        {Input.rpv.t1    = yytext2float(yytext,2,2); }
rpv_t2{SPACE}+{SIGNED_FLOAT}        {Input.rpv.t2    = yytext2float(yytext,2,2); }
rpv_scale{SPACE}+{SIGNED_FLOAT}     {Input.rpv.scale = yytext2float(yytext,2,2); }
rte_solver{SPACE}+{WORD}            {s =  yytext2string(yytext,2,2);
				     if      (strncasecmp(s,"sdisort",7)     ==0)  Input.rte.solver = SDISORT;
				     else if (strncasecmp(s,"spsdisort",9)   ==0)  Input.rte.solver = SPSDISORT;
				     else if (strncasecmp(s,"qdisort",7)     ==0)  Input.rte.solver = QDISORT;
				     else if (strncasecmp(s,"disort2",7)     ==0)  Input.rte.solver = DISORT2;
				     else if (strncasecmp(s,"disort",6)      ==0)  Input.rte.solver = DISORT;
				     else if (strncasecmp(s,"sos",3)         ==0)  Input.rte.solver = SUCORD;
				     else if (strncasecmp(s,"polradtran", 6) ==0)  Input.rte.solver = POLRADTRAN;
				     else if (strncasecmp(s,"twostrpp",8)    ==0)  Input.rte.solver = TWOSTRPP;
				     else if (strncasecmp(s,"twostreampp",11)==0)  Input.rte.solver = TWOSTRPP;
				     else if (strncasecmp(s,"twostream",9)   ==0)  Input.rte.solver = TWOSTR;
				     else if (strncasecmp(s,"twostr",6)      ==0)  Input.rte.solver = TWOSTR;
				     else if (strncasecmp(s,"montecarlo",10) ==0)  Input.rte.solver = MONTECARLO;
				     else if (strncasecmp(s,"tzs",3)         ==0)  Input.rte.solver = TZS;
				     else if (strncasecmp(s,"sssi",4)        ==0)  Input.rte.solver = SSSI;
				     else if (strncasecmp(s,"sss2",4)        ==0)  Input.rte.solver = SSS2;
				     else if (strncasecmp(s,"sss",3)         ==0)  Input.rte.solver = SSS;
				     else if (strncasecmp(s,"null",4)        ==0)  Input.rte.solver = NULLSOLVER;
				     else {fprintf(stderr,"Unknown 'rte_solver %s' on line %d\n", s, line_number+1);
				           ierror++;}
				     free(s);}
satellite_geometry{SPACE_WORD}      {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 1) {
                                       fprintf(stderr,"Error, wrong number of arguments for 'satellite_geometry' on line %d\n", line_number+1);
                                       fprintf(stderr,"usage: \n"); 
                                       fprintf(stderr,"   satellite_geometry  netCDF_file  \n"); 
                                       ierror++;
                                     }
                                     yytextCstring(Input.filename[FN_SATGEOMETRY],yytext,2,ntokens+1);
                                    }
satellite_pixel{SPACE_INTEGER}{SPACE_INTEGER} {ntokens = yytext2ntokens(yytext);
                                     if (ntokens != 2) {
                                       fprintf(stderr,"Error, wrong number of arguments for 'satellite_pixel' on line %d\n", line_number+1);
                                       fprintf(stderr,"usage: \n"); 
                                       fprintf(stderr,"   satellite_pixel  pixel_nr_x  pixel_nr_y  \n"); 
                                       ierror++;
                                     }
                                     Input.sat_pixel_x = yytext2int(yytext,2,ntokens+1); 
                                     Input.sat_pixel_y = yytext2int(yytext,3,ntokens+1);
                                    }
slit_function_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_SLITFUNCTION],yytext,2,2);
                                     Input.convolve = 1; }
solar_file{WORDS}                   {ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || ntokens > 2) {
                                       fprintf (stderr, "Error, 'solar_file' on line %d has one mandatory and one optional argument\n",line_number+1); 
                                       ierror++;
                                     }
                                     yytextCstring(Input.filename[FN_EXTRATERRESTRIAL],yytext,1+1,ntokens+1);
                                     if (ntokens == 2) {
                                       s =  yytext2string(yytext,2+1,ntokens+1);
                                       if      (strncasecmp("per_nm"     ,s, 6) == 0) Input.spectrum_unit= UNIT_PER_NM;
                                       else if (strncasecmp("per_cm-1"   ,s, 8) == 0) Input.spectrum_unit= UNIT_PER_CMm1;
                                       else if (strncasecmp("per_ck_band",s,11) == 0) Input.spectrum_unit= UNIT_PER_CK_BAND;
                                       else {fprintf(stderr,"Unknown optional argument for 'solar_file' %s on line %d\n", s, line_number+1);
				             ierror++;}
                                       free(s);
                                     }
                                    }
source{SPACE}+{WORD}                {s =  yytext2string(yytext,2,2);
                                     Input.source=SRC_NONE;
                                     if      (strcasecmp(s,"solar")  ==0)  Input.source = SRC_SOLAR;
				     else if (strcasecmp(s,"thermal")==0)  Input.source = SRC_THERMAL;
                                     else {fprintf(stderr,"Unknown 'source %s' on line %d\n", s, line_number+1);
				           ierror++;}
                                    }
spline{SIGNED_FLOATS}               {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.spline_lambda_0    = yytext2float(s,2,4); 
				     strcpy (s, yytext);
				     Input.spline_lambda_1    = yytext2float(s,3,4); 
                                     Input.spline_lambda_step = yytext2float(yytext,4,4);
                                     Input.spline = 1;
				     free(s);}
spline_file{SPACE}+{WORD}           {yytextCstring(Input.filename[FN_SPLINE],yytext,2,2);
                                     Input.spline = 1; }
surface_temperature{SPACE}+{SIGNED_FLOAT}  {Input.surface_temperature = yytext2float(yytext,2,2); }
surface_temperature_map{WORDS}      {ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || ntokens > 2) {
                                       fprintf (stderr, "Error, 'surface_temperature_map' on line %d has one mandatory and one optional argument\n",line_number+1); 
                                       fprintf (stderr, "optional argument is variable name of the surface temperature in the netCDF file \n");
                                       ierror++;
                                     }
                                     yytextCstring (Input.filename[FN_SURFACE_TEMP_MAP],yytext, 1+1, ntokens+1);
                                     if (ntokens == 2)
                                       yytextCstring (Input.netCDF_name_surf_T,yytext, 1+1, ntokens+1); 
                                    }
surface_type{SPACE_INTEGER}|albedo_surface{SPACE_INTEGER} {
                                     Input.alb.surface = yytext2int(yytext,2,2);
                                     Input.alb.surf_type_map = FALSE;}
surface_type_map{WORDS}             {ntokens = yytext2ntokens(yytext);
                                     Input.alb.surf_type_map = TRUE;
                                     yytextCstring (Input.filename[FN_SURFACE_TYPE_MAP],yytext, 1+1, ntokens+1);
                                     if (ntokens > 1) {
                                       if (ntokens != 2) {
                                         fprintf (stderr, "Error, 'surface_type_map' on line %d requires NO or ONE optional arguments,\n",line_number+1); 
                                         fprintf (stderr, "optional argument is variable name of the albedo in the netCDF file \n");
                                         ierror++;
                                       }
                                       else
                                         yytextCstring (Input.alb.netCDF_surf_name,yytext, 1+1, ntokens+1);  
                                     }
                                    }
sza{SPACE}+{SIGNED_FLOAT}           {Input.atm.sza      = yytext2float(yytext,2,2); 
                                     Input.atm.sza_source = SZA_DIRECT_INPUT;}
sza_file{SPACE}+{WORD}              {yytextCstring(Input.filename[FN_SZA],yytext,2,2);
                                     Input.atm.sza = NOT_DEFINED_FLOAT;
                                     Input.atm.sza_source = SZA_FROM_SZA_FILE;}
thermal_bandwidth{SPACE_FLOAT}{WORDS} {ntokens = yytext2ntokens(yytext);
                                     Input.bandwidth = yytext2double(yytext,1+1,ntokens+1);
                                     if (ntokens==2) {
                                       s =  yytext2string(yytext,2+1,ntokens+1);
				       if      (strcmp("nm",s)==0)   {Input.bandwidth_unit = UNIT_PER_NM ;}
				       else if (strcmp("cm-1",s)==0) {Input.bandwidth_unit = UNIT_PER_CMm1;}
				       else {
                                         fprintf(stderr,"Error, unknown unit %s for 'thermal_bandwidth' on line %d\n", s, line_number+1);
				         ierror++;
                                       }
                                     } 
                                    }
thermal_bands_file{SPACE}+{WORD}    {yytextCstring(Input.filename[FN_WLBANDS],yytext,2,2); }
time{WORDS}{FLOATS}                 {ntokens = yytext2ntokens(yytext);
                                     tmpint=0;
                                     s =  yytext2string(yytext,2,ntokens+1);
                                     if      (strncasecmp(s,"UTC",3) == 0) { Input.time_zone = TIME_UTC; tmpint=1; }
				     else if (strncasecmp(s,"LAT",3) == 0) { Input.time_zone = TIME_LAT; tmpint=1; }
                                     else {Input.time_zone = TIME_UTC; tmpint=0;}
                                     if (Input.time_zone == TIME_UTC) {
                                      if (ntokens == 6+tmpint) {
                                        Input.UTC.tm_year  = yytext2int  (yytext, 2+tmpint, ntokens+1) - 1900; /* !!! years after 1900             !!! */
                                        Input.UTC.tm_mon   = yytext2int  (yytext, 3+tmpint, ntokens+1) -    1; /* !!! month after January 0 ... 11 !!! */
                                        Input.UTC.tm_mday  = yytext2int  (yytext, 4+tmpint, ntokens+1);   
                                        Input.UTC.tm_hour  = yytext2int  (yytext, 5+tmpint, ntokens+1); 
                                        Input.UTC.tm_min   = yytext2int  (yytext, 6+tmpint, ntokens+1);
                                        Input.UTC.tm_sec   = yytext2float(yytext, 7+tmpint, ntokens+1);                                     
                                      }
                                      else {fprintf(stderr,"Error, wrong number of arguments %d for %s on line %d\n",ntokens, s, line_number+1);
				            ierror++;
                                      }
                                     }
                                     else if (Input.time_zone == TIME_LAT) {
                                      if (ntokens == 6+tmpint) {
                                        Input.LAT.tm_year  = yytext2int  (yytext, 2+tmpint, ntokens+1) - 1900; /* !!! years after 1900             !!! */
                                        Input.LAT.tm_mon   = yytext2int  (yytext, 3+tmpint, ntokens+1) -    1; /* !!! month after January 0 ... 11 !!! */
                                        Input.LAT.tm_mday  = yytext2int  (yytext, 4+tmpint, ntokens+1);   
                                        Input.LAT.tm_hour  = yytext2int  (yytext, 5+tmpint, ntokens+1); 
                                        Input.LAT.tm_min   = yytext2int  (yytext, 6+tmpint, ntokens+1);
                                        Input.LAT.tm_sec   = yytext2float(yytext, 7+tmpint, ntokens+1);                                     
                                      }
                                      else {fprintf(stderr,"Error, wrong number of arguments %d for %s on line %d\n",ntokens, s, line_number+1);
				            ierror++;
                                      }
                                     }
                                     else {
                                      fprintf(stderr,"Error, unknown time zone %d, please report this program bug!\n", Input.time_zone);
				      ierror++;
                                     }
                                    } 
time_interpolate{WORDS}             {Input.atm.time_interpolate = TIME_INTERPOLATION;}
transmittance                       {Input.calibration = OUTCAL_TRANSMITTANCE;}
transmittance_wl_file{SPACE}+{WORD} {yytextCstring(Input.filename[FN_WLTRANS],yytext,2,2); }
umu{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.numu     = yytext2ntokens(s); 
				     Input.rte.maxumu   = Input.rte.numu;
				     Input.rte.umu      = 
				       yytext2floats(yytext, Input.rte.numu); 
				     free(s); }
phi{SIGNED_FLOATS}                  {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
				     strcpy (s, yytext);
				     Input.rte.nphi   = yytext2ntokens(s); 
				     Input.rte.maxphi = Input.rte.nphi;
				     Input.rte.phi    = 
				       yytext2floats(yytext, Input.rte.nphi); 
				     free(s); }
verbose                             {Input.verbose = 1;}
wavelength{FLOATS}|wvn{FLOATS}      {ntokens = yytext2ntokens(yytext);
				     Input.wl.start = yytext2float(yytext,2,ntokens+1); 
                                     if (ntokens == 1)
                                       Input.wl.end   = Input.wl.start;
                                     else if (ntokens == 2)
                                       Input.wl.end   = yytext2float(yytext,3,ntokens+1);
                                     else {
                                       fprintf(stderr,"Wrong number of arguments for 'wavelength' on line %d\n", line_number+1);
				       ierror++;
				     }
                                    }
wavelength_index{INTEGERS}|wvn_index{INTEGERS}  {ntokens = yytext2ntokens(yytext); 
				     Input.wl.start_index = yytext2int(yytext,2,ntokens+1); 
                                     if (ntokens == 1)
                                       Input.wl.end_index = Input.wl.start_index;
				     else if (ntokens == 2) 
                                       Input.wl.end_index = yytext2int(yytext,3,ntokens+1);
                                     else {
                                       fprintf(stderr,"Wrong number of arguments for 'wavelength_index' on line %d\n", line_number+1);
				       ierror++;
				     }
                                    }
wc_cloudcover{SPACE}+{FLOAT}        {Input.wc_cloudcover=yytext2float(yytext,2,2); 
                                     Input.wcipa=1;}
wc_file{SPACE}+{WORD}               {Input.wc.source = CLOUDS_FROM_FILE;
                                     yytextCstring (Input.filename[FN_WCLOUD],       yytext,2,2); 
                                     Input.wcipa=0;
                                    }
wc_files{SPACE}+{WORD}              {Input.wc.source = CLOUDS_LAYER_FILES;
                                     yytextCstring (Input.filename[FN_WCLOUD_FILES], yytext,2,2);
                                     Input.wc.properties = WCPROP_EXPLICIT;
                                     Input.wcipa=0; 
                                    }
wc_ipa{SPACE_WORD}                  {Input.wc.source = CLOUDS_FROM_3D_FILE;
                                     Input.wcipa=1;
                                     yytextCstring (Input.filename[FN_WCLOUD_IPA_3D],yytext,2,2);
                                    }
wc_ipa_files{SPACE}+{WORD}          {Input.wc.source = CLOUDS_IPA_FILES;
                                     yytextCstring (Input.filename[FN_WCLOUD_IPA],   yytext,2,2);
                                     Input.wcipa=1;
                                     Input.wc.properties = WCPROP_HU;
                                    }
wc_layer                            {Input.wc.layer=TRUE;}
wc_properties{SPACE}+{WORD}         {s =  yytext2string(yytext,2,2);
                                     if      (strncasecmp("hu",s,2)==0)     Input.wc.properties = WCPROP_HU;
				     else if (strncasecmp("echam4",s,6)==0) Input.wc.properties = WCPROP_ECHAM4;
				     else if (strncasecmp("mie",s,3)==0)    Input.wc.properties = WCPROP_MIE;
				     else {                                 Input.wc.properties = WCPROP_FILE;
					                                    strcpy (Input.filename[FN_WCLOUD_PROP], s);}
				    }
wc_properties_interpolate           {Input.wc.interpolate=1; }
wc_saturate{SPACE}+{WORD}+{SPACE_FLOAT}* { ntokens = yytext2ntokens(yytext);
                                     if (ntokens < 1 || ntokens > 2) {
                                       fprintf(stderr,"Wrong number of arguments for 'wc_saturate' on line %d\n", line_number+1); 
                                       ierror++;
                                     }
				     s = yytext2string(yytext,2,ntokens+1);
				     if      (strcmp("off",s) == 0 || strcmp("OFF",s) == 0) {Input.wc.wv_saturation = SATURATION_NONE;}
                                     else if (strcmp("on" ,s) == 0 || strcmp("ON" ,s) == 0) {Input.wc.wv_saturation = SATURATION_CLOUD;}
                                     else if (strcmp("ipa",s) == 0 || strcmp("IPA",s) == 0) {Input.wc.wv_saturation = SATURATION_IPA;}
                                     else {
                                       fprintf(stderr,"Unknown 'saturate %s' on line %d\n", s, line_number+1); 
                                       ierror++;
                                     }
                                     free(s);
                                     if (ntokens == 2) 
                                       Input.wc.wv_sat_ratio = yytext2float(yytext,3,ntokens+1);
                                    }
wc_scale_gg{SPACE}+{FLOAT}          {Input.wc.scale_gg   = yytext2float(yytext,2,2); }
wc_scale_ssa{SPACE}+{FLOAT}         {Input.wc.scale_ssa  = yytext2float(yytext,2,2); }
wc_set_gg{SPACE}+{FLOAT}            {Input.wc.gg  = yytext2float(yytext,2,2); }
wc_set_ssa{SPACE}+{FLOAT}           {Input.wc.ssa = yytext2float(yytext,2,2); }
wc_set_tau{SPACE}+{FLOAT}           {Input.wc.tau = yytext2float(yytext,2,2); }
wc_set_tau550{SPACE}+{FLOAT}        {Input.wc.tau550 = yytext2float(yytext,2,2); }
z_interpolate{WORDS}                {s = (char *) calloc (strlen(yytext)+1,sizeof(char));
                                     strcpy (s, yytext);
                                     token_number = yytext2ntokens(s); 
                                     free(s);
                                     switch (token_number) {
                                     case 0:
                                       Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX;
                                       break;
                                     case 1:
                                       t = yytext2string(yytext, 2, token_number+1);
                                       if      (strncasecmp("default",t, 7)==0)  { Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; }
                                       else if (strncasecmp("lin_mix",t, 7)==0 || strncasecmp("linmix",t, 6)==0)
                                                                                 { Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; }
                                       else if (strncasecmp("log_lin",t, 7)==0 || strncasecmp("loglin" ,t, 6)==0)  
                                                                                 { Input.atm.z_interpolate = Z_INTERPOLATE_LOGLIN; }
                                       else if (strncasecmp("spline" ,t, 6)==0)  { Input.atm.z_interpolate = Z_INTERPOLATE_SPLINE; }
                                       else {fprintf (stderr, "Error, unknown 'z_interpolate %s' on line %d\n", t, line_number+1);
				             ierror++;}
                                       free(t);
                                       break;
                                     case 2:
				       t =  yytext2string(yytext,2,token_number+1);
				       if      (strcmp("O3",t)  ==0 || strcmp("o3",t)  ==0)  mol_id = MOL_O3; 
                                       else if (strcmp("O2",t)  ==0 || strcmp("o2",t)  ==0)  mol_id = MOL_O2; 
                                       else if (strcmp("H2O",t) ==0 || strcmp("h2o",t) ==0)  mol_id = MOL_H2O; 
                                       else if (strcmp("CO2",t) ==0 || strcmp("co2",t) ==0)  mol_id = MOL_CO2; 
                                       else if (strcmp("NO2",t) ==0 || strcmp("no2",t) ==0)  mol_id = MOL_NO2; 
                                       else if (strcmp("BRO",t) ==0 || strcmp("bro",t) ==0)  mol_id = MOL_BRO; 
                                       else if (strcmp("OCLO",t)==0 || strcmp("oclo",t)==0)  mol_id = MOL_OCLO; 
                                       else if (strcmp("HCHO",t)==0 || strcmp("hcho",t)==0)  mol_id = MOL_HCHO; 
                                       else if (strcmp("T" ,t)==0 || strcmp("t" ,t)==0)  mol_id = -1;
                                       else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				             ierror++;}
                                       free(t);  
                                       t =  yytext2string(yytext,3,token_number+1);
                                       if (mol_id > 0) {
                                         if      (strcmp("SPLINE",t)==0 || strcmp("spline",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_SPLINE;  
                                         else if (strcmp("LINEAR",t)==0 || strcmp("linear",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LINEAR; 
                                         else if (strcmp("LOG",t)   ==0 || strcmp("log"   ,t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LOG;
                                         else if (strcmp("LINMIX",t)==0 || strcmp("linmix",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LINMIX; 
                                         else if (strcmp("LOG_SPLINE",t)==0 || strcmp("log_spline",t)==0) 
                                           Input.atm.interpol_method_gas[mol_id] = INTERP_METHOD_LOG_SPLINE;
                                         else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				               ierror++;}
                                       }
                                       else if (mol_id == -1) {
                                         if      (strcmp("SPLINE",t)==0 || strcmp("spline",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_SPLINE;  
                                         else if (strcmp("LINEAR",t)==0 || strcmp("linear",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
                                         else if (strcmp("LOG",t)   ==0 || strcmp("log"   ,t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LOG;
                                         else if (strcmp("LINMIX",t)==0 || strcmp("linmix",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LINMIX; 
                                         else if (strcmp("LOG_SPLINE",t)==0 || strcmp("log_spline",t)==0) 
                                           Input.atm.interpol_method_temper = INTERP_METHOD_LOG_SPLINE;
                                         else {fprintf (stderr,"Error, unknown z_interpolate option: %s on line %d\n", t, line_number+1);
				               ierror++;}
                                       }
                                       free(t);
                                       break;
                                     default:
                                       fprintf (stderr,"Error, z_interpolate has wrong number of arguments on line %d.\n", line_number+1);
                                       ierror++;
                                     }
				    }
zout{SIGNED_FLOATS_OR_WORDS}        {Input.atm.nzout = yytext2ntokens(yytext);
                                     Input.atm.zout_sur = calloc (Input.atm.nzout,sizeof(float));
                                     for (i=2;i<=Input.atm.nzout+1; i++) {
                                       Input.atm.zout_source = OUTLEVEL_ZOUT_ABOVE_SUR;
				       t = yytext2string(yytext, i, Input.atm.nzout+1);
				       if      (strncasecmp("toa",t,3)==0) {Input.atm.zout_sur[i-2] = ZOUT_TOA;}
				       else if (strncasecmp("sur",t,3)==0) {Input.atm.zout_sur[i-2] = ZOUT_SURFACE;}
				       else if (strncasecmp("atm_levels",  t,10)==0) Input.atm.zout_source = OUTLEVEL_ATM_LEVELS;
				       else if (strncasecmp("all_levels",  t,10)==0) Input.atm.zout_source = OUTLEVEL_ALL_LEVELS;
				       else if (strncasecmp("model_levels",t,13)==0) Input.atm.zout_source = OUTLEVEL_MODEL_LEVELS;
                                       else                                {Input.atm.zout_sur[i-2] = yytext2float(yytext,i,Input.atm.nzout+1);}

                                       /* check only keyword and no other arguments allowed */
                                       if (strncasecmp("atm_levels",t,10)==0 ||
                                           strncasecmp("all_levels",t,10)==0 || 
                                           strncasecmp("model_levels",t,13)==0) {
                                         if (Input.atm.nzout != 1) {
                                           fprintf (stderr,"Error, using 'zout %s' in line %d accept no further levels!!!\n", t, line_number+1);
				           ierror++;
                                         }
                                       }
				     }
				    }
zout_sea{SIGNED_FLOATS_OR_WORDS}    {Input.atm.nzout = yytext2ntokens(yytext);
                                     Input.atm.zout_sea = calloc (Input.atm.nzout,sizeof(float));
                                     for (i=2;i<=Input.atm.nzout+1; i++) {
                                       Input.atm.zout_source = OUTLEVEL_ZOUT_ABOVE_SEA;
				       t = yytext2string(yytext, i, Input.atm.nzout+1);
				       if      (strncasecmp("toa",t,3)==0) {Input.atm.zout_sea[i-2] = ZOUT_TOA;}
				       else if (strncasecmp("sur",t,3)==0) {Input.atm.zout_sea[i-2] = ZOUT_SURFACE;}
				       else if (strncasecmp("atm_levels",t,10)==0) {
                                         Input.atm.zout_source = OUTLEVEL_ATM_LEVELS;
                                         if (Input.atm.nzout != 1) {
                                           fprintf (stderr,"Error, using 'zout %s' in line %d accept no further levels!!!\n", t, line_number+1);
				           ierror++;
                                         } 
                                       }
				       else if (strncasecmp("all_levels",t,10)==0) {
                                         Input.atm.zout_source = OUTLEVEL_ALL_LEVELS;
                                         if (Input.atm.nzout != 1) {
                                           fprintf (stderr,"Error, using 'zout %s' in line %d accept no further levels!!!\n", t, line_number+1);
				           ierror++;
                                         } 
                                       }
                                       else                                {Input.atm.zout_sea[i-2] = yytext2float(yytext,i,Input.atm.nzout+1);}
				     }
				    }
zout_interpolate                    {Input.atm.zout_interpolate = ZOUT_INTERPOLATE;}
mc_delta_scaling                    {Input.rte.mc.delta_scaling_mucut=0.99;} /*TZ ds*/
mc_delta_scaling{SPACE}+{SIGNED_FLOAT} {Input.rte.mc.delta_scaling_mucut = yytext2float(yytext,2,2);} /*TZ ds*/



{SPACE}+       ;         /* eat up blanks */ 

{COMMENT}*     ;         /* eat up comments */

\n                        { ++line_number;};

{WORD}                    {fprintf(stderr,"Unknown command on line %d\n", line_number+1);
                           fprintf(stderr,"%s\n", yytext);
                           ierror++;}


<include>[ \t]*      /* eat the whitespace */
<include>[^ \t\n]+   { /* got the include file name */
        if (include_stack_ptr >= MAX_INCLUDE_DEPTH) {
            fprintf (stderr, "Includes nested too deeply");
            exit (1);
        }

        include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;

        yyin = fopen(yytext, "r");

        if (!yyin) {
            fprintf (stderr, "Error including %s\n", yytext);
            exit(1);
        }

        yy_switch_to_buffer (yy_create_buffer (yyin, YY_BUF_SIZE));

        BEGIN(INITIAL);
        }

<<EOF>> {
        if (--include_stack_ptr < 0) {
            yyterminate();
        }
        else {
            yy_delete_buffer (YY_CURRENT_BUFFER);
            yy_switch_to_buffer (include_stack[include_stack_ptr]);
        }
        }

%%

/*************************************************************/
/* print usage information                                   */
/*************************************************************/

#ifndef UVSPEC_FUNCTION
static void version()
{
  fprintf (stderr, "uvspec, version %s\n", VERSION);
}

static void usage()
{

  fprintf (stderr, "\n");  
  version();
  fprintf (stderr, "\nCalculates the 1D radiation field of the Earth\'s atmosphere.\n");
  fprintf (stderr, "See libRadtran documentation for more information.\n\n");
  fprintf (stderr, "No error checking yet, so be careful.\n\n");

  fprintf (stderr, "Usage: uvspec [-h] < input_file > Output_file\n");
  fprintf (stderr, " -h             Print this message.\n");  
  fprintf (stderr, " -i input_file  Alternative way to specifiy the input file.\n");  
  fprintf (stderr, " -f input_file  Same as -i.\n");  

  fprintf (stderr, "\n");  
}
#endif


/* global variable, defined in uvspec_lex.l */
#if HAVE_LIBGSL 
  extern gsl_rng *uvspecrng;  /* global random number generator */
#endif

#ifndef UVSPEC_FUNCTION
int main(int argc, char **argv) 
#else
int calluvspec (char *outfilename)
#endif
{
  

/* @11c@

@include mystic.sdoc

@code{uvspec} calculates the radiation field in the Earth's atmosphere
for a given set of input parameters. It reads input from standard input, 
and outputs to standard output. It is normally invoked in the following
way: 

@example
@cartouche
uvspec < input_file > output_file
@end cartouche
@end example

@noindent The format of the input and output files are described
below. Several realistic examples of input files are subsequently 
given.

@noindent uvspec may produce a wealth of diagnostic messages 
and warnings, depending on your use of @code{verbose} or @code{quiet}.
Diagnostics, error messages, and warnings are written to @code{stderr} while
the uvspec output is written to @code{stdout}. To make use of this extra 
information, you may want to write the standard uvspec output to one file 
and the diagnostic messages to another. To do so, 
try @code{(./uvspec < uvspec.inp > uvspec.out) >& verbose.txt}. The 
irradiances and radiances will be written to @file{uvspec.out} while all 
diagnostic messages go into @file{verbose.txt}. This method can also be used  
to collect uvspec error messages. 


@strong{Warning:} Please note the error checking on input variables 
is not complete at the moment. Hence, if you provide erroneous input, 
the outcome is unpredictable.
  @c11@ */

  /*  @11_03c@                
@cindex input file, uvspec
@cindex uvspec, input file

The uvspec input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a @code{#}. Blank lines are ignored. 
The order of the lines is not important, with one exception: if the
same input option is used more than once, the second one will usually
over-write the first one. Be aware that also option in another @code{included} 
input file will overwrite options specified before.

The various input parameters are described in detail below.
@vtable @code

@item absorption
Switch off absorption by individual minor trace gases which are currently only
included when @code{correlated_k lowtran} is chosen. The syntax is
@example
  absorption species on/off
@end example
where species may be one of O4, N2, CO, SO2, NH3, NO, HNO3. By default all
are switched on. 
@item aerosol_angstrom
Scale the aerosol optical depth using the @AA{}ngstr@"om formula. Specify 
the @AA{}ngstr@"om alpha and beta coefficients. The optical thickness defined here 
is the integral from the user-definded @code{altitude} to TOA (top of atmosphere).
@item aerosol_default
Set up a default aerosol according to Shettle (1989). The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}.
@item aerosol_files
A way to specify aerosol optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{aerosol_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the @code{atmosphere_file}. The second
column is a the name of a file which defines the optical properties of the layer
starting at the given altitude. The files specified in the second column must have the 
following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The aerosol single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the aerosol phase function. 
@end table
For some simple examples see the files @file{examples/AERO_*.LAYER}. Note that 
if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
Please note that the uppermost line of the @code{aerosol_files}
denotes simply the top altitude of the uppermost layer. The optical
properties of this line are consequently ignored. There are two
options for this line: either an optical property file with zero
optical thickness is specified or "NULL" instead.
@item aerosol_gg_file
Location of aerosol asymmetry parameter file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the asymmetry parameter of each layer. 
At present, the asymmetry parameter defined here is constant with wavelength
but it is planned to introduce the wavelength dependence.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_haze
Aerosol type in the lower 2 km of the atmosphere. Integer.
See E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989.

@table @code
@item 1
Rural type aerosols.
@item 4
Maritime type aerosols.
@item 5
Urban type aerosols.
@item 6
Tropospheric type aerosols.
@end table
@item aerosol_moments_file
Location of aerosol moments file, a one-column file containing an
arbitrary number of Legendre terms of the phase function. The phase function 
@iftex
 @tex
 $p(\mu)$ is
  $$ p (\mu) = \sum_{m=0}^{\infty} (2m+1) \cdot k_m \cdot P_m (\mu) $$
  where $k_m$ is the m'th moment and $P_m (\mu)$ is the m'th Legendre polynomial.
 @end tex
@end iftex
@ifinfo
 p(mu) is @*
 p (mu) = sum (m=0 to infinity) (2m+1) * k(m) * Pm (mu) @*
 where k(m) is the m'th moment and Pm (mu) is the m'th Legendre polynomial.
@end ifinfo
If not specified, a Henyey-Greenstein phase function is assumed 
where the asymmetry parameter g is either a default value depending on the 
aerosol type or may be specified using @code{aerosol_set_gg}.
@item aerosol_refrac_file
File containing the wavelength-dependent refractive index of the
aerosol. Three columns are expected: wavelength [nm] and the 
real and imaginary parts of the refractive index.
Together with @code{aerosol_sizedist_file} this forms the 
input to Mie calculations of the aerosol optical properties. Please note that 
only the single-scattering albedo, the scattering phase function, and the
wavelength-dependence of the extinction coefficient are affected by the Mie calculation 
while the absolute value of the extinction coefficient is taken from other sources;
generally, the extinction coefficient at the first @emph{internal} wavelength is
taken from whatever is available (either default Shettle (1989) or user-defined);
the extinction at all other wavelengths is scaled according to the Mie calculation. 
For this reason, the absolute numbers are not relevant - only the shape of the 
size distribution matters. In detail:
If the aerosol properties are defined using the refractive index and the 
size distribution, the wavelength dependence of the optical properties is 
determined by Mie theory. At present there are at least three ways to define the 
absolute value of the optical thickness: (1) @code{visibility} defines the 
profile at the first @emph{internal} wavelength; for a monochromatic calculation
and in correlated-k mode, the first @emph{internal} wavelength equals the 
first wavelength output by @code{uvspec}; for spectral calculations, the 
first wavelength might be a little bit smaller than the first wavelength 
output by @code{uvspec}; (2) @code{aerosol_tau_file} defines the optical 
thickness profile at the first @emph{internal} wavelength; or (3) absolute
optical thickness and wavelength-dependence are defined by @code{aerosol_angstrom}.
@emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
properties externally e.g. with mie and to pass them to uvspec with 
aerosol_files}.
@item aerosol_refrac_index
Wavelength-independent refractive index of the aerosol; if wavelength-dependence 
is required, use @code{aerosol_refrac_file} instead. Together with 
@code{aerosol_sizedist_file} this forms the input to Mie calculations of the 
aerosol optical properties. Please see the description of @code{aerosol_refrac_file}
to learn how the optical properties are set up.
@emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
properties externally e.g. with mie and to pass them to uvspec with 
aerosol_files}.
@item aerosol_scale_ssa
Scale the aerosol single scattering albedo for all wavelengths and altitudes
with a positive number. If the resulting scaled single scattering
albedo is larger than 1 it is set to 1.
@item aerosol_scale_tau
Scale the aerosol extinction for all wavelengths and altitudes
with a positive number.
@item aerosol_set_gg
Set the aerosol asymmetry parameter for all wavelengths and altitudes
to a constant value between -1.0 and 1.0.
@item aerosol_set_ssa
Set the aerosol single scattering albedo for all wavelengths and altitudes
to a constant value between 0.0 and 1.0.
@item aerosol_set_tau
Set the aerosol optical thickness for all wavelengths and altitudes
to a constant value. The optical thickness defined here is the
integral from the user-definded @code{altitude} to TOA (top of atmosphere). 
@item aerosol_set_tau550
Set the aerosol optical thickness at 550nm. Other wavelengths are scaled accordingly.
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wavelength} does contain 550nm.
The optical thickness defined here is the integral from the user-definded 
@code{altitude} to TOA (top of atmosphere). 
@item aerosol_season
Specify season to get appropriate aerosol profile.
@table @code
@item 1
Spring-summer profile.
@item 2
Fall-winter profile.
@end table
@item aerosol_sizedist_file
Aerosol size distribution. Two columns are expected: The radius
[micron] and the particle number. Together with @code{aerosol_refrac_index} or
@code{aerosol_refrac_file} this forms the input to Mie calculations of the 
aerosol optical properties. Please note that 
only the single-scattering albedo, the scattering phase function, and the
wavelength-dependence of the extinction coefficient are affected by the Mie calculation 
while the absolute value of the extinction coefficient is taken from other sources;
generally, the extinction coefficient at the first @emph{internal} wavelength is
taken from whatever is available (either default Shettle (1989) or user-defined);
the extinction at all other wavelengths is scaled according to the Mie calculation. 
For this reason, the absolute numbers are not relevant - only the shape of the 
size distribution matters. For details see also the description of @code{aerosol_refrac_file}.
@emph{It is recommended to avoid this option and rather to calculate the aerosol optical 
properties externally e.g. with mie and to pass them to uvspec with 
aerosol_files}.
@item aerosol_ssa_file
Location of aerosol single scattering albedo file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the single scattering albedo of each layer. 
At present, the single scattering albedo defined here is constant with wavelength
but it is planned to introduce the wavelength dependence.
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_tau_file
Location of aerosol optical depth file. The file must have two columns.
Column 1 is the altitude in km. The altitude grid must be exactly equal to 
the altitude grid specified in the file @code{atmosphere_file}.
Column 2 is the aerosol optical depth of each layer. 
To get wavelength dependence use the parameter @code{aerosol_angstrom}. 
Comments start with @code{#}. Empty lines are ignored.
@item aerosol_visibility
Visibility in km.
@item aerosol_vulcan
Aerosol situation above 2 km. Integer.
@table @code
@item 1
Background aerosols.
@item 2
Moderate vulcanic aerosols.
@item 3
High vulcanic aerosols.
@item 4
Extreme vulcanic aerosols.
@end table
@item albedo
The Lambertian surface albedo, a number between 0.0 and 1.0,
constant for all wavelengths. For wavelength dependent surface albedo 
use @code{albedo_file}. The default albedo is 0.0.
@item albedo_file
Location of surface albedo file for wavelength dependent surface albedo. 
The file must have two columns.
Column 1 is the wavelength in nm, and column 2 the corresponding 
Lambertian surface albedo. The wavelength grid may be freely set. The 
albedo will be interpolated (linear) to the wavelength grid used for the 
radiation calculation. Comments start with @code{#}. Empty lines are ignored.
A large collection of spectral albedos is available e.g. at http://speclib.jpl.nasa.gov/.
@item albedo_library
Albedo libraries are a collection of spectral albedos of different surface types. 
This option must be used either with @code{surface_type} 
or @code{surface_type_map}, in order to select the specific surface type.
There are two possibilities for libraries: 
the built-in IGBP library or a user defined albedo library.

The built-in library of the International Geosphere Biosphere Programme is 
selected with @code{albedo_library IGBP}.
The IGBP library contains 20 surface types:
@example
 1 evergreen_needle_forest
 2 evergreen_broad_forest
 3 deciduous_needle_forest
 4 deciduous_broad_forest
 5 mixed_forest
 6 closed_shrubs
 7 open_shrubs
 8 woody_savanna
 9 savanna
10 grassland
11 wetland
12 cropland
13 urban
14 crop_mosaic
15 antarctic_snow
16 desert
17 ocean_water
18 tundra
19 fresh_snow
20 sea_ice
@end example
Surface types 1 - 17 are defined by the International Geosphere Biosphere 
Programme (IGBP); additionally there are tundra, fresh_snow, and sea_ice surface types.
The spectral albedo of the ground is determined as a function of solar 
zenith angle, precitable water, and clouds. The spectral resolution 
equals the grid of the correlated-k Fu/Liou parameterisation.
This library originates from the NASA CERES/SARB Surface Properties Project, 
see Belward and Loveland (1996).

For creating your own albedo library use @code{albedo_library path}, where @code{path} is 
the path of the directory where the albedo data is stored. The files are expected to have the 
names @code{albedo_01.dat, albedo_02.dat, ...} If @code{surface_type 1} is specified
the albedo from @code{albedo_01.dat} will be used, and so on.  
Each file is required to have two columns:
Column 1 is the wavelength in nm, and column 2 the corresponding 
Lambertian surface albedo. The wavelength grid may be freely set. The 
albedo will be interpolated linearely to the wavelength grid used for the 
radiation calculation. Comments start with @code{#}. Empty lines are ignored.
This option is similar to @code{albedo_file}, except that it offers an
easy way to use the option @code{surface_type_map} in combinition with albedo files.)
@item albedo_map
@emph{This option is preliminary and still subject to change (no wavelength dependency yet)!}
A possibility to specify a wavelength independent albedo with the help of 
a @emph{netCDF} file, which is used in combinition 
with the options @code{latitude}, @code{longitude}, and @code{time}.
@example
albedo_map file [variable_name]
@end example
where @code{file} is the location of the @emph{netCDF} file. 
With the optional argument the name of the albedo variable in 
the @emph{netCDF} file can be specified (the default name is @code{AL}).
The albedo must be provided as function of @code{latitude} and @code{longitude} @code{AL(lat, lon)}, 
and may also depend on time @code{AL(time, lat, lon)}. 
The latitude, longitude, and timr grids must be provided as doubles 
@code{double lat(lat)}, @code{double lon(lon)}, and @code{double time(time)}.
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.
If a time-dependent albedo is provided, the albedo data nearest to the specified time 
will be selected (or interpolated if @code{time_interpolate} is switched on). 
@item altitude
Set the bottom level in the model atmosphere provided in 
@code{atmosphere_file} to be at altitude (km).
@example
altitude 0.73   # Altitude of IFU, Garmisch-Partenkirchen 
                # Be aware, for this to work the atmosphere
                # file must start at 0 km.
@end example
The profiles of pressure, temperature, molecular absorbers, 
ice and water clouds are cut at the specified altitude. 
The aerosol profile is not affected by @code{altitude} but starts
right from the model surface. This is a convenient way for the user to calculate the 
radiation at other altitudes than sealevel. Note that @code{altitude} is very different
from @code{zout} where the radiation is calculated at an altitude of zout
above the surface. E.g. to calculate the radiation field 1 km above the surface 
at a location at 0.73 km above sealevel, one would specify '@code{altitude} 0.73' 
and '@code{zout} 1.0'.
If an altitude is specified which is below the lowest level in the 
@code{atmosphere_file}, the atmospheric profiles are extrapolated assuming a constant 
gradient for temperature and mixing ratios.

A second optional argument may be given to @code{altitude} as e.g.
@example
altitude 0.73 0.5
@end example
Here the bottom level will be at 0.73 km and the vertical resolution of the model
atmosphere will be redistributed to have a spacing between levels specified by the second number,
here 0.5 km. Be aware that specifying a fine vertical spacing will produce many layers thus increasing the 
computing time. Also the radiative transfer equation solvers implemented in Fortran 77 might need
to have some array sizes increased (see @file{src_f/DISORT.MXD}).
@item altitude_map
Specifies an altitude map which is used in combinition with 
@code{latitude}, @code{longitude} in order to 
select the altitude for the simulation.
No interpolation is done between the pixels of the map.
The format of the call is:
@example
altitude_map file [variable_name]
@end example
where @code{file} is the location of the altitude map file. The map is expected to 
be in @emph{netCDF} format. The file must contain @code{double lat(lat)}, 
@code{double lon(lon)}, and the altitude variable, where @code{variable_name} is the 
name of the surface elevation variable in the @emph{netCDF} file. The default name is Z.
The altitude variable must be @code{altitude(lat, lon)}. 

For format discribtion see also the example map included in libRadtran, data/altitude/ELEVATION_GTOPO_10min.cdf.
To use this map in @code{uvspec}, you may also use @code{altitude_map GTOPO}. This map has a resolution 
of 10 arc minutes and the unit of the altitude is meter. Please note that this resolution
might not ne adequate for your application. 

If an altitude in the map is below the lowest level of the 
@code{atmosphere_file}, the atmospheric profiles are extrapolated assuming a constant 
gradient for temperature and mixing ratios.

@item atmosphere_file
Location of the atmospheric data file. Must have at least three columns containing the altitude,
pressure, and temperature. Missing profiles are filled with 0 (e.g., if you did not specify 
the ozone profile, there will be no ozone absorption!), with exception of the air density which 
is calculated from pressure and temperature. Other traces gases may be set by @code{dens_file}.  
The columns are interpreted as follows:
@table @code
@item 1. z(km) 
Altitude above sea level in km
@item 2. p(mb)
Pressure in hPa
@item 3. T(K)
Temperature in K
@item 4. air_density(cm-3) 
Air density in cm-3
@item 5. o3_density(cm-3)
Ozone density in cm-3
@item 6. o2(cm-3)
Oxygen density in cm-3
@item 7. h2o(cm-3) 
Water vapour density in cm-3
@item 8. co2(cm-3)
CO2 density in cm-3
@item 9. no2(cm-3)
NO2 density in cm-3
@end table
The atmosphere is specified
top-down, that is, the top level is the first line in the file, the bottom
(surface) level the last line. All properties refer to model @emph{level} z,
not to model @emph{layer}. It is important that the correct units are 
used, otherwise unpredictable results are guaranteed.
Comments start with @code{#}. Empty lines are ignored. Please note that there 
is some redundancy: Assuming that air is an ideal gas, the air density can be
calulated from pressure and temperature, dens = p / kT. @code{uvspec} will check 
if this relation is fulfilled and will issue a warning if it is not. 
libRadtran provides the six standard atmospheres by Anderson et al. (1986):
@example
afglt   Tropical                                              
afglms  Midlatitude Summer                                    
afglmw  Midlatitude Winter                                    
afglss  Subarctic Summer                                      
afglsw  Subarctic Winter                                      
afglus  U.S. Standard                                         
@end example
If no atmosphere file is given, libRadtran will automatically select an @code{atmosphere_file}. If the information 
@code{time}, @code{latitude} and @code{longitude} are provided in the input file libRadtran will 
choose from the first 5 files, otherwise it takes the U.S. Standard atmosphere.
@item brightness
Convert radiances / irradiances to brightness temperatures.
@item ch4_mixing_ratio
The mixing ratio of CH4 in ppm (default: 1.6 ppm).
@ifset mystic
@item cloud_fraction_file
File containing a cloud fraction profile. Two columns are expected:
altitude [km] and cloud fraction, including ice and water clouds. If @code{cloud_fraction_file}
is defined, effective cloud properties are calculated assuming either random overlap or maximum random overlap
of the cloud layers (see also @code{cloud_overlap}).
@item cloud_overlap
Cloud overlap assumption. Two options are implemented:
@table @code
@item rand 
Random overlap of cloud layers
@item maxrand 
Maximum random overlap scheme
@end table
Per default "random overlap" is assumed.
@end ifset
@item co2_mixing_ratio
The mixing ratio of CO2 in ppm; scale the profile so that the mixing 
ratio at the user-defined @code{altitude} assumes the specified value.
@item correlated_k
To calculate integrated shortwave or longwave irradiance, or to simulate 
satellite instrument channels, choose between the following 
correlated-k schemes:
@table @code
@item Kato 
Kato et al. (1999), shortwave; based on HITRAN 96. Please note that the 
bands above 2.5 micron are not very reliable which, however, has only little impact
on integrated shortwave radiation.
@item Kato2
Kato et al. (1999), shortwave; optimized version (Seiji Kato, personal communication, 2003);
please note that @code{Kato2} only has 148 subbands (that is, calls to the @code{rte_solver}) 
compared to 575 for @code{Kato} which translates to an increase in computational speed by 
up to a factor of 4 with only little increase in uncertainty. The absorption data are
based on HITRAN 2000. Please note that the bands above 2.5 micron are not very reliable which, 
however, has only little impact on integrated shortwave radiation.
@item Kato2.96
Kato et al. (1999), shortwave; optimized version (Seiji Kato, personal communication, 2003);
similar to @code{Kato2} but based on HITRAN96. Please note that the bands above 2.5 micron 
are not very reliable which, however, has only little impact on integrated shortwave radiation.
@item Fu
Fu and Liou (1992/93), shortwave and longwave; fast parameterization, developed for climate models.
@item AVHRR_KRATZ
Kratz (1995), AVHRR instrument channels
@item LOWTRAN
Gas absorption parameterization from LOWTRAN; code adopted from SBDART (Ricchiazzi et al., 1998); 
please see the section on "Spectral resolution".
@item SBDART
Identical to LOWTRAN.
@end table
If @code{correlated_k} KATO/KATO2/KATO2.96/FU/AVHRR_KRATZ is specified, the extraterrestrial 
flux is taken from
internally defined files specific for each parameterization, not 
from @code{solar_file}. The output is the integrated irradiance for 
each band. To get e.g. integrated shortwave irradiance, simply add all 
bands of the Kato et al. (1999) or the Fu and Liou (1992/93)
parameterization. The five AVHRR channels are weighted sums of the
libRadtran output. Examples how to integrate the output in the
@code{AVHRR_KRATZ} case are included in the @code{uvspec} self-test
which is initiated with  
@code{make check}.
@item cox_and_munk_pcl
Pigment concentration for Cox and Munk ocean surface BRDF (in mg/m-3); 
at present only available with @code{rte_solver DISORT2}. 
The number of streams (@code{nstr}) is automatically increased
to 16 if cox_and_munk BRDF is switched on, to avoid numerical
problems. 
@item cox_and_munk_pcl_map
A possibility to specify pigment concentration (in mg/m3) for the Cox and Munk 
ocean surface BRDF with the help of an @emph{netCDF} file, which is used in combinition 
with options @code{latitude}, @code{longitude}, and @code{time}.
@example
cox_and_munk_pcl_map file [variable_name]
@end example
where @code{file} is the location of the @emph{netCDF} file. 
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.

The default name of the pigment concentration variable is @code{chlorophyll}, but can be changed with the
optional argument @code{variable_name}.
The pigment concentration must be provided as function of @code{latitude} and @code{longitude}, @code{chlorophyll(lat, lon)}, 
or additionally may also depend on time @code{chlorophyll(time, lat, lon)}. 
If a time-dependent pigment concentration is specified, the pigment concentration will be interpolated 
according to the option @code{time_interpolate}. All grids must be provided  in the file as
@code{double lat(lat)}, @code{double lon(lon)}, and @code{double time(time)}.
@item cox_and_munk_sal
Salinity for Cox and Munk ocean surface BRDF (in ppt); 
at present only available with @code{rte_solver DISORT2}.
The number of streams (@code{nstr}) is automatically increased
to 16 if cox_and_munk BRDF is switched on, to avoid numerical
problems. 
@item cox_and_munk_sal_map
Specify ocean salinity (in ppt) for the Cox and Munk 
ocean surface BRDF with the help of an @emph{netCDF} file, which is used in combinition 
with the options @code{latitude}, @code{longitude}, and @code{time}.
@example
cox_and_munk_pcl_map file [variable_name]
@end example
where @code{file} is the location of the @emph{netCDF} file. 
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.

The expected name of the pigment concentration variable is per default @code{salinity}, but can be changed with the
optional argument @code{variable_name}.
The pigment concentration must be provided as function of @code{latitude} and @code{longitude}, @code{salinity(lat, lon)}, 
or additionally may also depent on time @code{salinity(time, lat, lon)}. 
If a time-dependent salinity is specified, the salinity will be interpolated 
according to the option @code{time_interpolate}. All grids must be provided as 
@code{double lat(lat)}, @code{double lon(lon)}, and @code{double time(time)}.
@item cox_and_munk_u10
Wind speed for Cox and Munk ocean surface BRDF (in m/s); 
at present only available with @code{rte_solver DISORT2}.
The minimum allowed wind speed is 1 m/s because otherwise the
strong specular reflection causes numerical problems. If a lower value
is specified, the wind speed is automatically set to 1m/s. Also, the
number of streams (@code{nstr}) is automatically increased to 16 if
cox_and_munk BRDF is switched on, to avoid numerical problems.
@item cox_and_munk_u10_map
Specify wind speed (in m/s) for the Cox and Munk 
ocean surface BRDF with the help of an @emph{netCDF} file, which is used in combinition 
with the options @code{latitude}, @code{longitude}, and @code{time}.
@example
cox_and_munk_u10_map file
@end example
where @code{file} is the location of the @emph{netCDF} file. 
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.

The file must contain the elements of the wind vector @code{U10} and @code{V10}.
These must be specified as functions of @code{latitude} and @code{longitude} @code{U10(lat, lon)}, @code{V10(lat, lon)}, 
or additionally may also depent on time @code{U10(time, lat, lon)}, @code{V10(time, lat, lon)}. 
If the variable time is present in the file, the wind speed will be interpolated 
according to the option @code{time_interpolate}. All grids must be provided as 
@code{double lat(lat)}, @code{double lon(lon)}, and @code{double time(time)}.
@item crs_file
May be used to specify cross sections of ozone (O3), nitrogendioxide (NO2), 
bromine oxide (BRO), OCLO, or HCHO to be used instead of one of those coming with 
libRadtran. No temperature dependence may be specified. Use as follows:
@example
crs_file NO2 ../examples/no2_crs.dat
@end example
The @code{NO2} or @code{O3,BRO} or @code{OCLO} or @code{HCHO} must be specified to identify the
species for which the cross section applies.
The cross section file has two columns:
@table @code
@item 1. wavelength (nm)
@item 2. cross section (cm+2)
@end table
@item data_files_path
The path to the directory where all uvspec internal data files live, e.g.
the files that are in the subdirectories of the @file{data} directory
that comes with the uvspec distribution. The default is @code{../data/}.
@item day_of_year
@emph{Obsolete, but still supported. Please use @code{time} instead.}
Integer, to correct the calculated radiation quantities for the 
Sun-Earth distance for the specified Julian day (1-365). If not given, 
the Earth-Sun distance is 1 AU (i.e. equinox distances), 
that is, no correction is applied to the extraterrestrial 
irradiance @code{solar_file}.
@item deltam
Turn delta-M scaling on/off. Set to either @code{on} or @code{off}. Note 
that for the @code{rte_solver disort2} delta-M scaling is hardcoded to 
be always on.
@item dens_column
Set the total column of a density profile. The column is integrated between the 
user-defined altitude and TOA (top of atmosphere). The syntax is
@example
dens_column species column [unit]
@end example
where species is one of O3, O2, H2O, CO2, NO2, BRO, OCLO, or HCHO, see also @code{dens_file}.
The second argument is the total column value, and the optional third argument is the unit, 
in which the column is given. The unit can be DU (Dobson units) or CM_2 (molecules/cm2).  
The default units are DU for O3, and CM_2 for all other gases.
@example
dens_column NO2 1.2 DU
@end example
@item dens_file
Specify density profiles (or matrix, see below) of various traces gases to be included in the radiative 
transfer calculation. The entry of the input file looks like this:
@example
dens_file gas_species [unit] filename
@end example
At the moment following gas_species are included: ozone (O3), nitrogen dioxide (NO2), water vapor (H2O),
bromine oxide (BRO), chlorine dioxide (OCLO), formaldehyde (HCHO), and carbon dioxide (CO2). The gas species is identified 
by their abbrevations given in the parenthesis above.

Unit is an optional argument to define the unit of the density. The profiles can
be given in particles per cm3 (cm-3), in particles per m3 (m-3), as volume mixing ratio (vmr), as mass mixing 
ratio (mmr), or as relative humidity (only for water). The default for unit is cm-3.
The model expects a density file with two columns:
@table @code
@item 1. z(km)
Altitude above sea level in km.
@item 2. density of trace gas [in the specified unit]
The density of the trace gase
@end table
The height grid may be different from the @code{atmosphere file}. All densities inside the range 
of the dens_file are replaced. For all other heights the values from the atmosphere_file are
used. If the density is specified as -1 at a level, the value from atmosphere_file is used.

To scale the profile to a total column value use @code{dens_column}.

For airmass factor calculations it is for some species necessary to account for the 
variation of the profile with sza. This may be accomplished by specifying a @code{dens_file} 
in the following format:
@example
0.0       SZA1      SZA2 ...
z(1)    dens(1,1)    ...
z(2)      .           .
 .        .           .
@end example
where z(i) are the altitude levels above sea level in km, SZA is the solar zenith 
angle in degrees, and dens is the density [in the specified unit] of the trace gases as 
function of solar zenith angle and altitude. 
The matrix may only be specified for one specie. It may however be combined with profiles
of other species. For examples see the @code{examples} directory.
A density matrix can only be used in connection with @code{rte_solver sdisort}!
@ifset mystic
@item emissivity_map
@emph{This option is preliminary and still subject to change (no wavelength dependency jet)!}
Specify a wavelength independent emissivity with the help of 
an @emph{netCDF} file, which is used in combinition 
with the options @code{latitude}, @code{longitude}, and @code{time}.
@example
emissivity_map file [variable_name]
@end example
where @code{file} is the location of the @emph{netCDF} file. 
With the optional argument @code{variable_name} the name of the emissivity variable in 
the @emph{netCDF} file can be specified. (By default the expected name is @code{EMIS}.)
The emissivity must be specified as function of @code{latitude} and @code{longitude} @code{EMIS(lat, lon)}, 
or additionally may also depent on time @code{EMIS(time, lat, lon)}. 
All grids must be provided as 
@code{double lat(lat)}, @code{double lon(lon)}, and @code{double time(time)}.
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.
If the variable time is present in the file, the emissivity data nearest to the specified time 
will be selected (or interpolated if @code{time_interpolate} is switched on). 

@item ECMWF_atmosphere_file
Reads in combination with the options @code{latitude}, @code{longitude}, and @code{time} (all mandatory)
the pressure, temperature, ozone, and water vapour from an ECMWF netCDF data file and will
combine it with the data given by the @code{atmosphere_file}.
If time is between two dates given in the ECMWF file, the data will be linearly interpolated.
@item ECMWF_wc_file
Reads in combination with the options @code{latitude}, @code{longitude}, and @code{time} (all mandatory)
the pressure, temperature, and cloud liqid water content (CLWC) from an ECMWF netCDF data file.
@item ECMWF_ic_file
Reads in combination with the options @code{latitude}, @code{longitude}, and @code{time} (all mandatory)
the pressure, temperature, and cloud ice water content (CIWC) from an ECMWF netCDF data file.
@end ifset
@item f11_mixing_ratio
The mixing ratio of F11 in ppm (default: 0.000268 ppm).
@item f12_mixing_ratio
The mixing ratio of F12 in ppm (default: 0.000503 ppm).
@item f22_mixing_ratio
The mixing ratio of F22 in ppm (default: 0.000105 ppm).
@item filter_function_file
If specified, the calculated spectrum is multiplied with a filter function 
defined in @file{filter_function_file}. The file must contain two columns. 
Column 1 is the wavelength, in nm. Column 2 is the corresponding filter 
function value. Comments start with @code{#}. Empty lines are ignored.
In combination with @code{output sum} or @code{output integrate} this option 
is useful e.g. to calculate weighted irradiances or actinic fluxes or 
to simulate broadband or satellite observations.
@item fisot
Specifies that isotropic illumination is used at top-boundary instead
of beam source. Useful for those who want to calculate the reflectance 
for a homogeneous or inhomogeneous atmosphere. The intensity is still set by
@code{solar_file}.
@item flexstor
Output is in flexstor format. May not be combined with @code{header}.
Also, does not currently work when @code{umu} and/or @code{phi} is specified.
@item h2o_mixing_ratio
The mixing ratio of H2O in ppm. Scale the profile so that the mixing 
ratio at the user-define @code{altitude} assumes the specified value.
@item h2o_precip
Precipitable water in kg / m2 (which is approximately 1mm). 
The water vapor profile is scaled accordingly. The precipitable water 
is integrated from the user-defined @code{altitude} to TOA (top of atmosphere).
@item header
Output information on some of the input parameters. May not be combined
with @code{flexstor}.
@item heating_rate
Calculation of heating rates. Output is only provided at altitudes specified by @code{zout}.
To get heating rate profiles a number of altitudes must thus be specified.
Heating rates is the change of temperature with time in units of K/day.
For spectral calculations the default output is a matrix: 
@example
0.0        zout1          zout2 ...
lambda1    heating_rates  ...
lambda2      .
   .         .
   .         .
@end example
For integrated calculations (@code{output sum} or @code{output integrate}) the default output 
is in two columns with column 1 being the @code{zout}-altitudes and column 2 the heating rates.
The output of @code{heating_rate} can also be specified with the @code{output_user} option.
Note that heating rates are only well-behaved up to altitudes
for which the respective correlated-k options are valid. E.g. about 60 km for 
@code{Fu} and about 80 km for @code{Kato}, @code{Kato2}, @code{Kato2.96}, and @code{LOWTRAN}.
Attention: For spectral calculations, the extraterrestrial spectrum is assumed to be in 
mW/(m2 nm).

Two different methods are implemented to calculate the heating rate, which can be selected
with an optional keyword:
@example
heating_rate [method]
@end example
where @code{method} may be either @code{layer_cd} (heating rates are derived from centered 
differences of the flux, this is the default method) or @code{local} (heating rates are derived 
from the actinic flux).
@item n2o_mixing_ratio
The mixing ratio of N2O in ppm (default: 0.28 ppm).
@item ic_cloudcover
Set the fraction of the horizontal sky area which is covered by clouds.
When a cloud cover is specified, the result will be calculated by
the independent pixel approximation (IPA), that is, as weighted average 
of cloudless sky and overcast sky, where the cloud properties are taken 
from @code{ic_file}, etc. Please note that, if both @code{wc_cloudcover} and
@code{ic_cloudcover} are set, both must be equal.
@item ic_file
Location of file defining ice cloud properties. The file must
contain three columns. Column 1 is the altitude in km, column 2 
the ice water content in grams per cubic meter, and column 3 the effective
particle radius in micron. The ice water content and effectice particle
radius must be specified at the same altitude grid as 
in @code{atmosphere_file}. Note that the definition of cloud altitudes   
in @code{ic_file} refers to sea level, not to altitude above
ground. E.g., when altitude is set to 1.63km, and the first cloud level is 
defined at 3km, the cloud would start at 1.37km above ground.
The optical depth of a layer is calculated 
using information from the upper and lower levels defining the layer. 
Comments start with @code{#}. Empty lines are ignored. An example 
of an ice cloud is given in @file{examples/IC.DAT}. Per default, the cloud properties are 
interpreted as properties at a given altitude @emph{level}. If @code{ic_layer} is defined,
they are interpreted as @emph{layer} properties (please see the section about
water clouds for a realistic example how the contents of the
@code{ic_file} are converted to optical properties).
@item ic_files
A way to specify ice cloud optical depth, single scattering albedo, 
and phase function moments for each layer. The file specified by 
@code{ic_files} has two columns where column 1 is the altitude in km. The altitudes 
must be the same as those specified in the file @code{atmosphere_file}. The second
column is the name of a file which defines the optical properties of the level 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
Please note that the uppermost line of the @code{ic_files} denotes
simply the top altitude of the uppermost layer. The optical properties
of this line are consequently ignored. There are two options for this
line: either an optical property file with zero optical thickness is
specified or "NULL" instead.
@item ic_fu_tau
Specify if the Fu (1996) optical properties are delta-scaled or not. 
With @code{ic_fu_tau scaled} delta-scaling is switched on, with 
@code{ic_fu_tau unscaled} it is switched off. The default is with
delta-scaling, as suggested in the paper (equation 3.8 and
description; equations A.2a - A.2c and description). 
If you define a cloud only by its microphysical properties (ice water
content, effective radius), delta-scaling should certainly be switched
on and you do not need to read further. If, however, you want to
use the Fu (1996) parameterization in combination with one of 
@code{ic_set_tau/tau550/gg/ssa} or @code{ic_scale_gg/ssa} it might be
reasonable to switch delta-scaling off and you should make sure that
you understand the following. Citing from Fu (1996): "For nonspherical
particles in cirrus clouds, it is found that a simple representation
of the scattering phase function through the asymmetry factor is
inadequate (Fu and Takano 1994). As demonstrated in appendix A, the
fraction of scattered energy residing in the forward peak, f, needs to
be removed from the scattering parameters to incorporate the strong
forward peak contribution in multiple scattering." Or in other words,
the sharp forward peak is truncated and added to the unscattered
direct radiation. The remaining phase function (excluding the sharp
forward peak) can be safely approximated by a Henyey-Greenstein
function. The scaling implies a reduction of the optical thickness,
the asymmetry parameter, and the single scattering albedo. This
reduction can be rather severe, e.g. a factor of about 3 for the
optical thickness in the visible spectral range. This implies
seemingly inconsistent optical properties: For idential IWC content
and effective radius, @code{ic_properties key/yang} would give an
(unscaled) optical thickness about three times higher than
@code{ic_properties fu}. The effect on the radiation field, however,
will be comparable, due the consistent scaling of optical thickness,
asymmetry parameter, and single scattering albedo. If you, however,
adjust the optical thickness using e.g. @code{ic_set_tau}, the effect
on the radiation field will be completely different because the
(unscaled) optical thickness by Key (2002) has a completely different
meaning as the (scaled) optical thickness by Fu (1996). In such cases
it might be reasonable to switch scaling off. This is a complicated
and confusing topic and it is suggested that you play around a bit
with the options, read the Fu (1996) paper, and make heavy use of the
@code{verbose} feature.
@item ic_fu_reff
If @code{ic_fu_reff yang} is specified, the Fu (1996/98) parameterization
uses the same definition of the effective radius as the Key et al. (2002) 
and Yang et al. (2000) parameterization; see discussion of @code{ic_properties}.
@item ic_habit
Ice crystal habit for the Yang et al. (2000) and  Key et al. (2002)
parameterizations, see also  @code{ic_properties key/yang}. May be one of solid-column, 
hollow-column, rough-aggregate, rosette-4, rosette-6, plate, droxtal,
and spheroid. Please note that this parameterization is only valid for
a restricted size range, depending on the habit (see table 1 in 
Key et al. (2002)). Also, some of the habits are only available for
wavelengths below 5 micron (rosette-4) while others are only available
for wavelengths larger than 3 micron (droxtal, spheroid).
@item ic_ipa_files
A two-column file, defining ice cloud property files (see @code{ic_file}) in the first 
column and the correspoding weights in the second column. The radiative transfer 
calculation is performed independently for each cloud column and the result is 
the weighted average of all independent columns. If @code{ic_ipa_files} and 
@code{wc_ipa_files} are both defined, both must have the same columns in the same 
order, otherwise @code{uvspec} will complain.
@item ic_layer
Specify ice cloud properties for model @emph{layers} instead of @emph{levels} 
(see also @code{ic_file}). 
@item ic_properties
Defines how ice water content and effective particle radius are translated 
to optical properties. Possible choices are
@table @code
@item fu 
Parameterization by Fu (1996), Fu et al. (1998), see ic_file; 
this is the default setting. Note that this is a parameterization
which has been created to calculate fluxes but not radiances. 
Note also that the optical properties in the solar range provided by 
Fu (1996) are delta-scaled properties (that is, the forward peak of
the phase function is truncated and optical thickness, asymmetry
parameter, and single scattering albedo are reduced accordingly).
Please see the detailed discussion in the description of @code{ic_fu_tau}!
For wavelengths up to 4 micron Fu (1996) is used while for wavelengths 
larger than 4 micron Fu et al. (1998) is chosen. Please note that 
Fu (1996) is based on ray-tracing calculations while Fu et al. (1998)
is a mixture of ray-tracing and Mie calculations (which is required for 
the infrared wavelengths where the geometrical assumption does not hold).
Hence, both parameterizations are not fully consistent. Rather, differences
of some % are to be expected in the wavelength region where both 
parameterizations overlap. Also, the wavelength dependence in the solar 
and infrared parts is treated differently: In the solar part (Fu, 1996) 
the optical properties are defined for wavelength bands - hence they 
are assumed constant within each band. In the infrared (Fu et al. 1998) 
they are defined at certain wavelengths and linearely interpolated 
in between. If you use this option, please see also the
discussion of @code{ic_fu_tau} and @code{ic_fu_reff}.
The allowed range for the effective radius is from 9.315 -  65.120 micro meter. 
@item echam4
Use the very simple two-band parameterization of the ECHAM4 climate model, described 
in Roeckner et al. (1996); this is probably only meaningful if you want to compare
your results with ECHAM4, the two bands are 0.2 - 0.68 micron and 0.68 - 4.0 micron;
within these bands, the optical properties are assumed constant.
@item key
Parameterization by Key et al. (2002). This parameterization can also
be used to calculate radiances because it uses a
double-Henyey-Greenstein phase function which better represents both
forward and backward peaks. This parameterization covers the region  
from 0.2 to 5.0 micron and is available for the following habits:
solid-column, hollow-column, aggregate, rosette-4, rosette-6, and plate.
@ifset yang
@item yang
Parameterization similar to Key et al. (2002) but based on more recent
single scattering calculations. Below 3.4 micron it actually equals
the Key et al. (2002) parameterization while from 3.4 - 100 micron new 
coefficients have been calculated with much higher wavelength
resolution and better accuracy. Hence, yang should give a reasonably
consistent approximation from 0.2 - 5 micron, suitable for spectrally
resolved calculations of radiance and irradiance.
The covered range for the effective radius depends on the @code{ic_habit}.
(In micro meter: solid columns [5.96, 84.22], hollow columns [4.97, 70.24],
 rough aggregate [3.55, 108.10],rosettes-4 [2.77, 45.30], rosettes-6 [2.85, 46.01], 
plates  [4.87, 48.18], dendrites [0.45, 1.88], droxtals [9.48, 293.32], 
spheroids  [6.58, 203.39]). Please pay attention to the Yang's definition of the 
effective radius of the ice habit, see below.
@end ifset
@item baum
Use ice cloud parameterization from Brian Baum's webpage, 
http://www.ssec.wisc.edu/~baum/Cirrus/IceCloudModels.html. The phase function
is described by 100 Legendre coefficients, calculated with delta-fit from 
Hu et al. (2000). This parameterization covers the region  
from 0.4 to 2.2 micron. If very high accuracy is needed in the vicinity of the halo, 
the forward peak, or the backscatter peak, @code{ic_properties baum_detailed}
is recommended.
@item baum_detailed
See @code{ic_properties baum}; the phase function is described with 10,000 
Legendre coefficients which guarantees a more accurate representation 
of the Halo etc. For this option you need to obtain the respective tables 
separately (100 MBytes!).
@item mie
Use pre-calculated Mie tables; useful for @code{correlated_k}; 
the tables are expected in @code{data_files_path}/correlated_k/../ @*
For spectral or pseudo-spectral (@code{correlated_k sbdart}) calculations, 
a set of pre-calculated tables is also available; the wavelength grid points of these 
data has been carefully selected such that the extinction cross section, 
single scattering albedo, and the asymmetry parameter are accurate to 1% 
(compared to the fully-resolved Mie calculation) for all wavelengths
between 250nm and 100 micron. For spectral or pseudo-spectral
calculations @code{wc_properties_interpolate} has to be defined explicitely to
initiate the interpolation of the optical properties to the internal wavelength grid.
Please note that this option may be extremely memory-consuming because for each 
internal wavelength a full set of Legendre moments of the phase function is 
stored (up to several thousands). The Mie tables are not part of the standard distribution 
(because of their large size) but they are freely available from http://www.libradtran.org. 
Note that a Mie calculation assumes spherical ice particles, the scattering function of 
which differs systematically from non-spherical particles. Hence, @code{ic_properties mie}
is usually not representative of natural ice clouds.
@item filename
Read optical properties from specified filename; file format is as produced 
by the mie-tool of the @code{libRadtran-tool} box (see @code{output_user cloudprp}) 
or by Frank Evans' @code{cloudprp}; for each of the internal (computational) wavelengths, 
a separate file is expected (this option is subject to change).
@end table
Please note also that, in contrast to spherical particles, there is no unique 
definition of effective size for non-spherical particles. In particular, the
above parameterizations use different definitions which, however, differ only by 
a constant factor. 
@iftex
 @tex
  Yang et al. (2000), Key et al. (2002) use the general definition 
  $$ r_{\rm eff} = {{3}\over{4}}{{\int V(h) n(h) dh}\over{\int A(h) n(h) dh}}$$
  where h is the maximum dimension of an ice crystal, n(h) is the
  number of particles with maximum dimension h in the size distribution,
  and V and A are the volume and mean
  projected area of the particles, respectively. The volume and area are
  based on the spherical diameter with equivalent volume and the
  spherical diameter with equivalent projected area as defined by 
  Yang et al. (2000). On the other hand, Fu et al. (1996,1998) use
  hexagonal columns and use the following definition
  $$ r_{\rm eff} =  {{\int D^2 L n(L) dL}\over{2 \int (D L + {\sqrt{3}\over{4}} D^2) n(L) dL}}$$
  where D is the width of the ice crystal (that is, the maximum diameter of the 
  hexagonal area) and L is the length. The integrand in the numerator is proportional 
  to the volume while that in the denominator is proportional to the projected area. 
  Evaluating these formulas one finds that, for the same hexagonal particle, the effective
  radius would be $3 \sqrt{3} / 4 = 1.299$ times larger following the 
  Yang et al. (2000), Key et al. (2002) definition than the Fu (1996,1998) definition.
  As an example, an effective radius of 20$\mu m$ with "ic\_properties fu" and
  1.299 $\cdot$ 20$\mu m$ = 26$\mu m$ with "ic\_properties yang" would give comparable results
  for hexagonal columns. 
 @end tex
@end iftex
@ifinfo
@end ifinfo
To use consistent definitions of the effective radius in both parameterizations, use
@code{ic_fu_reff yang}.
@item ic_properties_interpolate
Interpolate ice cloud optical properties over wavelength; useful for precalculated 
optical property files defined with @code{ic_properties}. Please note that this option 
may be extremely memory-consuming because for each internal wavelength a full set of 
Legendre moments of the phase function is stored (up to several thousands). 
@item ic_saturate
With this option, the relative humidity inside ice clouds can easily be adjusted 
to a user-defined value, e.g. saturated with respect to water. 
This option has one nessesary and one optional argument:
@example
ic_saturate switch [relative_humidity]
@end example
where @code{switch} is @code{on}, @code{off}, or @code{ipa}. 
The second optional argument determines the relative humidity in % 
(with respect to water!) inside the cloud. The default value is 100.
ipa is only relevant for independent column calculations. 

Details for independent column simulations: Using @code{switch} @code{on}, the air 
in @emph{all} columns will be saturated, if there is a cloud in at least 
one of the columns (this option should be used for stratiform clouds).
Using @code{switch} @code{ipa}, only cloudy columns are 
affected (this option should be used for convective cloud fields.)
@item ic_scale_gg
Scale the ice cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0. If you use this option in combination with the ice
cloud properties by Fu (1996), please make sure that you understand
the explanation of @code{ic_fu_tau}.
@item ic_scale_ssa
Scale the ice cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0. If you use this option in combination with the ice
cloud properties by Fu (1996), please make sure that you understand
the explanation of @code{ic_fu_tau}.
@item ic_set_gg
Set the ice cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0. If you use this option in combination with the ice
cloud properties by Fu (1996), please make sure that you understand
the explanation of @code{ic_fu_tau}.
@item ic_set_ssa
Set the ice cloud single scattering albedo for all wavelengths
and altitudes to a value between 0.0 and 1.0. If you use this option
in combination with the ice cloud properties by Fu (1996), please make
sure that you understand the explanation of @code{ic_fu_tau}.
@item ic_set_tau
Set the total ice cloud optical depth to a constant value for all wavelengths.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). This option is useful 
only for monochromatic calculations or in wavelength regions 
where the optical properties of ice clouds can be considered constant, e.g. the 
ultraviolet region. If you use this option in combination with the ice
cloud properties by Fu (1996), please make sure that you understand
the explanation of @code{ic_fu_tau}.
@item ic_set_tau550
Set the ice cloud optical thickness at 550nm. Other wavelengths are scaled accordingly.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). 
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wavelength} does contain 550nm.
If you use this option in combination with the ice
cloud properties by Fu (1996), please make sure that you understand
the explanation of @code{ic_fu_tau}.
@item include
Include a file into the uvspec input. Works exactly like the C @code{#include} or 
the Fortran @code{INCLUDE} statements.
@ifset mystic
@item mc_absorption
Calculate MYSTIC absorption of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo}.
@item mc_actinic
Calculate MYSTIC actinic flux for each grid box by dividing the absorbed energy by the 
absorption coeffcient; this method is much better than the traditional photon counting 
which usually comprises spikes (because in the latter method each photon is weighted with 
1/cos(theta) which may be a very large number); may need a considerable amount of memory, 
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo}.
@item mc_albedo_file
Define a MYSTIC 2D albedo input file (default: albedo2D.dat).
The expected format of the elevation file is:
@example
Nx  Ny  dx  dy
ix  iy  albedo
@end example
where Nx and Ny are the number of grid boxes in x- and y-direction,
dx and dy are the size of the grid boxes in km.
In the second and the following lines the indices in x- and y-direction
and the albedo of the box are specified. 
@item mc_backward
Backward tracing of photons. @code{mc_backward} takes either two or four coordinates:
either the indices of the sample pixels to be calculated (0-Nx-1, 0-Ny-1), or the 
range of coordinates to be sampled (lower left and upper right). @code{mc_backward} 
currently computes radiances and downward diffuse irradiances. 
@item mc_backward_output 
Specify quantity to be calculated using backward Monte Carlo. So far the following options are implemented: Downwelling flux @code{edn}, upwelling flux @code{eup}, actinic flux @code{f} and heating rate @code{heating}. Per default the downwelling flux is calculated. If heating is selected the absorption is calculated, in the thermal region, emission needs to be calculated separately using the option @code{mc_emission} and afterwards the emission values have to be substracted from the absorption.  
@item mc_backward_writeallpixels
If set, write all pixels to the output files; otherwise, only those are written 
which are actually calculated.
@item mc_basename
Filename for MYSTIC 3D output (default: mc).
@item mc_cloud_grid
3D cloud grid size (Nx,Ny,Nz) for MYSTIC. Weird, but required to keep MYSTIC 
a function that can be called from a Fortran program.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_ambrals_file
Define a MYSTIC 2D AMBRALS BRDF input file. AMBRALS is the 
Algorithm for Modeling[MODIS] Bidirectional Reflectance Anisotropies 
of the Land Surface by Wanner et al. (1995, 1997). See also 
http://www-modis.bu.edu/brdf/models.html. The file consists of a header 
followed by the three parameters iso, vol, and geo for each pixel.
@item mc_azimuth_old
Use old MYSTIC azimuth convention (0 degree = "looking from the
direction of the sun; 180 degree = looking into the direction of the
sun; that is, exactly opposite to the disort convention). The MYSTIC
azimuth was changed March 1, 2004 - hence this option was introduced
for compatibility reasons.
@item mc_elevation_file
Define a MYSTIC 2D elevation input file (default: elevation2D.dat). The 
expected format of the elevation file is:
@example
Nx  Ny  dx  dy
ix  iy  elevation
@end example
where Nx and Ny are the number of grid boxes in x- and y-direction,
dx and dy are the size of the grid boxes in km.
In the second and the following lines the indices in x- and y-direction
and the elevation in km at this point are specified.

Attention: While the other files refer to grid boxes,
the elevation is defined at grid points. It has to be this 
way because each "elevation pixel" contains a surface 
which is defined by the four cornes of the pixel. If the
grid covers an area of 200x200 km^2 and the pixel sizes 
are dx = 1km and dy = 1km, the elevation has to be defined 
at 201x201 points (ix = 1 .. Nx, iy = 1 .. Ny).
@item mc_emission
Calculate MYSTIC emission of each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Note that emission is calculated directly without tracing 
any photons. Increasing @code{mc_photons} will therefore not change the result.
Only meaningful with @code{rte_solver montecarlo} and @code{source thermal}. 
@item mc_escape
Calculate MYSTIC radiances via escape probabilities; slows down the tracing 
but usually requires much less photons. Only meaningful with @code{rte_solver montecarlo}.
@item mc_heating
Calculate MYSTIC heating rates for each grid box; may need a considerable amount of memory,
depending on the 3D cloud grid. Only meaningful with @code{rte_solver montecarlo. @code{mc_heating}
is identical to @code{mc_absorption} for @code{source solar}. For @code{source thermal}, however, 
the emission of a photon in a grid box is counted as cooling}. 
@item mc_icloud_file
Define a MYSTIC 3D ice cloud input file.
The format of the 3D ice cloud file is identical to the 3d water cloud file 
(see mc_wcloud_file), except that 
gspec needs to be 3, that is, the file must contain ice water 
content and particle effective radius. The conversion to optical
properties is done with the parameterization of Yang/Key/Mayer.
@item mc_ipa
Run MYSTIC in independent pixel mode. Only meaningful with @code{rte_solver montecarlo}.
@item mc_momcld_file
Does not exist anymore as 3D cloud properties are now handled with 
wc_properties. 
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_pcld_file
Cloud phase function file for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons
Total number of photons to be traced by the Monte Carlo solver, MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_photons_file
Distribution of photons over wavelength bands; to be used with @code{correlated_k};
for an example see @file{data/correlated_k/kato/photons.dat}. No error checking yet!
Do only use if you are absolutely sure what you are doing.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_radial_pathlength
I3RC case 7, laser beam experiment. Radiances are sampled in radial 
and pathlength elements. Specify the number of radial (Nr) and time (Nt) intervals. 
The radius increment is calculated from Nr and the domain size. The time interval 
may  be specified with @code{mc_radial_pathlength_dt}. 
@item mc_radial_pathlength_dt
Specify time increment for @code{mc_radial_pathlength}. Time is converted to pathlength 
assuming a speed of light of 3E8 m/s
@item mc_rpv_file
Define a MYSTIC 2D RPV BRDF input file.
@item mc_rpv_type
File containing wavelength-dependent RPVs to be used in combination with @code{mc_rpv_file}.
@item mc_sample_grid
Sampling grid size (Nx,Ny) for MYSTIC.
Only meaningful with @code{rte_solver montecarlo}.
@item mc_randomseed
Provide your own random seed (positive integer) for the random number generator. 
Usually a random seed is determined from current time plus process id. 
This option is useful to re-run a simulation and for debugging.
@item mc_spherical
Spherical geometry in MYSTIC. Works only in "1D" - @code{wcloud_file} 
and @code{icloud_file} are not yet considered. If @code{mc_spherical} is selected
@code{mc_backward} is switched on automatically. 
@item mc_std
Calculate standard deviation of the average.
@item mc_surface_reflectalways
Usually, a photon is either absorbed or reflected at the surface, 
with a probability defined by the surface albedo. If 
@code{mc_surface_reflectalways} is specified, each photon is reflected and 
the albedo is considered by reducing the photon weight. In case of BRDF,
@code{mc_surface_reflectalways} is switched on automatically because the 
other method is no longer implemented for non-Lambertian BRDFs, due to 
implementation and numerical problems. For small albedos, the computational 
time is increased if @code{mc_surface_reflectalways} is used; however, 
the accuracy of the upward radiance (reflected by the surface) is increased 
considerably. In case of clouds, however, computational time might be 
increased considerably without gaining accuracy. 
@item mc_surfaceparallel
Calculate irradiance parallel to the surface instead of horizonal irradiance. This
option is obviously only interesting for topograpy. 
@item mc_truncate
Truncate phase function at the specified polar angle mu. 
USE ONLY IF YOU REALLY KNOW WHAT YOU ARE DOING!
@item mc_wcloud_file
Define a MYSTIC 3D water cloud input file (default: wcloud3D.dat).
The expected format of the cloud file is:
@example
Nx  Ny  Nz  gspec wspec
dx  dy  z(bottom) z(2) ... z(top)
ix  iy  iz  ext           (if gspec == 0)
ix  iy  iz  ext  g        (if gspec == 1)
ix  iy  iz  ext  r_eff    (if gspec == 2)
ix  iy  iz  LWC  r_eff    (if gspec == 3)
ix  iy  iz  xxx  yyy    w (if wspec == 1)
@end example
where Nx, Ny and Nz are the number of grid boxes in x, y, and z-direction.
gspec and wspec are parameters, which determin the format of the 3rd and 
following lines. In the second line dx and dy are the sizes of the boxes in x- 
and y-direction in km.
In the third and following lines the indices ix, iy, and iz specify, where the cloud is located. 
The optical properties of the cloud, are given by the other parameters in the line, 
where ext is the extinction coefficient [1/km], g the assymetry parameter,
r_eff the effective radius [micro m], and w the single scattering albedo.
@end ifset
@item molecular_tau_file
Location of molecular absorption optical depth file. Usually, molecular absorption
is calculated from trace gas concentrations provided in @code{atmosphere_file}
(scaled with @code{ozone_column}, etc. Use this option only if you want to specify
the optical depth directly (e.g. for a model intercomparison) or for a line-by-line
calculation. If a spectral @code{molecular_tau_file} is specified, the wavelength 
grid defined there is used as internal wavelength grid for the radiative transfer
calculation, if not defined otherwise with @code{transmittance_wl_file}.
@code{molecular_tau_file} can be either of the following three formats:
@table @asis
@item Monochromatic:
Column 1 is the altitude in km where the altitude grid must be exactly equal 
to the altitude grid specified in @code{atmosphere_file}. 
Column 2 is the absorption optical depth of each layer.
@item Spectral, ASCII:
The first line contains the level altitudes in decreasing order; the following lines 
contain the wavelength [nm] in the first column and then the absorption optical depths
of each layer. 
@item Spectral, netcdf:
netcdf is a common platform independent format; the description, a library to read and 
write netcdf including some tools to generate netcdf is available at 
http://www.unidata.ucar.edu/packages/netcdf/. A @code{molecular_tau_file} must obey 
certain rules; an example is available at the libRadtran homepage, 
@file{UVSPEC.O2A.afglms.cdf}, a line-by-line spectrum of the oxygen A-Band 
around 760nm, calculated for the mid-latitude summer 
atmosphere by Anderson et al. (1986). The advantage of
netcdf compared to ASCII is that it is much faster to read, and that the file 
is a self-contained, including data and a description of the variables and arrays.
It is therefore particularly useful for line-by-line calculations where usually 
many spectral data points are involved.
@end table
Comments start with @code{#}. Empty lines are ignored.
@item no_absorption
Switch all (molecular, aerosol, cloud, and ice cloud) absorption off. Please 
note that this option simply sets the absorption optical thickness to 0. 
If used together with xxx_set_tau this might be a bit confusing but 
probably the most logical way. E.g. when using @code{aerosol_default} and 
@code{aerosol_set_tau 1}, the aerosol optical thickness is set to 1,
with 0.940539 scattering and 0.059461 absorption. If @code{no_absorption} 
is added, the absorption optical thickness is set to 0 while the scattering
optical thickness is preserved at 0.940539 (even though 1 was specified by 
the user). We find this the most logical solution of the problem because 
by switching @code{no_absorption} off and on one tests the effect of the 
absorber in an isolated way, rather than mixing absorption and scattering. 
The same is true for water and ice clouds. Note, that thermal emission of 
molecules is also switched off. 
@item no_molecular_absorption
Switch off molecular absorption.
@item no_rayleigh
Switch Rayleigh scattering off.
@item no_scattering
Switch scattering off.
@item no2_column_du
Obsolete, use @code{dens_column} instead.
Set the NO2 column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
Dobson units. If value is negative or @code{no2_column_du}
is not specified, the NO2 column is not scaled.
@item no2_column_moleccm-2
Obsolete, use @code{dens_column} instead.
Set the NO2 column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
molecules / cm2. If value is negative or @code{no2_column_moleccm-2}
is not specified, the NO2 column is not scaled.
@item nscat
The order of scattering for the @code{sos} radiative transfer equation solver. 
Default is 20. May also be used with the @code{sdisort} solver. If set to 1 @code{sdisort} 
will run in single scattering mode while if set in to 2, @code{sdisort} runs in full multiple 
scattering mode.
@item nrefrac
For the @code{rte_solver sdisort} refraction may be included by specifying 
@code{nrefrac}. If refraction is included also set parameter 
@code{refraction_file}.
@table @code
@item 0
No refraction, default.
@item 1
Refraction included using fast, but harsh method.
@item 2
Refraction included using slow, but accurate method.
@end table
@item nstr
Number of streams used to solve the radiative transfer equation.
Default is 6 for fluxes and 16 for radiances.
@item o2_mixing_ratio
The mixing ratio of O2 in ppm; scale the profile so that the mixing 
ratio at the user-defined @code{altitude} assumes the specified value.
@item o3_crs
Choose between the following ozone cross sections.
@table @code
@item Bass_and_Paur
Bass and Paur ozone cross section.
@item Molina
Molina and Molina (1986) ozone cross section.
@item Daumont
Ozone cross section by Daumont et al. (1992), Malicet et al. (1995).
@end table
Molina and Molina is default.
@item ozone_column
Obsolete, use @code{dens_column} instead.
Set the ozone column to a given value. The column is integrated between the 
user-defined @code{altitude} and TOA (top of atmosphere). The value must be in 
Dobson units. If value is negative or @code{ozone_column}
is not specified, the ozone column is not scaled.
@item output
Output processing. Choose between the following options:
@table @code
@item sum
Sum output over wavelength. Useful in combination with the 
@code{correlated_k} option (Kato, Kato2, Kato2.96, Fu, AVHRR_KRATZ).
@item integrate
Integrate output over wavelength for solar and over wavenumber for thermal simulations. 
Useful for spectral calculations and @code{correlated_k} LOWTRAN.
@item per_nm 
Output is given in W/(m2 nm) or mW/(m2 nm) (W or mW is determined by the extraterrestrial spectrum.)
@item per_cm-1 
Output is given in W/(m2 cm-1) or mW/(m2 cm-1).
@item per_ck_band 
Output is given in W/m2 or mW/m2 per correlated-k band. (This option can not be used for 
spectral calculations and @code{correlated_k} LOWTRAN in the solar range.)
@item none
No processing - output spectral information (default).
@end table
@ifset mystic
@item output_format
Specification of the output format.
@example
output_format format
@end example
where @code{format} is either @code{ascii} (default) or @code{flextor}.
There is also the possibility to write @code{uvspec} simulation results to an existing @emph{netCDF} file 
@example
output_format format output_file
@end example
where @code{format} is @code{netCDF} if the @code{output_file} contains a lat/lon/time grid 
or @code{sat_picture} if the @code{output_file} contains a pixel_x/pixel_y/time grid.
@end ifset 
@item output_user
User defined output.(This option is case sensitive.) Here the user may specify the columns 
desired for output. Default output is the 
wavelength lambda, edir, edn, eup, uavgdir, uavgdn, uavgup 
for disort, sdisort, and spsdisort, whereas the default for 
twostr is lambda, edir, edn, eup, uavg.
The lines containing radiances and the output of @code{rte_solver polradtran} are not affected. 
@table @code
@item lambda
Wavelength in nm.
@item wavenumber
Wave number in cm-1.
@item sza
solar zenith angle
@item zout
Output altitude in km.
@item edir, eglo, edn, eup, enet, esum
Direct, global, diffuse downward, and diffuse upward irradiance.
Net is global - upward, sum is global + upward. 
@item fdir, fglo, fdn, fup, f
Direct, global, diffuse downward, diffuse upward, and total actinic flux.
@item uavgdir, uavgglo, uavgdn, uavgup, uavg
Direct, global, diffuse downward, diffuse upward, and total diffuse 
mean intensity (= actinic flux / 4 pi).
@item albedo
Albedo.
@item heat
Heating rate in K/day.
@end table
There are also some atmospheric output variables, which might be useful.
@table @code
@item p
pressure [hPa], ,
@item T, T_d
temperature [K], dewpoint temperature [K]
@item theta 
potential temperature [K]
@item theta_e
equivalent potential temperature [K]
@item n_xxx
number density of the gas xxx [cm-3]
@item rho_xxx
mass density of the gas xxx [kg/m3]
@item mmr_xxx
mass mixing ratio of the gas xxx [kg/kg]
@item vmr_xxx
volume mixing ratio of the gas xxx [m3/m3]
@item rh
relative humidity over water [per cent]
@item rh_ice
relative humidity over ice   [per cent]
@item c_p
specific heat capacity of the air (humidity and temperature dependent) 
@end table
where @code{xxx} is one of 
O3, O2, H2O, CO2, NO2, BRO, OCLO, or HCHO.
@item phi
Azimuth output angles (in degrees) in increasing order. 
The radiance is output at @code{phi} and @code{umu}.
@example
Sensor in the North (looking South)   0 deg
Sensor in the East  (looking West)   90 deg
Sensor in the South (looking North) 180 deg
Sensor in the West  (looking East)  270 deg
@end example
@item phi0
Azimuth angles of the sun (0 to 360 degrees). 
@example
Sun in the South    0 degrees
Sun in the West    90 degrees
Sun in the North  180 degrees
Sun in the East   270 degrees
@end example
If @code{phi0} varies as a function of wavelength use 
@code{sza_file}.
@item polradtran_aziorder
Order of Fourier azimuth series: 0 is azimuthally symmetric case.
Default 0.
@item polradtran_max_delta_tau
Initial layer thickness for doubling; governs accuracy, 10E-5 should be
adequate. Do not go beyond half the real precision, i.e. 10e-8 for REAL*8.
Default 1.e-05.
@item polradtran_nstokes
Number of Stokes parameters
@table @code
@item 1 
for I (no polarization, default)
@item 2
for I,Q
@item 3
for I,Q,U
@item 4 
for I,Q,U,V
@end table
Default is 1.
@item polradtran_quad_type
Type of quadrature used: 
@table @code
@item G (default)
gaussian
@item D
double gaussian, 
@item L
Lobatto
@item E
extra-angle(s), this must be used of @code{polradtran} is used in 
combination with @code{umu}. Will internally use Gaussian scheme (@code{G}). 
See also radtran documentation.
@end table
Default G.                                  
@item polradtran_src_code
Radiation sources included:
@table @code
@item 0
none
@item 1
solar
@item 2
thermal
@item 3
both
@end table
Default 1.
@item pressure
The surface pressure (at the user-defined @code{altitude}) in hPa. 
The pressure profile as well as air, O2 and CO2 density profiles 
are scaled accordingly.
@item pressure_out
Specify the output levels in pressure coordinates. The syntax is  
@example
pressure_out p1 p2 ...
@end example
where '@code{p1 p2 ...}' are the output levels in hPa. 
The pressure output levels must be sorted in decreasing order. 
Output pressure levels must be within the range defined in the
@code{atmosphere_file}. 
@item prndis
Specify one or more integers between 1 and 7.
Print various disort input and output in disorts own format. See 
@file{disort.doc} for more information.
@strong{Warning:} Produces a lot of output. 
@item quiet
If specfied, informative messages are turned off. See also @code{verbose}.
@item radiosonde
This option allows to change the temperature and pressure profile, and optionally to 
specify one or more density profiles. The entry in the input file looks like this:
@example
radiosonde filename [gas_species] [unit] ...
@end example
Currently the following gas_species are included: ozone (O3), nitrogen dioxide (NO2), 
water vapor (H2O), bromine oxide (BRO), chlorine dioxide (OCLO), formaldehyde (HCHO), and carbon dioxide (CO2). 
Each gas species is identified by its abbrevations given in parentheses above.

Unit is an optional argument to defines the unit of the density. The profiles can
be given in particles per cm3 (CM-3), in particles per m3 (M-3), as volume mixing ratio (VMR), as mass mixing 
ratio (MMR), or as relative humidity (RH) (only for water). The default unit is RH for water vapour, 
MMR for ozone, and CM-3 for all other gases.

The radiosonde file must have (2 + number of gases) columns:
@table @code
@item 1  
pressure in hPa
@item 2  
temperature in Kelvin
@item 3, 4 ... 
density of trace gas in the specified unit
@end table
A new z-grid will be calculated, starting at @code{altitude} and assuming a linear temperature variation
between levels. The air density will be recalculated according to the ideal gas law, and the density of 
the well mixed gases O2 and CO2 will be scaled accordingly.
The atmospheric data above the radiosonde data is taken from the @code{atmosphere_file} level by level, starting 
at the first pressure level above the radiosonde data. The z-grid of the @code{atmosphere file} in 
this height region is shifted accordingly.
Also if the density in the radiosonde file is specified as -1 at a level, 
the value from the @code{atmosphere_file} is used.

Possible calls are
@example
radiosonde ../examples/radiosonde.dat
@end example
just in order to change the temperature and pressure profile, or
@example
radiosonde ../examples/radiosonde2.dat H2O RH O3 MMR NO2
@end example
where water vapour density will be given as relative humidity, ozone as mass mixing ratio, 
and NO2 in cm-3 (default).

@item rayleigh_crs
Choose between the following Rayleigh scattering cross sections.
@table @code
@item Bodhaine
Bodhaine et al (1999) Rayleigh scattering cross section.
@item Nicolet
Nicolet (1984) Rayleigh scattering cross section.
@end table
Bodhaine et al. is default.
@item rayleigh_depol
Rayleigh depolarization factor; the Rayleigh scattering phase function is
p(mu) = a + b * mu**2 where a = 1.5*(1+depol)/(2+depol) and  
b = 1.5*(1-depol)/(2+depol). By default the depolarization is calculated
using the expressions from Bodhaine et al. (1999).
@item rayleigh_tau_file
Location of Rayleigh scattering optical depth file. Usually, the Rayleigh scattering
cross section is calculated from the air pressure provided in @code{atmosphere_file}
(scaled with @code{pressure}). Use this option only if you really want to specify
the optical depth directly (e.g. for a model intercomparison). The
optical thickness profile may be either monochromatic or spectral. 
The format is exactly the same as for @code{molecular_tau_file}.
@item reflectivity
Calculate transmission / reflectivity instead of absolute quantities. 
For irradiances / actinic fluxes the transmission T is defined as 
@iftex
 @tex
  $$ T = {{E}\over{E_0 \cos{\theta}}}$$
  where $E$ is the irradiance / actinic flux, $E_0$ is the extraterrestrial flux,
  and $\theta$ is the solar zenith angle.
 @end tex
@end iftex
@ifinfo
 T = E / (E0 cos(theta)) @*
 where E is the irradiance / actinic flux, E0 is the extraterrestrial flux,
 and theta is the solar zenith angle.
@end ifinfo
The reflectivity R is defined as 
@iftex
 @tex
  $$ R = {{\pi \cdot L}\over{E_0 \cos{\theta}}}$$
 where $L$ is the radiance, $E_0$ is the extraterrestrial flux,
 and $\theta$ is the solar zenith angle.
 @end tex
@end iftex
@ifinfo
 R = pi L / (E0 cos(theta)) @*
 where L is the radiance, E0 is the extraterrestrial flux,
 and theta is the solar zenith angle.
@end ifinfo
Obviously, reflectivities do not depend on Sun-Earth distance. Please 
note the difference to @code{transmittance}.
@item reverse
Option for the strong and bold. Reverses the atmospheric input to the radiative
transfer solvers. That is, the atmosphere is turned on the head.
Yes, that is actually useful for some purposes. If you think
you need this contact the author. Otherwise, do not use.
@item rh_file
File that defines a profile of relative humidity. If specified, the water vapour 
profile in atmosphere_file is over-written. If -1 is specified at a level, the value 
from atmosphere_file is used.
@item rpv_file
4 column file, containing the Rahman, Pinty, and Verstraete (RPV) BDRF
parameterization, Rahman et al. (1993).
Bidirectional reflectance distribution functions for a variety of surfaces are 
given in the paper. This option in only supported with DISORT 2.0 and
MYSTIC. The columns of the input file are wavelength [nm], rho0, k, and theta. 
The parameters are interpolated linearely to the internal wavelength grid. 
To make sure that the results are reasonable, specify the RPV data on a wavelength 
grid similar or equal to that used internally for the radiative transfer calculation!
Optionally, a fifth column with a constant scaling factor may be defined which, however,
is only used by @code{rte_solver disort2}.
@item rpv_k
Constant RPV rho0, see @code{rpv_file}. @code{rpv_k} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_rho0 
Constant RPV rho0, see @code{rpv_file}. @code{rpv_rho0} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_theta
Constant RPV theta, see @code{rpv_file}. @code{rpv_theta} overwrites the wavelength-dependent
value defined in @code{rpv_file}.
@item rpv_sigma
Constant RPV sigma, to be used for snow (Deguenther and Meerkoetter, 2000).
A wavelength dependent sigma is not yet available.
@item rpv_t1
Constant RPV t1, to be used for snow (Deguenther and Meerkoetter, 2000).
A wavelength dependent sigma is not yet available.
@item rpv_t2
Constant RPV t2, to be used for snow (Deguenther and Meerkoetter, 2000).
A wavelength dependent sigma is not yet available.
@item rpv_scale
Apply a constant scaling factor for the RPV BRDF. Required e.g. if the the albedo
should be set to a certain value. This factor is only used by @code{rte_solver disort2}.
@item rpv_library
The rpv libraries are collections of spectral BRDFs of different surface types, 
This option must be used either with @code{surface_type} 
or @code{surface_type_map}, in order to select the specific surface type.

For using a @code{rpv_library} write 
@example
rpv_library library_path 
@end example
where @code{library_path} is the path of the directory, where the BRDF data is stored. The files are 
expected to have the names @code{IGBP.01.rpv, IGBP.02.rpv, ...} If @code{surface_type 1} is specified
the BRDF from @code{IGBP.01.rpv} will be used, and so on.
Each file must have the structure like an @code{rpv_file}.
(This option is quite the same as @code{rpv_file}, except that it offers you an
easy way to use the option @code{surface_type_map} in combinition with your @code{rpv_files}.)

@ifset mystic
@emph{For our group there is also a built-in library for IGBP surface types.}
@example
rpv_library IGBP
@end example
The built-in library contains the first 17 surface types see @code{albedo_library}.
The data is given for the wavelength 443nm, 565nm, 670nm, and 865nm. Stay near this wavelength
in order to get reasonable results. In future this the rpv-library will be
NDVI dependent, but until now the most common NDVI class is selected automatically.
@end ifset
@item rte_solver
Set the radiative transfer equation solver to be used. Options are
@vtable @code
@item disort
The standard plane--parallel disort algorithm by Stamnes et
al. (1988), version 1.3. For documentation see @file{src_f/DISORT.doc} as well as  the
papers and the DISORT report at @*
ftp://climate.gsfc.nasa.gov/pub/wiscombe/Multiple_Scatt/. To optimize for computational time 
and memory, please adjust the parameters in src_f/DISORT.MXD for your application and re-compile. 
@item disort2
Version 2 of disort. For documentation see @file{src_f/DISORT2.doc} as well as  the
papers and the DISORT report at @*
ftp://climate.gsfc.nasa.gov/pub/wiscombe/Multiple_Scatt/.
disort2 has several improvements compared to its 'ancestor' disort
1.3. Hence we recommend to use disort2 rather than the older version.
To optimize for computational time 
and memory, please adjust the parameters in src_f/DISORT.MXD for your application and re-compile. 
@item sdisort
Pseudospherical disort as described by Dahlback and Stamnes (1991). Double precision
version. To optimize for computational time and memory, please adjust the parameters 
in src_f/DISORT.MXD for your application and re-compile. 
@item spsdisort
Pseudospherical disort as described by Dahlback and Stamnes (1991), single precision  
version. @strong{Warning:} it is not recommended to use spsdisort for
really large solar zenith angles nor for cloudy conditions. For large
optical thickness it is numerically unstable and may produce wrong
results. To optimize for computational time and memory, please adjust
the parameters in src_f/DISORT.MXD for your application and re-compile. 
@item polradtran
The plane-parallel radiative transfer solver of Evans and Stephens (1991).
Includes polarization. Note that polarization effects of aerosols and 
clouds are not yet fully included. In case of doubt, please contact the authors. 
@item twostr
The two--stream radiative transfer solver described by Kylling et al. (1995),
in pseudo-spherical geometry.
@item twostrpp
The two--stream radiative transfer solver described by Kylling et al. (1995),
in plane-parallel geometry.
@item sos
A scalar pseudospherical succesive orders of scattering code. Works for 
solar zenith angles smaller than 90 degrees. Can calculate azimuthally 
averaged radiances. Set @code{nscat} to specify the order of scattering.
@item montecarlo
The MYSTIC code, see http://www.bmayer.de/mystic.html. Note that
MYSTIC is not part of the libRadtran distribution at
present. However, it has been given to some users on a collaborative basis.
@ifset mystic
For a more detailed description of MYSTIC read libRadtran/doc/README.MC.
@end ifset
@item tzs
TZS stands for "thermal, zero scattering" and is a very fast analytical solution 
for the special case of thermal emission in a non-scattering atmosphere. Please 
note that TZS does only radiance calculations at top of the atmosphere.
@item sss
SSS stands for "solar, single scattering" and is an analytical single
scattering approximation which might be reasonable for a optically
thin atmosphere. Please note that SSS does only radiance calculations 
at top of the atmosphere. 
@item null
The NULL solver does not solve the radiative transfer equation. However, it sets up the
optical properties, and does the post-processing; useful if you are either interested 
in the overhead time required by a particular model input or if you are simply interested 
in the optical properties, as output by @code{verbose}.
@end vtable
@ifset mystic
@item satellite_geometry
With this option the satellite geometry is determinded. The argument for this option
@example
satellite_geometry  netCDF_file 
@end example
is the location of a @code{netCDF_file}, which must contain latitude and longitude position
as well as zenith and azimuth viewing angle for each pixel.
For format specification see the example file data/satellite/MSG_seviri/MSG_seviri_geometry.nc.
This option has to be used in combination with @code{satellite_pixel}.
You can also use the abbreviation
@code{satellite_geometry MSG}
for MSG simulation, @emph{but in order to use this, you have to copy the netCDF file}
@example
cp -r /data/A3/satellite_geometry/satellites libRadtran/data/
@end example
@emph{which is not in cvs version, as it is to large.}
@item satellite_pixel
This option is specifies the pixel of the satellite image, which should be simulated. 
@example
satellite_pixel  pixel_x pixel_y
@end example
The arguments @code{pixel_nr_x} and @code{pixel_nr_y} specifies the pixel position in the native 
system of the satellite, which is determinded by the option @code{satellite_geometry}.
@end ifset
@item slit_function_file
If specified, the calculated spectrum is convolved with the function found
in the @file{slit_function_file}. The file must
contain two columns. Column 1 is the wavelength, in nm, and relative to the
center wavelength. Column 2 is the corresponding slit function value. It must
be unity at the maximum. The wavelength steps in the slit function file must
be equidistant. Comments start with @code{#}. Empty lines are ignored. Please 
note that prior to convolution the spectrum is interpolated to the wavelength
steps of the slit function. For this reason, make sure that the resolution
of the slit function is high enough even if the slit function is e.g. a 
simple triangle which could in principle be described with 3 grid points. 
@item solar_file
Location of file holding the extraterrestrial spectrum.
@example
solar_file filename [unit]
@end example
The file must contain two columns. Column 1 is the wavelength in nm, and column 2 
the corresponding extraterrestrial flux. The user may freely use any units
he/she wants on the extraterrestrial flux. The wavelength grid specified
defines the wavelength resolution at which results are returned. However, 
the wavelength range is determined by @code{wavelength}. @code{solar_file} may be 
omitted for thermal radiation calculations (@code{source thermal}) as well as 
@code{transmittance} and @code{reflectivity} calculations. If omitted, the 
output resolution equals the internal wavelength grid which the model chooses 
for the radiative transfer calculation.
Comments start with @code{#}. Empty lines are ignored.

For some purpuses it is useful to tell libRadtran the units of the spectrum.
This can be done with the optional second argument. If @code{unit} is set to @code{per_nm}
libRadtran assumes that the unit of the spectrum is W/(m2 nm), if set to @code{per_cm-1}
it assumes W/(m2 cm-1).

Note that @code{solar_file} is ignored if @code{correlated_k} is specified.
@item source
Solar or thermal source. Set to either @code{solar} or @code{thermal}.
Solar radiation is per default output in W/(m2 nm) for spectral and 
@code{correlated_k} LOWTRAN calculations. For all other @code{correlated_k} options
the output is integrated over the wavelength band.

Thermal radiation is per default output in W/(m2 cm-1), if the bandwidth 
is equal to 1 cm-1 (default for @code{correlated_k LOWTRAN} calculations). 
Otherwise the output is the integrated flux over the
wavenumber interval specified by @code{thermal_bandwith}, @code{thermal_bands_file},
or by the @code{correlated_k} option (Kato, Kato2, Kato2.96, Fu, AVHRR_KRATZ, 
or Generic).
@item spline
Spline interpolate to wavelengths lambda_0 to lambda_1 in steps of lambda_step,
in nm. Specified as e.g.
@example
spline 290. 365. 0.5
@end example
Here, the calculated spectrum is interpolated to wavelengths 290., 290.5, 291., ...
, 364.5, 365. For interpolation to arbitrary wavelengths use @code{spline_file}.
The specified wavelength interval must be within the one specified by @code{wavelength}.
@item spline_file
Spline interpolate to arbitrary wavelengths, in nm, given as a single column in file 
@file{spline_file}. The specified wavelengths must be within the range specified 
by @code{wavelength}. Comments start with @code{#}. Empty lines are ignored.
@item surface_temperature
Surface temperature, used for thermal infrared calculations. If not specified, the
temperature of the lowest atmospheric level is used as surface temperature.
@item surface_temperature_map
Specify a surface_temperature map with a @emph{netCDF} file which is used in combinition 
with the options @code{latitude}, @code{longitude}, and @code{time}.
@example
surface_temperature_map file [variable_name]
@end example
where @code{file} is the location of the @emph{netCDF} file. 
@code{libRadtran} reads the value at the nearest pixel to the given @code{latitude} and 
@code{longitude}. No spatial interpolation or averaging of the values is done.
@item surface_type
With this option the @code{surface_type} is selected.
This option can be used with @code{albedo_library} in order to select an spectral albedo 
or with @code{rpv_library} in order to select a BRDF function.
@example
surface_type surface_type_number
@end example
where @code{surface_type_number} is an integer starting from 0,
where 0 refers to a black surface and the following numbers to 
the entries in the specified @code{library}.
@item surface_type_map
Specify a surface type map, which is used in combinition with 
@code{albedo_library}, @code{latitude}, and @code{longitude} in order to 
select the surface type relevant for the simulation.
No pixel interpolation is done.
The format of the call is:
@example
surface_type_map file [variable_name]
@end example
where @code{file} is the location of the surface type map file. The map is expected to 
be in @emph{netCDF} format. The file must contain the variables @code{double lat(nlat)}, 
@code{double lon(nlon)}, and @code{byte surface_type (nlat, nlon)}. If the name of the 
surface type variable is different, the optional argument can be used in order to specify 
the variable name. For format specification see also data/albedo/IGBP/SURFACE_TYPE_IGBP_10min.cdf.

For using the IGBP map, the call is @code{surface_type_map IGBP}. This map has a resolution of 10 
minutes and contains the surface types 1 to 18 defined in the @code{albedo_library IGBP}. 
Fresh snow and sea ice are not included, as their extent is too variable. (Attention:  
That means, that the surface type of the north pole is ocean_water and not sea_ice!)

Locations on the pixel boundaries are interpreted as the pixel northward and
eastward respectively. E.g. location 0 N, 0 E is interpreted like the pixel ranging from
0 to 10min North and from 0 to 10min East.
@item sza
The solar zenith angle. If the solar zenith angle varies with wavelength,
use @code{sza_file}. The default solar zenith angle is 0.0.
@item sza_file
Location of solar zenith angle file for wavelength dependent solar
zenith angle. The file must have two or three columns. Column 1 is the wavelength, 
in nm, and column 2 the corresponding  solar zenith angle. Optionally
the third column may contain the corresponding solar azimuth angle. The solar 
azimuth angle is only needed when calculating radiances. The wavelength 
grid may be freely set. The solar zenith and azimuth angle will be interpolated to 
the wavelength grid used for the radiation calculation.
Comments start with @code{#}. Empty lines are ignored.
@item thermal_bands_file
File with the center wavelengths and the wavelength band intervals to be used for 
calculations in the thermal range. The following three columns are expected:
center (or reference) wavelength, lower wavelength limit, upper wavelength limit [nm].
@code{thermal_bands_file} defines the wavelength grid for the radiative transfer 
calculation. The RTE solver is called for each of the wavelengths in the first column. 
The atmospheric (scattering, absorption, etc) properties are also evaluated at these 
wavelengths. For thermal radiation calculations, the Planck function is integrated 
over the wavelength bands defined in the second and third columns. The result will
therefore be a band-integrated irradiance which does only make sense when the 
@code{solar_file} grid equals the @code{thermal_bands_file} grid.
@item thermal_bandwidth
Specify a constant bandwidth in cm-1 for thermal calculations. The default is 1 cm-1.
This option is ignored if used together with @code{correlated_k KATO/KATO2/KATO2.96/FU/AVHRR_KRATZ}.
@item time
Specifies the time to simulate.
@example
time YYYY MM DD hh mm ss
@end example
where @code{YYYY} is the year, @code{MM} the month, @code{DD} the day, 
@code{hh} the hour, @code{mm} the minute, @code{ss} the second in UTC.
The time information will be used for a couple of things: 

@code{time} is used to correct extraterrestrial 
irradiance for the Sun-Earth distance with the day of year. If not given, 
the Earth-Sun distance is 1 AU (i.e. equinox distance).

@code{time} in combination with @code{latitude} and @code{longitude} is used to calculate 
the solar zenith angle.

@code{time} in combination with @code{latitude} and @code{longitude} is used to choose a
suitable default atmosphere file, if no @code{atmosphere_file} is specified.

@ifset mystic
@code{time} in combination with an @code{ECMWF_atmosphere_file} is used to interpolate
the date to the given time. 
@end ifset
@item transmittance
Calculate transmittance / reflectance instead of absolute quantities. 
That is, set the extraterrestrial irradiance to 1 and do not correct for Sun-Earth distance:
@iftex
 @tex
  $$ T = {{E}\over{E_0}}$$
  where $E$ is the irradiance / actinic flux / radiance and $E_0$ is the extraterrestrial flux.
 @end tex
@end iftex
@ifinfo
 T = E / E0 @*
 where E is the irradiance / actinic flux and E0 is the extraterrestrial flux.
@end ifinfo
Please note the difference to @code{reflectivity}.
@item transmittance_wl_file
Location of single column file that sets the wavelength grid used for the 
internal transmittance calculations. The wavelengths must be in nm.
Do not use this option unless you know what you are doing.
Comments start with @code{#}. Empty lines are ignored.
@item umu
Cosine of output polar angles in increasing order, starting with 
negative (downward) values (if any) and on through positive (upward)
values. Must not have any zero values. The azimuthally averaged intensity
@code{u0u} is output at @code{umu}.
@item verbose
If specified abundances of informative messages are output to stderr. To make
use of this information, you may want to write the standard uvspec output to 
one file and the diagnostic messages to another. To do so, try
@code{(./uvspec < uvspec.inp > uvspec.out) >& verbose.txt}. The irradiances 
and radiances will be written to @file{uvspec.out} while all 
diagnostic messages go into @file{verbose.txt}. See also @code{quiet}.
@item wavelength
Set the wavelength range by specifying first and last wavelength in nm. 
The default output wavelength grid is that defined in @code{solar_file}, 
unless @code{spline} is specified. Note that the radiative transfer calculations 
are done on an internal grid which can be influenced with @code{transmittance_wl_file}
or @code{molecular_tau_file}
@item wavelength_index
Set the wavelengths to be selected. To be used together with predefined wavelength grids,
such as @code{transmittance_wl_file} @code{molecular_tau_file} and particularly 
useful in combination with the @code{correlated_k} option where often only a 
specified number of wavelength bands is required. E.g., in combination with
@code{correlated_k AVHRR_KRATZ}, @code{wavelength_index 15 15} will select wavelength
index 15 which corresponds to channel 4, or @code{wavelength_index 10 14} will select 
those bands required for channel 3. Indices start from 1.
@item wc_cloudcover
Set the fraction of the horizontal sky area which is covered by clouds.
When a cloud cover is specified, the result will be calculated by
the independent pixel approximation (IPA), that is, as weighted average 
of cloudless sky and overcast sky, where the cloud properties are taken 
from @code{wc_file}, etc. Please note that, if both @code{wc_cloudcover} and
@code{ic_cloudcover} are set, both must be equal.
@item wc_file
Location of file defining water cloud properties. The file must
contain three columns: Column 1 is the altitude in km, column 2 
the liquid water content (LWC) in grams per cubic meter, and column 3 the effective
droplet radius in micron. Empty lines are ignored. Comments start with @code{#}.
Note that the definition of cloud altitudes in @code{wc_file} refers to sea level, 
not to altitude above ground. E.g., when altitude is set to 1.63km, and the first 
cloud level is defined at 3km, the cloud would start at 1.37km above ground.  
An example of a cloud is given in @file{examples/WC.DAT}.

Per default (@code{wc_layer} is not defined) the cloud properties are interpreted 
as properties at a given altitude @emph{level}. The optical depth of a layer is 
calculated using information from the upper and lower levels defining the layer, 
if the LWC in both layers is not 0. If the LWC in one of them is 0, there is no 
cloud in this layer. (@code{correlated_k fu} works only for cloud layer properties.)

If @code{wc_layer} is defined, the informations given in @code{wc_file} are interpreted 
as @emph{layer} properties (please see the section about water clouds for a realistic 
example how the contents of the @code{wc_file} are converted to optical properties).
@item wc_files
A way to specify cloud extinction coefficient, single scattering albedo, 
and scattering phase function for each layer. The file specified by 
@code{wc_files} has two columns where column 1 is the altitude in km. The second
column is the name of a file which defines the optical properties of the layer 
starting at the given altitude. The files specified in the second column must 
have the following format:
@table @code
@item Column 1: 
The wavelength in nm. These wavelengths may be different from those in @code{solar_file}. 
Optical properties are interpolated to the requested wavelengths.
@item Column 2:
The extinction coefficient of the layer in units km-1. 
@item Column 3:
The single scattering albedo of the layer.
@item Column 4-(nmom+4):
The moments of the scattering phase function. 
@end table
Note that if using the @code{rte_solver disort2} it makes good sense to make the 
number of moments larger than @code{nstr}. For @code{rte_solver disort} and 
@code{rte_solver polradtran} the number of moments included in the calculations 
will be @code{nstr}+1. Higher order moments will be ignored for these solvers.
Please note that the uppermost line of @code{wc_files} denotes simply
the top altitude of the uppermost layer. The optical properties of
this line are consequently ignored. There are two options for this
line: either an optical property file with zero optical thickness is
specified or "NULL" instead.
@ifset mystic
@item wc_ipa
Independent column calculation for a 3D cloud field.
As argument a name of a 3D cloud file must be given. This file has to 
be in the format as needed by MYSTIC.
@end ifset
@item wc_ipa_files
A two-column file, defining water cloud property files (see @code{wc_file}) in the first 
column and the corresponding weights in the second column. The radiative transfer 
calculation is performed independently for each cloud column and the result is the weighted 
average of all independent columns. If @code{ic_ipa_files} and @code{wc_ipa_files} 
are both defined, both must have the same columns in the same order, 
otherwise @code{uvspec} will complain.
@item wc_layer
Specify cloud properties for layers instead of levels (see also 
wc_file). If wc_layer is specified, cloud properties are 
assumed to be constant over the layer. The layer reaches from the 
level, where the properties are defined in the @code{wc_file} to the level
above that one.
For example, the following lines
@example
#      z     LWC    R_eff
#     (km)  (g/m^3) (um)    
      4.000   0.0   0.0
      3.000   1.0  10.0
@end example
define a cloud in the layer between 3 and 4 km with sharp boundaries.
@item wc_properties
Define how liquid water content and effective droplet radius are translated 
to optical properties. Possible choices are
@table @code
@item hu 
Parameterization by Hu and Stamnes (1993); 
this is the default setting. Note that 
the parameterization is somewhat different for 'correlated_k FU' than for all other 
cases because in the latter case the parameterization from the newer (March 2000) 
Fu and Liou code is used while otherwise the data are taken from the original 
Hu and Stamnes paper. Note that this parameterization has been
developed to calculate irradiances, hence it is less suitable for radiances.
This is due to the use of the Henyey-Greenstein phase function as an approximation
of the real Mie phase function.
@item echam4
Use the very simple two-band parameterization of the ECHAM4 climate model, described 
in Roeckner et al. (1996); this is probably only meaningful if you want to compare
your results with ECHAM4, the two bands are 0.2 - 0.68 micron and 0.68 - 4.0 micron;
within these bands, the optical properties are assumed constant.
@item mie
Use pre-calculated Mie tables; useful for @code{correlated_k}; 
the tables are expected in @code{data_files_path}/correlated_k/../ @*
For spectral or pseudo-spectral (@code{correlated_k sbdart}) calculations, 
a set of pre-calculated tables is also available; the wavelength grid points of these 
data has been carefully selected such that the extinction cross section, 
single scattering albedo, and the asymmetry parameter are accurate to 1% 
(compared to the fully-resolved Mie calculation) for all wavelengths
between 250nm and 100 micron. For spectral or pseudo-spectral
calculations @code{wc_properties_interpolate} has to be defined explicitely to
initiate the interpolation of the optical properties to the internal wavelength grid.
Please note that this option may be extremely memory-consuming because for each 
internal wavelength a full set of Legendre moments of the phase function is 
stored (up to several thousands). The Mie tables are not part of the standard distribution 
(because of their large size) but they are freely available from http://www.libradtran.org. 
This is the correct option to calculate radiances, to be preferred over the 
Henyey-Greenstein approach of Hu and Stamnes (1993).
@item filename
Read optical properties from specified filename; file format is as produced 
by the mie-tool of the @code{libRadtran-tool} box (see @code{output_user cloudprp}) 
or by Frank Evans' @code{cloudprp}; for each of the internal (computational) wavelengths, 
a separate file is expected. Use only if you really know what you are doing
(this option is subject to change).
@end table
@item wc_properties_interpolate
Interpolate water cloud optical properties over wavelength; useful for precalculated 
optical property files defined with @code{wc_properties}. Please note that this option 
may be extremely memory-consuming because for each internal wavelength a full set of 
Legendre moments of the phase function is stored (up to several thousands). 
@item wc_saturate
With this option, the relative humidity inside water clouds can easily be adjusted 
to a user-defined value, e.g. saturated with respect to water. 
This option has one nessesary and one optional argument:
@example
ic_saturate switch [relative_humidity]
@end example
where @code{switch} is @code{on}, @code{off}, or @code{ipa}. 
The second optional argument determines the relative humidity in % 
(with respect to water!) inside the cloud. The default value is 100.
ipa is only relevant for independent column calculations. 

Details for independent column simulations: Using @code{switch} @code{on}, the air 
in @emph{all} columns will be saturated, if there is a cloud in at least 
one of the columns (this option should be used for stratiform clouds).
Using @code{switch} @code{ipa}, only cloudy columns are 
affected (this option should be used for convective cloud fields.)

@item wc_scale_gg
Scale the water cloud asymmetry factor for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_scale_ssa
Scale the water cloud single scattering albedo for all wavelengths and altitudes
with a float between 0.0 and 1.0.
@item wc_set_gg
Set the water cloud asymmetry factor for all wavelengths and altitudes
to a float between -1.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_ssa
Set the water cloud single scattering albedo for all wavelengths
and altitudes to a float between 0.0 and 1.0. This option is useful only for monochromatic 
calculations or in wavelength regions where the optical properties of water clouds 
can be considered constant, e.g. the ultraviolet range.
@item wc_set_tau
Set the total water cloud optical thickness to a constant value for all wavelengths.
The optical thickness defined here is the integral from the surface at the 
user-defined @code{altitude} to TOA (top of atmosphere). This option is useful only 
for monochromatic calculations or in wavelength regions where the optical properties 
of water clouds can be considered constant, e.g. the ultraviolet range.
@item wc_set_tau550
Set the water cloud optical thickness at 550nm. The optical thickness defined here 
is the integral from the surface at the user-defined @code{altitude} 
to TOA (top of atmosphere). Other wavelengths are scaled accordingly.
Note that this option requires for technical reasons that the
wavelength interval defined by @code{wavelength} does contain 550nm.
@item zout 
This option is used to specify the output altitudes in km @emph{above surface altitude}. 
One or more altitudes may be specified in increasing magnitude. 
@example
zout 0 1 2 3 4 5 ...
@end example
Output altitudes must be within the range defined in the
@code{atmosphere_file}. Note that @code{zout} does not restructure the
atmosphere model. Hence, if you specify @code{zout 0.730} and have your
atmosphere model in @code{atmosphere_file} go all the way down to 
sea level, i.e. 0.0km., output is presented at 0.730km and calculations
performed with an atmosphere between 0.0 and 0.730 km (and above of course). 
If you want calculations done for e.g. an elevated site you have to restructure
the atmosphere model and make sure it stops at the appropriate altitude.
This you may either due by editing the atmosphere file or by using 
@code{altitude}. Note that for @code{rte_solver polradtran} the atmosphere file must 
contain the altitudes specified by @code{zout}. You can also use @code{toa} for 
@code{top of atmosphere} and @code{sur} for @code{surface altitude}.

@ifset mystic
Instead of specifying the altitudes in km, it is also possible to use a keywords as 
argument for this option. Possible keywords are @code{atm_levels}, 
@code{all_levels}, and @code{model_levels}. For @code{atm_levels}, all levels from 
the @code{atmosphere_file} are used as output levels.  For @code{all_levels}, all 
levels (including levels from @code{atmosphere_file}, @code{dens_file}, cloud files, 
@code{altitude} options) are used as output levels. For @code{model_levels} all levels
from any ECMWF or ECHAM input file are used as output level.
@end ifset
@item zout_sea
like zout, but @emph{above sea surface}
@item z_interpolate
The profile in the @code{atmosphere_file} provides the constituents of the atmosphere 
at the given levels. Where additional levels are introduced and in order to calculate
layer properties, an assumption about the variation of the
property within the layer is required. These interpolation methods can be 
changed by the @code{z_interpolate} option. Two arguments are required, the 
property, and the interpolation method:
@example
z_interpolate property interpolation_method
@end example
Properties which may be specified are:
@table @code
@item O3, O2, H2O, CO2, NO2, BRO, OCLO, HCHO
@item T
temperature (here linmix is not suitable)
@end table
Possible interpolation methods are:
@table @code
@item linear
The specified property (number density of the gas or temperature) varies linearly with height.
@item log
The specified property (number density of the gas or temperature) vaires logarithmically with hight.
This is a reasonable option for all well mixed trace gases.
@item linmix    
This option is only possible for gas profiles.
The mixing ratio of the gas (assuming a logarithmically varying 
air density) varies linearly with hight.
@end table
For all gas densities the default interpolation method is @code{linmix},
for temperature it is @code{linear}.
@item zout_interpolate
The z-grid of optical properties is determined by the @code{atmosphere_file},
and, if specified, by other profile files like @code{dens_file}, @code{rh_file}, 
or @code{refractive_index_file}. 
Additional levels might be introduced by the @code{zout} 
option and the second argument of the @code{altitude} option. By default 
(if @code{zout_interpolate} is not specified) levels introduced 
by the @code{zout} and @code{altitude} options will not affect the optical property 
profiles, that is, the optical properties are constant within the layers specified by 
the @code{atmosphere_file} and profile files.
If @code{zout_interpolate} is specified, the atmospheric profiles (tracegases, temperature ...) 
are interpolated to the levels introduced by @code{zout} and @code{altitude}, and optical 
properties are determined from the interpolated atmospheric properties.
If @code{heating_rate} or @code{rte_solver polradtran} is specified, @code{zout_interpolate} 
will also be automatically activated. 
@code{zout_interpolate} generally causes smoother variation of the optical properties.
@end vtable
  @c11_03@ */

  /*  @11_02c@                
@cindex output, uvspec
@cindex uvspec, output
The uvspec output differs for the different solvers. It may be controlled to 
some degree using @code{output_user}.

@node DISORT, SDISORT and SPSDISORT
@subsubsection DISORT, SDISORT and SPSDISORT
For the disort, sdisort and spsdisort solvers @code{uvspec} outputs one block of data to 
standard output (stdout) for each wavelength. The format of the block is

@example
lambda edir edn eup uavgdir uavgdn uavgup
@end example

@noindent if @code{umu} is not specified. If @code{umu} is specified the format
of the block is

@example
lambda edir edn eup uavgdir uavgdn uavgup 
umu(0) u0u(umu(0)) 
umu(1) u0u(umu(1)) 
  .     .          
  .     .          
@end example

@noindent If both @code{umu} and @code{phi} is specified
the output format of each block is

@example
lambda edir edn eup uavgdir uavgdn uavgup
                            phi(0)     ...        phi(m)
umu(0) u0u(umu(0))   uu(umu(0),phi(0)) ... uu(umu(0),phi(m))
umu(1) u0u(umu(1))   uu(umu(1),phi(0)) ... uu(umu(1),phi(m))
  .     .             .                     .
  .     .             .                     .
umu(n) u0u(umu(n))   uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
@end example

@noindent and so on for each wavelength.

@node TWOSTR
@subsubsection TWOSTR
The format of the output line for the twostr solver is

@example
lambda edir edn eup uavg
@end example

for each wavelength.

@node POLRADTRAN
@subsubsection POLRADTRAN
The output from the @code{polradtran} solver depends on the number of
Stokes parameters, @code{polradtran_nstokes}. For each wavelength the 
output block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
@end example

@noindent if @code{phi} is not specified. Here @code{is} is the number
of Stokes parameters specified by @code{polradtran_nstokes}. If 
@code{phi} and @code{umu} are specified the block is

@example
lambda down_flux(1) up_flux(1) ...  down_flux(is) up_flux(is)
                             phi(0)                 ...                 phi(m)
Stokes vector I
umu(0) u0u(umu(0))   uu(umu(0),phi(0)) ... uu(umu(0),phi(m))
umu(1) u0u(umu(1))   uu(umu(1),phi(0)) ... uu(umu(1),phi(m))
  .     .             .                     .
  .     .             .                     .
umu(n) u0u(umu(n))   uu(umu(n),phi(0)) ... uu(umu(n),phi(m))
Stokes vector Q
  .       .                                          .	                    
  .       .                                          .	                    
@end example

@noindent Note that @code{polradtran} outputs the total (=direct+diffuse) 
downward flux. Also note that u0u is always zero for @code{polradtran}.

@node Description of symbols
@subsubsection Description of symbols

In the above output blocks the symbols used have the following meaning. 

@vtable @code
@item cmu 
Computational polar angles from @code{polradtran}.
@item down_flux, up_flux
The total (direct+diffuse) downward (down_flux) and upward (up_flux) irradiances.
Same units as extraterrestrial irradiance.
@item lambda
Wavelength (nm)
@item edir
Direct beam irradiance (same unit as extraterrestrial irradiance, e.g mW/(m2 nm) if 
using the @file{atlas3} spectrum in the @file{data/solar_flux} directory.)
@item edn
Diffuse down irradiance, i.e. total minus direct beam (same unit as @code{edir}).
@item eup
Diffuse up irradiance (same unit as @code{edir}).
@item uavg
The mean intensity. Proportional to the actinic flux: To obtain the actinic flux, 
multiply the mean intensity by 4 pi (same unit as @code{edir}).
@item uavgdir
Direct beam contribution to the mean intensity. (same unit as @code{edir}).
@item uavgdn
Diffuse downward radiation contribution to the mean intensity. (same unit as @code{edir}).
@item uavgup
Diffuse upward radiation contribution to the mean intensity. (same unit as @code{edir}).
@item u0u
The azimuthally averaged intensity at @code{numu} user specified 
angles @code{umu}. (units of e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu
The radiance (intensity) at @code{umu} and @code{phi} user specified 
angles (unit e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  
@item uu_down, uu_up
The downwelling and upwelling radiances (intensity) at @code{cmu} and 
@code{phi} angles (unit e.g. mW/(m2 nm sr) if using the @file{atlas3} spectrum 
in the @file{data/solar_flux} directory.)  

@end vtable

@noindent The total downward irradiance is given by

@example
irr_down = edir + edn
@end example

@noindent The total mean intensity is given by

@example
uavg = uavgdir + uavgdn + uavgup
@end example

@noindent If @code{deltam} is on it does not make sense to look at the individual 
contributions to @code{uavg} since they are delta-M scaled.

  @c11_02@ */

  /*  @11_01c@                

uvspec is controlled in a user-friendly way. The control options are
named in a (hopefully) intuitive way. In the following, several
examples are given, how to create an input file, how to define a
cloudless sky atmosphere, how to add aerosols and clouds, etc. All
following examples are taken from the libRadtran examples directory
and are part of the uvspec self-check. For a complete listing and
explanation of all input options, have a look at section 
@ref{Complete description of input parameters}.

The uvspec input file consists of single line entries, each making
up a complete input to the uvspec program. First on the line comes
the parameter name, followed by one or more parameter values. The 
parameter name and the parameter values are separated by white space.
Filenames are entered without any surrounding single or double quotes.
Comments are introduced by a @code{#}. Blank lines are ignored. 
The order of the lines is not important, with one exception: if the
same input option is used more than once, the second one will usually
over-write the first one. Be aware that also option in another @code{included} 
input file will overwrite options specified before.

@subsubsection Cloudless, aerosol-free atmosphere

The simplest possible input file contains only a few lines:
@example
@cartouche
@include ../examples/UVSPEC_SIMPLE.INP
@end cartouche
@end example

The first three statements define the location of some data files: the
uvspec data path (@code{data_files_path}) where all internal files are
expected, the atmospheric profile
(@code{atmosphere_file}), and the extraterrestrial spectrum
(@code{solar_file}). The last line defines the desired wavelength
range which is a monochromatic data point in this example. All other data
which are not explicitely mentioned assume a default value which is
"0" in most cases. Here, the solar zenith angle is 0, the
surface albedo is 0, and the atmosphere does not contain clouds 
nor aerosols. Pressure, temperature, ozone concentration, etc. are 
read from @code{atmosphere_file}.

An example of a more complete input file for a clear sky atmosphere is

@example
@cartouche
@include ../examples/UVSPEC_CLEAR.INP
@end cartouche
@end example

@noindent 
The atmosphere model, i.e. pressure, temperature, and ozone concentration
profiles are read from @code{../data/atmmod/afglus.dat}. The 
extraterrestrial solar flux is read from the file 
@code{../data/solar_flux/atlas_plus_modtran}.

A wavelength dependent @b{surface albedo} may be specified 
using @code{albedo_file} instead of @code{albedo}. Non-Lambertian
surface reflectance (BRDF) for vegetation and water may also be
defined (please note that these require the use of 
@code{rte_solver disort2}. 
The BRDF of Vegetation is specified using @code{rpv_rho0},
@code{rpv_k}, and @code{rpv_theta}, following the definition of 
Rahman et al. (1993). Wavelength-dependent BRDF for vegetation can
be defined with @code{rpv_file}. The BRDF of water surfaces is parameterized
following Cox and Munk (1954a, 1954b) and Nakajima (1983). The
respective parameters are the wind speed @code{cox_and_munk_u10}, 
the pigment concentration @code{cox_and_munk_pcl}, and the salinity
@code{cox_and_munk_sal}. A complete description of these parameters is
given in section @ref{Complete description of input parameters}.

It is helpful to know some details about the @b{input/output
wavelength resolution} in uvspec and how it can be influenced by the
user. Basically there are three independent wavelength grids, the
@strong{input grid}, the @strong{internal grid}, and the 
@strong{output grid}. The essential thing to know is that the
internal grid is chosen by uvspec itself in a reasonable way, if not
explicitely defined in the input file with @code{transmittance_wl_file}
or @code{molecular_tau_file}.
The output grid is completely independent of the internal grid and 
is entirely defined by the @code{solar_file}. The wavelength grid of
all other input data (e.g. albedo, optical properties of aerosols and 
clouds, etc) is also completely independent. These data are 
automatically interpolated to the resolution of the internal
wavelength grid. Hence, only two constraints are set to the gridding
of the input data: (1), the wavelength range has to cover all internal
grid points; and (2), it should be chosen in a reasonable manner to
allow reasonable interpolation (which essentially means, dense enough).

In the ultraviolet/visible, uvspec uses an internal grid with a step
with of 0.5nm below 350nm and 1nm above 350nm. This is a conservative
choice which fully resolves the broad ozone absorption bands and the
slowly varying Rayleigh, aerosol, and cloud extinctions. The idea is
outlined in the following figure which is taken from Mayer et
al. (1997):

@image{./transpz}

The transmittance (or reflectance) is calculated on a moderate
resolution grid which reduces the number of calls to the
@code{rte_solver} and hence the computational time. Then, the
transmittance is interpolated to the wavelengths in the @code{solar_file}
(which is usually defined with higher spectral resolution), multiplied 
with the extraterrestrial irradiance, and possibly post-processed.
Hence, the wavelength in the output spectrum are those
contained in the @code{solar_file} which has two important
implications: (1) Only those wavelengths are output that are contained
in the @code{solar_file}. If e.g. a monochromatic calculation is 
defined by setting '@code{wavelength 327.14}', there will only be output
if the wavelength 327.14 is explicitely listed in @code{solar_file};
(2) this is also true at thermal wavelengths where the
extraterrestrial irradiance is zero; hence, even for a calculation in the
thermal range a @code{solar_file} can be specified which defines the
output grid in the first column and arbitrary values in the second
column. Keeping these points in mind, @code{solar_file} is a
convenient way to define an arbitrary output grid. 
@code{solar_file} may be omitted for thermal radiation calculations 
(@code{source thermal}) as well as for @code{transmittance} 
and @code{reflectivity} calculations. If omitted, the output grid
equals the internal wavelength grid.

If required, a @b{user-defined internal grid} can be specified with 
@code{transmittance_wl_file} or @code{molecular_tau_file}. 
Note that this is a way to speed up the calculation considerably. 
E.g., for some applications the internal grid in the UV-A and visible 
can be set to 10nm which would reduce computational times 
by up to a factor of 10. 

Things are completely different if one of the @code{correlated_k}
parameterizations is selected (see below). In this case all flexibility
is taken away from the user which is an inherent feature of the
k distribution method. Internal grid as well as the extraterrestrial
file are in this case defined by the choice of the parameterization itself.


@subsubsection Spectral resolution

@code{uvspec} offers four different ways of spectral calculations:
@enumerate
@item
@strong{Spectrally resolved calculation} in the UV and visible spectral ranges;
@item 
@strong{Line-by-line calculation} with user-defined molecular absorption data;
@item 
@strong{The correlated-k method}.
@item 
@strong{Pseudo-spectral calculation} with exponential-sum-fit,
from LOWTRAN; code adopted from SBDART (Ricchiazzi et al., 1998);
@end enumerate

The choice of the method is determined by the problem and the decision
is therefore entirely up to the user. The spectrally resolved
calculation and the line-by-line calculation are more or less exact
methods while the correlated-k distribution and the pseudo-spectral
calculation are approximations that provide a compromise between speed
and accuracy. In the following it is briefly described which method
fits which purpose: 

A @strong{spectrally resolved calculation} is the most straightforward
way, and will be the choice for all users interested in the
ultraviolet and visible spectral ranges. In the UV/vis gas absorption
generally occurs in broad bands with only slow spectral variation, the
most important of these being the Hartley, Huggins, and Chappuis bands
of ozone. Hence, a radiative transfer calculation every 1nm usually is
sufficient to fully  resolve any spectral variation using the method
described in the last section. Absorption cross sections for various
species are included, among them the most important O3 and NO2.

In the infrared, however, molecular absorption spectra are
characterized by thousands of narrow absorption lines. There are  
two ways to treat these, either by highly resolved spectral
calculations, so-called @strong{line-by-line} calculations, or by a band
parameterization. Concerning line-by-line, uvspec offers the
possibility to define a spectrally resolved absorption cross section
profile using @code{molecular_tau_file}. There is no option in
libRadtran to generate such a @code{molecular_tau_file}, because 
(1) the HITRAN database which forms the basis for such calculations
amounts to about 100 MByte which are updated continuously; and (2), 
there are sophisticated line-by-line programs available, like
e.g. @code{genln2} (Edwards, 1992). Using @code{genln2} it is
straightforward to create the input for uvspec line-by-line
calculations. It is also planned to make line-by-line cross sections
available for the six standard profiles that come with
libRadtran. The following figure shows an example of a line-by-line
calculation of the atmospheric transmittance in two selected solar
and thermal spectral ranges, the O2A-absorption band around 760nm and
a region within the infrared window around 10 micron.

@image{./lblo2a}

All spectral lines in the left figure are due to absorption by oxygen,
while the ones in the right figure are due to ozone, water vapour, and
CO2. Line-by-line is obviously the exact way for radiation
calculations. For most applications, however, line-by-line is far too
slow. Here one 
needs a band parameterization, and the most accurate of these is the
so-called @strong{correlated-k approximation}. uvspec contains several
correlated-k parameterizations which are invoked with 
@code{correlated_k}, in particular Kato et al. (1999), 
Fu and Liou (1992/93), Kratz (1995), as well as the possibility to
specify a user-defined one. Kato et al. (1999) is a accurate parameterization 
for the solar spectral range. @code{uvspec} contains three different versions:
@vtable @code
@item Kato 
The original tables provide by Seiji Kato which should correspond to the 
full version described in Kato et al. (1999); 575 subbands total, that is,
575 calls to the @code{rte_solver}
@item Kato2
A new, optimized version of the tables, provided by Seiji Kato, 2003, with only
148 subbands (that is, calls to the @code{rte_solver}); the uncertainty is only 
slightly higher than @code{Kato}; the absorption coefficients are based on HITRAN 2000.
@item Kato2.96
Similar to @code{Kato2} but based on HITRAN96.
@end vtable
The following Figure shows a comparison between the three parameterization which 
are part of libRadtran and the data from Figure 3 by Kato et al. [1999]. It is 
immediately obvious that the uncertainty is high for all bands above 2.5 micron 
which is probably due to the treatment of band overlap. For this reasons, the 
results for the indiviudal bands should not be trusted while the integrated 
shortwave radiation (the sum of all 32 bands) is calculated with high accuracy because
(1) the bands above 2.5 micron contribute only little to the integrated irradiance; 
and (2) errors are random and cancel each other to some degree.

@image{./figure3}

For more information on these parameterizations please refer to the mentioned
publications. Correlated-k is a powerful way to calculate spectrally
integrated quantities, however, it takes away some flexibility. In
particular, this means that the wavelength grid is no longer chosen by
the user but by the parameterization, that is, by @code{uvspec}. 
The @code{uvspec} output is then no longer spectral quantities,
e.g. W / (m2 nm), but integrated over the spectral bands, e.g. 
W / m2.

A simple but complete example for a correlated-k approximation 
of the solar spectrum:

@example
@cartouche
@include ../examples/UVSPEC_KATO.INP
@end cartouche
@end example

Here, the solar spectrum is split up into 32 bands according to 
Kato et al. (1999). In order to calculate integrated shortwave 
irradiance, simply sum the outputs, or even simpler, add 
@code{output sum} to the input file.

For @strong{pseudo-spectral calculations} in the whole spectral range, we have
adopted the molecular absorption parameterization from LOWTRAN/SBDART by 
Ricchiazzi et al. (1998). The respective section of this paper says:

@quotation
SBDART relies on low-resolution band models developed for the LOWTRAN
7 atmospheric trans-mission code (Pierluissi and Peng, 1985).  
These models provide clear-sky atmospheric transmission from 0 to
50000 cm-1 and include the effects of all radiatively active molecular
species found in the earth s atmosphere. The models are derived from
detailed line-by-line calculations that are degraded to 20 cm-1
resolution for use in LOWTRAN. This translates to a wavelength
resolution of about 5 nm in the visible and about 200 nm in the
thermal infrared. These band models represent rather large wavelength
bands, and the transmission functions do not necessarily follow Beers
Law. This means that the fractional transmission through a slab of
material depends not only on the slab thickness, but also on the
amount of material penetrated before entering the slab. Since the
radiative transfer equation solved by SBDART assumes Beers Law
behavior, it is necessary to express the transmission as the sum of
several exponential functions (Wiscombe and Evans, 1977). SBDART uses
a three-term exponential fit, which was also obtained from LOWTRAN
7. Each term in the exponential fit implies a separate solution of the
radiation transfer equation. Hence, the RT equation solver only needs
to be invoked three times for each spectral increment. This is a great
computational economy compared to a higher order fitting polynomial,
but it may also be a source of significant error. 
@end quotation

The LOWTRAN/SBDART gas parameterization is invoked with
@code{correlated_k LOWTRAN}. The spectral resolution may be arbitrarily
chosen by the user. If not explicitely defined with
@code{transmittance_wl_file}, an internal grid with a step width of
0.5nm below 350nm and 1nm above 350nm is chosen which is practically
overkill for most applications in the infrared. An extraterrestrial
spectrum covering the complete solar range is provided at two
different resolutions, @code{data/solar_flux/kurudz_1.0nm.dat} and 
@code{data/solar_flux/kurudz_0.1nm.dat}. An example for the solar
range is shown in @code{examples/UVSPEC_SBDART_SOLAR.INP}:

@example
@cartouche
@include ../examples/UVSPEC_SBDART_SOLAR.INP
@end cartouche
@end example

while @code{examples/UVSPEC_SBDART_THERMAL.INP} shows how to do a
thermal calculation:

@example
@cartouche
@include ../examples/UVSPEC_SBDART_THERMAL.INP
@end cartouche
@end example

@image{./sbdart}

The figure shows the results of the solar and thermal
calculations. The water vapour absorption bands in the solar range
are clearly visible, as is the absorption window around 10 micron in
the thermal. Please note the following points:
@itemize @bullet
@item 
Thermal radiation is per default output in W/(m2 cm-1), if the bandwidth 
is equal to 1 cm-1 (default for @code{correlated_k LOWTRAN} calculations). 
Otherwise the output is the integrated flux over the
wavenumber interval specified by @code{thermal_bandwith}, @code{thermal_bands_file},
or by the @code{correlated_k} option (Kato, Kato2, Kato2.96, Fu, AVHRR_KRATZ, 
or Generic). To convert e.g. to W/(m2 nm) use @code{output per_nm} or multiply with  
k/lambda where k is the wavenumber [cm-1] and lambda is the 
wavelength [nm]. To calculate band-integrated thermal quantities
please consider @code{thermal_bands_file}.
@item 
Even though no extraterrestrial irradiance is required, a
@code{solar_file} may be specified for the thermal case. The reason
is that, as explained initially, the @code{solar_file} defines the
output grid. The second column in @code{solar_file} can be chosen
arbitrarily in this case because it is ignored. 
@item 
For the choice of the wavelength grid for the calculation
(@code{transmittance_wl_grid}) please consider that the resolution of
the absorption parameterization is 5cm-1 which translates to 0.3nm at
750nm and to 50nm at 10 micron. Choosing higher resolutions for 
the internal wavelength grid (@code{transmittance_wl_file}) is usually
a waste of computational time.
@item
Please also make sure to choose a fine enough spectral resolution in
order to capture all absorption features.
@end itemize

The following figure shows two selected wavelength intervals of the
solar and thermal range, to demonstrate the spectral resolution of the
LOWTRAN/SBDART absorption parameterization.

@image{./sbdarto2a}

The resolution is about 5cm-1 which translates to about 0.3nm in the
left figure (oxygen A-band) and 50nm in the right figure (ozone
absorption band in the atmospheric window). Compare this figure to the
above line-by-line example to get an impression about the differences
between both methods.

@subsubsection Aerosol

All options to set up and modify aerosol properties start with 
@code{aerosol_}.
Aerosols may be specified in a hierarchical way. The most simple way to define 
an aerosol is by the command @code{aerosol_default} which will set up the
aerosol model by
E.P. Shettle, "Models of aerosols, clouds and precipitation for 
atmospheric propagation studies", in "Atmospheric propagation in the 
uv, visible, ir and mm-region and related system aspects", 
AGARD Conference Proceedings (454), 1989. The default properties are 
a rural type aerosol in the boundary layer, background aerosol above 2km,
spring-summer conditions and a visibility of 50km. These settings may
be modified with @code{aerosol_haze}, @code{aerosol_vulcan}, @code{aerosol_season},
and @code{aerosol_visibility}. More information can be introduced step by step,
overwriting the default parameters. @code{aerosol_tau_file}, @code{aerosol_ssa_file},
and @code{aerosol_gg_file}, can be used to define the profiles of optical thickness,
single scattering albedo, and asymmetry parameter. The integrated optical thickness 
can be set to a constant value using @code{aerosol_set_tau} or scaled 
with @code{aerosol_scale_tau}. The single scattering albedo may be scaled by 
@code{aerosol_scale_ssa} or set to a constant value by @code{aerosol_set_ssa}.  
The aerosol asymmetry factor may be set by @code{aerosol_set_gg}. 
The wavelength dependence of the aerosol optical depth is specified using the 
@code{aerosol_angstrom} parameter. @code{aerosol_moments_file} allows specification of 
the scattering phase function. If microphysical properties are available these 
may be introduced by defining the complex index of refraction @code{aerosol_refrac_index} 
or @code{aerosol_refrac_file} and the size distribution @code{aerosol_sizedist_file}. 
Finally, one may define the aerosol optical properties of each layer explicitely 
using @code{aerosol_files}.

The following list is an overview of the aerosol description parameters. The 
entries are arranged in a way that a parameter 'overwrites' all values higher up 
in the list.

@table @code
@item aerosol_default
Generate default aerosol according to Shettle (1989)
@item aerosol_vulcan, aerosol_haze, aerosol_season, aerosol_visibility
Set Shettle (1989) aerosol properties (aerosol type, visibility)
@item aerosol_files
Specify optical properties of each layer explicitly, that is, extinction coefficient,
single scattering albedo, and the moments of the phase function
(everything as a function of wavelength).
@item aerosol_tau_file, aerosol_ssa_file, aerosol_gg_file
Overwrite profiles of optical thickness, single scattering albedo, and
asymmetry parameter
@item aerosol_moments_file
Specify a phase function to be used instead of the Henyey-Greenstein phase function
@item aerosol_refrac_index, aerosol_refrac file, aerosol_sizedist_file
Calculate optical properties from size distribution and index of refraction using 
Mie theory. Here is an exception from the rule that ALL values defined above are 
overwritten because the optical thickness profile is re-scaled so that the 
optical thickness at the first internal wavelength is unchanged. It is done 
that way to give the user an easy means of specifying the optical 
thickness at a given wavelength. 
@item aerosol_set_gg, aerosol_set_ssa, aerosol_scale_ssa, aerosol_set_tau, aerosol_scale_tau
Overwrite profiles of asymmetry parameter and single scattering albedo
@item aerosol_angstrom
Overwrite the integrated optical thickness (profiles are not changed).
@end table

An example for a uvspec aerosol description is

@example
@cartouche
@include ../examples/UVSPEC_AEROSOL.INP
@end cartouche
@end example

@noindent 
By combining this with the clear sky example given above a complete uvspec
input file including aerosol is constructed.


@subsubsection Water clouds

All options to set up and modify water cloud properties 
start with @code{wc_}. 

The easiest way to define a water cloud is to specify a 
@code{wc_file} which defines the liquid water content 
and effective droplet radius at each model level. By combining the
following lines with the clear sky example given above a complete
uvspec input file including water clouds is constructed. 

@example
@cartouche
@include ../examples/UVSPEC_WC.INP
@end cartouche
@end example

A typical example for a wc_file looks like:

@example
@cartouche
#      z     LWC    R_eff
#     (km)  (g/m^3) (um)  
      5.000   0      0     
      4.000   0.2    12.0 
      3.000   0.1    10.0 
      2.000   0.1     8.0 
@end cartouche
@end example

The three columns are the level altitude [km], the liquid water
content [g/m3], and the effective droplet radius [micron]. Per
default, these quantities are defined at the given model levels, 
that is, the value 0.2 g/m3 refers to altitude 4.0km, as e.g. in 
a radiosonde profile. The properties of each layer are calculated as
average over the adjacent levels. E.g. the single scattering
properties for the model layer between 3 and 4km are obtained by
averaging over the two levels 3km and 4km. To allow easy definition of
sharp cloud boundaries, clouds are only formed if both liquid water
contents above and below the respective layer are larger than
0. Hence, in the above example, the layers between 2 and 3 as well as
between 3 and 4km are cloudy while those between 1 and 2km and between
4 and 5km are not. 

@code{wc_layer} provides a useful alternative if one prefers to define
layer quantities. If specified, the properties are interpreted as
layer properties, and in the above example, the cloud would extend
from 2 to 5km, with e.g. a LWC of 0.2 g/m3 for the layer between 4 and
5km. To make sure that the clouds really look as you want them to look,
it is recommended to use the @code{verbose} option. This option shows
not only where the cloud is actually placed, it rather tells the user
exactly how LWC and effective radius are translated into optical
properties, depending on the choice of parameterisation. Please also
note that the definition of the empty top level at 5km is important to
tell uvspec where the cloud ends. If omitted, the cloud would
extend all the way to the top of the atmosphere.

There are different ways to convert the microphysical properties to
optical properties. Either a parameterization is used, like the one 
by Hu and Stamnes (1993) (which is the default), or by Mie
calculations. The latter are very time-consuming, hence we decided not
to include these online into uvspec but rather have an option to read
in pre-calculated Mie tables. The option @code{wc_properties} controls
the method: @code{hu} selects the Hu and Stamnes (1993)
parameterization, @code{mie} selects pre-calculated Mie tables which
are available at http://www.libradtran.org. If @code{wc_properties
mie} is selected, the model expects one Mie cloud property file for
each internal wavelength which is useful for the fixed wavelength
grids used by the correlated-k parameterisations 
@code{correlated_k kato}, @code{correlated_k fu}, etc. For a spectral
calculation with free wavelength grid, there is also the possibility
to use a pre-defined set of Mie tables (available at the web site) and
to defined @code{wc_properties_interpolate} to automatically
interpolate the Mie properties to the internal wavelength
grid. Although this is an extremely useful option, please use it
careful because it might consume enourmous amounts of memory. 
Finally, there is the
option to define an arbitrary file which is in the format as generated
by Frank Evans' @code{cloudprp} which comes with SHDOM, see 
http://nit.colorado.edu/~evans/shdom.html.

As for the aerosol, there are several options to modify the optical
properties of the clouds. And of course there is also the option of 
defining all cloud properties explicitely using @code{wc_files}.

@subsubsection Ice clouds

Ice clouds are generated in a similar way to water clouds.
All options to set up and modify ice cloud properties 
start with @code{ic_}. The main difference between water and ice clouds
is that the latter usually consist of non-spherical particles. Hence,
the conversion from microphysical to optical properties is much less
defined, and several parameterizations are available. Please note in
addition that there are different definitions of the effective radius
and e.g. the parameterizations by Key et al. (2002) and by Fu (1996)
actually use different definitions (see explanation of
@code{ic_properties}). Finally, the sharp forward peak which is
typical for ice particles is also treated differently: E.g. Fu (1996)
provides delta-scaled optical properties while Key et al. (2002) uses
unscaled parameters (see explanation of @code{ic_fu_tau}). The
following figure illustrates the implications.
Plotted are extinction coefficient, asymmetry parameter, and single
scattering albedo for ice clouds as a function of wavelength. If
treated consistently, both Key et al. (2002) and Fu (1996) provide
nearly identical results (blue and red solid line). However, Fu (1996)
uses a different definition of effective radius and recommends to use
the delta-scaled properties (red dotted line). The larger part of the
difference compared to the solid line is due to delta-scaling which
can be seen by comparing the red dotted (delta-scaled) and red-dashed
(not delta-scaled) lines. Lately, a new set of optical properties was
derived, combining the Key et al. (2002) data below 3.4 micron with
new calculations above 3.4 micron, to be invoked with
@code{ic_properties yang}.

@image{./fuyang}


@subsubsection Calculation of radiances

To calculate radiances the following lines will do the job when combined
with the clear sky example above

@example
@cartouche
@include ../examples/UVSPEC_RADIANCES.INP
@end cartouche
@end example

More examples with output are found in the @file{examples} directory.

      @c11_01@   */

  /***************** End of documentation *********************/

  int i=0;
  int brdftype=0;
  int status=0;
  int io=0;

#if HAVE_LIBNETCDF
  int ncid=0;
  int id_var_test=0;
#endif

  char infilename[FILENAME_MAX]="";

  char randomseedfilename [FILENAME_MAX] = "randomseed";
  FILE *randomseedfile=NULL;
  int rseed=0;

  char tmp_filename_solar [FILENAME_MAX] = "";
  FILE *f=NULL;

  /* char base[255]=""; */

  int time_UTC_in_s = NOT_DEFINED_INTEGER;
  int time_LAT_in_s = NOT_DEFINED_INTEGER;

  #if HAVE_LIBGSL 
    const gsl_rng_type *T;
  #endif

  
#ifndef UVSPEC_FUNCTION
  int c=0;
#endif

  char function_name[]="main";
  char file_name[]="uvspec_lex.l";

  /* Initialization */

  /* allocate memory for filenames */

  Input.filename         = calloc (FN_NN,     sizeof(char *));
  Input.aer.filename     = calloc (FN_AER_NN, sizeof(char *));
  Input.atm.filename     = calloc (MOL_NN,    sizeof(char *));
  Input.rte.mc.filename  = calloc (FN_MC_NN,  sizeof(char *));
  Output.crs.filename    = calloc (MOL_NN,    sizeof(char *));

  for (i=0; i<FN_NN; i++)
    Input.filename[i]        = calloc (FILENAME_MAX+1, sizeof(char));
  
  for (i=0; i<FN_AER_NN; i++) 
    Input.aer.filename[i]    = calloc (FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Input.atm.filename[i]    = calloc (FILENAME_MAX+1, sizeof(char));
 
  for (i=0; i<FN_MC_NN; i++)
    Input.rte.mc.filename[i] = calloc (FILENAME_MAX+1, sizeof(char));

  for (i=0; i<MOL_NN; i++) 
    Output.crs.filename[i]   = calloc (FILENAME_MAX+1, sizeof(char));

  strcpy (Input.filename[FN_PATH], "../data/");

  Input.quiet             = 0;
  Input.verbose           = 0;
  Input.output_format     = OUTPUT_FORMAT_ASCII;
  Input.header            = 0;
  Input.source            = SRC_SOLAR;               /* default: solar source     */
  Input.spectrum_unit     = UNIT_NOT_DEFINED;        /* default: unit of solar spectrum is unknown */
  Input.output_unit       = UNIT_NOT_DEFINED;        /* default: output units are unknown, same as unit of solar spectrum */
  Input.bandwidth         = 1.0;                     /* default: bandwidth in cm-1 (thermal) or 1nm (solar) */
  Input.bandwidth_unit    = UNIT_NOT_DEFINED;        /* cm-1 for thermal, nm for solar */
  Output.print_phi        = 1;                       /* default: print radiances   */
  Input.ck_scheme         = CK_NONE;
  Input.ck_h2ocont        = 1;                       /* default is h2o-continuum is on, version 2.1 */
  Input.ck_o4abs          = 1;
  Input.ck_n2abs          = 1;
  Input.ck_coabs          = 1;
  Input.ck_so2abs         = 1;
  Input.ck_nh3abs         = 1;
  Input.ck_noabs          = 1;
  Input.ck_hno3abs        = 1;

  Input.wc_cloudcover     = NOT_DEFINED_INTEGER;
  Input.ic_cloudcover     = NOT_DEFINED_INTEGER;
  Input.cloud_overlap     = NOT_DEFINED_INTEGER;

  Input.wcipa             = 0;
  Input.icipa             = 0;
  Input.convolve          = 0;
  Input.spline            = 0;
  Input.ozone_spectrum    = O3_MOLINA;
  Input.co2_spectrum      = CO2_YOSHINO;
  Input.o2_spectrum       = O2_DEFAULT;
  Input.no2_spectrum      = NO2_GOME;
  Input.bro_spectrum      = BRO_IASB;
  Input.oclo_spectrum     = OCLO_WAHNER;
  Input.hcho_spectrum     = HCHO_IASB;

  Input.UTC.tm_yday       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_year       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_mon        = NOT_DEFINED_INTEGER;
  Input.UTC.tm_mday       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_hour       = NOT_DEFINED_INTEGER;
  Input.UTC.tm_min        = NOT_DEFINED_INTEGER;
  Input.UTC.tm_sec        = NOT_DEFINED_FLOAT;

  Input.LAT.tm_yday       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_year       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_mon        = NOT_DEFINED_INTEGER;
  Input.LAT.tm_mday       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_hour       = NOT_DEFINED_INTEGER;
  Input.LAT.tm_min        = NOT_DEFINED_INTEGER;
  Input.LAT.tm_sec        = NOT_DEFINED_FLOAT;

  Input.precip_water        = NOT_DEFINED_INTEGER;
  Input.pressure            = NOT_DEFINED_INTEGER;
  Input.rayleigh_crs        = RAYLEIGH_BODHAINE;
  Input.surface_temperature = NOT_DEFINED_INTEGER;
  Input.rayleigh_depol      = NOT_DEFINED_INTEGER;

  Input.aer.standard      = 0;
  Input.aer.spec          = 0;
  Input.aer.alpha         = 0.0;
  Input.aer.beta          = NOT_DEFINED_FLOAT;
  Input.aer.haze          = 0;
  Input.aer.scale_ssa     = NOT_DEFINED_FLOAT;
  Input.aer.scale_tau     = NOT_DEFINED_FLOAT;
  Input.aer.tau550        = NOT_DEFINED_FLOAT;
  Input.aer.set_gg        = NOT_DEFINED_FLOAT;
  Input.aer.set_tau       = NOT_DEFINED_FLOAT;
  Input.aer.set_ssa       = -1.0;
  Input.aer.seasn         = 0;
  Input.aer.visibility    = NOT_DEFINED_FLOAT;
  Input.aer.vulcan        = 0;
  Input.aer.n_species     = NOT_DEFINED_INTEGER;          /* number of aerosol species */
  strcpy(Input.aer.filename[FN_AER_SPECIES_LIB],"OPAC");  /* default aerosol library   */


  Input.latitude          = NOT_DEFINED_FLOAT; 
  Input.longitude         = NOT_DEFINED_FLOAT; 
  Input.sat_pixel_x       = NOT_DEFINED_FLOAT;
  Input.sat_pixel_y       = NOT_DEFINED_FLOAT;

  Input.alb.source        = NOT_DEFINED_INTEGER;    /* first of all, no defined albedo, default setting see below */
  Input.alb.surface       = NOT_DEFINED_INTEGER;    /* there is no default */
  Input.alb.albedo        = 0.0;
  Input.alb.scale_factor  = 1.0;
  Input.alb.surf_type_map = FALSE;
  Input.alb.netCDF_alb_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.alb.netCDF_surf_name = calloc (FILENAME_MAX+1, sizeof(char));

  Input.alt.source        = ALT_NOT_DEFINED;
  Input.alt.altitude      = NOT_DEFINED_FLOAT;
  Input.alt.scale_factor  = 1.0;
  Input.alt.netCDF_alt_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.alt.altitude_dz   =    0.0;

  Input.netCDF_name_surf_T = calloc (FILENAME_MAX+1, sizeof(char));

  Input.atm.rs_source     = RS_NO_DATA;                /* default no radiosonde-file */

  Input.atm.sza           = NOT_DEFINED_FLOAT;        
  Input.atm.sza_source    = SZA_BY_TIME_AND_LOCATION;

  Input.atm.nzout         = 0;
  Input.atm.zout_source   = OUTLEVEL_ZOUT_ABOVE_SUR;
  Output.atm.n_z_model    = NOT_DEFINED_INTEGER;
  Input.atm.z_interpolate = Z_INTERPOLATE_LINMIX; /* alternative default: Z_INTERPOLATE_LOGLIN */
  Input.atm.interpol_method_press  = NOT_DEFINED_INTEGER;
  Input.atm.interpol_method_temper = NOT_DEFINED_INTEGER;
  Input.atm.interpol_method_refind = NOT_DEFINED_INTEGER;   
  Input.atm.interpol_method_gas = calloc(MOL_NN, sizeof(int));
  Input.atm.well_mixed_gas      = calloc(MOL_NN, sizeof(int));
  for (i=0; i<MOL_NN; i++) {
    Input.atm.interpol_method_gas[i] = NOT_DEFINED_INTEGER;
    Input.atm.well_mixed_gas     [i] = NOT_DEFINED_INTEGER;
  }
  Input.atm.ECMWF_new_format = NOT_DEFINED_INTEGER;
  Input.atm.zout_interpolate = NO_ZOUT_INTERPOLATE;
  Input.atm.time_interpolate = TIME_NEAREST_DATE;

  Input.brdf                = BRDF_NONE;
  Input.cm.u10              = NOT_DEFINED_FLOAT;
  Input.cm.pcl              = NOT_DEFINED_FLOAT;
  Input.cm.pcl_netCDF_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.cm.pcl_scale_factor = 1.0;
  Input.cm.sal              = NOT_DEFINED_FLOAT;
  Input.cm.sal_netCDF_name  = calloc (FILENAME_MAX+1, sizeof(char));
  Input.cm.sal_scale_factor = 1.0;

  Input.rpv.rho0          = 0.0;
  Input.rpv.k             = 0.0;
  Input.rpv.theta         = 0.0;
  Input.rpv.sigma         = 0.0;
  Input.rpv.t1            = 0.0;
  Input.rpv.t2            = 0.0;
  Input.rpv.scale         = 1.0;
  Input.rpv.source        = RPV_CONSTANT;

  Input.calibration       = OUTCAL_ABSOLUTE;
  Input.rte.fisot         = 0;
  Input.rte.ichap         = 1;
  Input.rte.maxphi        = 0;
  Input.rte.nscat         = 20;
  Input.rte.nrefrac       = 0;
  Input.rte.nstr          = 6;
  Input.rte.solver        = DISORT2;
  Input.rte.pol_stokes[0] = 'I';
  Input.rte.pol_stokes[1] = 'Q';
  Input.rte.pol_stokes[2] = 'U';
  Input.rte.pol_stokes[3] = 'V';

  Input.rte.mc.absorption    = MCABS_NONE;
  Input.rte.mc.photons       = 100000;
  Input.rte.mc.Nx_sample     = 0;
  Input.rte.mc.Ny_sample     = 0;
  Input.rte.mc.Nr            = 0;
  Input.rte.mc.Nt            = 0;
  Input.rte.mc.dt            = 1e-7; /* 100 ns, 30m */
  Input.rte.mc.passback3D    = 0;
  Input.rte.mc.ipa           = 0;
  Input.rte.mc.delta_scaling_mucut = NOT_DEFINED_INTEGER; /* TZ ds */
  Input.rte.mc.truncate            = NOT_DEFINED_INTEGER;
  Input.rte.mc.bcond         = MCBCOND_PERIODIC;
  Input.rte.mc.reflectalways = 0;
  Input.rte.mc.spherical     = 0;
  Input.rte.mc.std           = 0;
  Input.rte.mc.randomseed    = NOT_DEFINED_INTEGER;
 
  Input.rte.mc.backward.yes = 0;
  Input.rte.mc.backward.output = MCBACKWARD_EDN;
  
  Input.rte.mc.azimuth    = MCAZIMUTH_NEW;

  Input.wl.start          = NOT_DEFINED_INTEGER;
  Input.wl.end            = NOT_DEFINED_INTEGER;
  Input.wl.start_index    = NOT_DEFINED_INTEGER;
  Input.wl.end_index      = NOT_DEFINED_INTEGER;

  Input.rte.nphi          = 0;
  Input.rte.numu          = 0;
  Input.rte.nprndis       = 0;
  Input.rte.deltam        = 1;
  Input.rte.pol_aziorder  = 0;
  Input.rte.pol_max_delta_tau = 1e-5;
  Input.rte.pol_nstokes   = 1;
  Input.rte.pol_quad_type = "G";
  Input.rte.pol_src_code  = 1;
  Input.rte.reverse       = 0;
  Input.wc.source         = NO_CLOUDS;
  Input.wc.tau            = NOT_DEFINED_INTEGER;
  Input.wc.gg             = NOT_DEFINED_INTEGER;
  Input.wc.ssa            = NOT_DEFINED_INTEGER;
  Input.wc.scale_gg       = NOT_DEFINED_INTEGER;
  Input.wc.scale_ssa      = NOT_DEFINED_INTEGER;
  Input.wc.tau550         = NOT_DEFINED_INTEGER;
  Input.wc.layer          = 0;
  Input.wc.properties     = WCPROP_NONE;
  Input.wc.unscaled       = 0;
  Input.wc.interpolate    = 0;
  Input.wc.wv_saturation  = SATURATION_NONE;
  Input.wc.wv_sat_ratio   = 100.0;
  Input.ic.source         = NO_CLOUDS;
  Input.ic.tau            = NOT_DEFINED_INTEGER;
  Input.ic.gg             = NOT_DEFINED_INTEGER;
  Input.ic.ssa            = NOT_DEFINED_INTEGER;
  Input.ic.scale_gg       = NOT_DEFINED_INTEGER;
  Input.ic.scale_ssa      = NOT_DEFINED_INTEGER;
  Input.ic.tau550         = NOT_DEFINED_INTEGER;
  Input.ic.layer          = 0;
  Input.ic.properties     = ICPROP_NONE;
  Input.ic.interpolate    = 0;
  Input.ic.wv_saturation  = SATURATION_NONE;
  Input.ic.wv_sat_ratio   = 100.0;
  Input.ic.unscaled       = 0;
  Input.ic.habit          = IC_HABIT_SOLIDCOLUMN;
  Input.absorption        = 1;
  Input.molabs            = MOLABS_CALC;
  Input.rayleigh          = RAYLEIGH_CALC;
  Input.scattering        = 1;
  Input.mixing_ratio = calloc(MX_NN, sizeof(float));
  for (i=0; i<MX_NN; i++)
    Input.mixing_ratio[i] = NOT_DEFINED_INTEGER;
  Input.atm.column       = calloc(MOL_NN, sizeof(float));
  for (i=0; i<MOL_NN; i++) 
    Input.atm.column[i] = NOT_DEFINED_INTEGER;
  Input.atm.unit_column           = calloc(MOL_NN, sizeof(int));
  Input.atm.unit_column[MOL_AIR]  = CM_2;
  Input.atm.unit_column[MOL_O3]   = DU;
  Input.atm.unit_column[MOL_O2]   = CM_2;
  Input.atm.unit_column[MOL_H2O]  = CM_2;
  Input.atm.unit_column[MOL_CO2]  = CM_2;
  Input.atm.unit_column[MOL_NO2]  = CM_2;
  Input.atm.unit_column[MOL_BRO]  = CM_2;
  Input.atm.unit_column[MOL_OCLO] = CM_2;
  Input.atm.unit_column[MOL_HCHO] = CM_2;
  Input.atm.unit_profile           = calloc(MOL_NN, sizeof(int));
  Input.atm.unit_profile[MOL_AIR]  = CM_3;
  Input.atm.unit_profile[MOL_O3]   = CM_3;
  Input.atm.unit_profile[MOL_O2]   = CM_3;
  Input.atm.unit_profile[MOL_H2O]  = CM_3;
  Input.atm.unit_profile[MOL_CO2]  = CM_3;
  Input.atm.unit_profile[MOL_NO2]  = CM_3;
  Input.atm.unit_profile[MOL_BRO]  = CM_3;
  Input.atm.unit_profile[MOL_OCLO] = CM_3;
  Input.atm.unit_profile[MOL_HCHO] = CM_3;
  Input.atm.well_mixed_gas [MOL_AIR]  = YES;
  Input.atm.well_mixed_gas [MOL_O3]   = NO;
  Input.atm.well_mixed_gas [MOL_O2]   = YES;
  Input.atm.well_mixed_gas [MOL_H2O]  = NO;
  Input.atm.well_mixed_gas [MOL_CO2]  = YES;
  Input.atm.well_mixed_gas [MOL_NO2]  = NO;
  Input.atm.well_mixed_gas [MOL_BRO]  = NO;
  Input.atm.well_mixed_gas [MOL_OCLO] = NO;
  Input.atm.well_mixed_gas [MOL_HCHO] = NO;
  Input.atm.n_rs_gas = 0;
  Input.atm.rs_gas  = calloc(MOL_NN, sizeof(int));
  Input.atm.rs_unit = calloc(MOL_NN, sizeof(int));
  Input.atm.rs_unit[MOL_AIR]  = CM_3;
  Input.atm.rs_unit[MOL_O3]   = MMR;
  Input.atm.rs_unit[MOL_O2]   = CM_3;
  Input.atm.rs_unit[MOL_H2O]  = RH;
  Input.atm.rs_unit[MOL_CO2]  = CM_3;
  Input.atm.rs_unit[MOL_NO2]  = CM_3;
  Input.atm.rs_unit[MOL_BRO]  = CM_3;
  Input.atm.rs_unit[MOL_OCLO] = CM_3;
  Input.atm.rs_unit[MOL_HCHO] = CM_3;
  Input.atm.mol_mass = calloc(MOL_NN, sizeof(float));
  Input.atm.mol_mass[MOL_AIR]  = mol_mass_air; /* from function ck.c in g/MOL */
  Input.atm.mol_mass[MOL_O3]   = 48.000;       /* from function ck.c in g/MOL */
  Input.atm.mol_mass[MOL_O2]   = 31.998609;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_H2O]  = 18.015;       /* from function ck.c */
  Input.atm.mol_mass[MOL_CO2]  = 44.009646;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_NO2]  = 46.005333;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_BRO]  = 95.902831;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_OCLO] = 67.451347;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */
  Input.atm.mol_mass[MOL_HCHO] = 30.026294;    /* http://www.ch.cam.ac.uk/magnus/MolWeight.html in g/MOL */

  Input.processing         = PROCESS_NONE;
  Input.n_output_user      = NOT_DEFINED_INTEGER;

  
#ifndef UVSPEC_FUNCTION
  while ((c=getopt (argc, argv, "hvf:i:")) != EOF)  {
    switch(c)  {
    case 'h': 
      usage();
      return (-1);
      break;
    case 'f':
    case 'i':
      strcpy (infilename, optarg);
      break;
    case 'v': 
      version();
      return (-1);
      break;
    default:
      usage();
      return (-1);
    }
  }
#endif

  /* copy protection :-) */
  #ifndef UVSPEC_FUNCTION
  /*
  if ((int) time(NULL) > 1166372390) {
    remove(argv[0]);
    return -1;
  }
  else
    fprintf (stderr, " ... valid for %d days, %d hours, ...\n", 
		     (1166372390-(int)time(NULL))/86400, 
		     ((1166372390-(int)time(NULL))%86400)/60);
  */
  #endif  


  /***********************/
  /* Read the input file */
  /***********************/

  if (strlen(infilename)>0)
    yyin = fopen (infilename, "r");

  yylex();

  if (ierror!=0)  {
    fprintf (stderr, "Error in input file! Please check your input file and retry!\n");
    return -1;
  }

  if (strlen(infilename)>0)
    fclose (yyin);

  /*****************************************/
  /* check combinition of parameters       */
  /* and do input dependent initialisation */
  /*****************************************/


  /* if uvspec is told to be verbose, it cannot be quiet at the same time */
  if (Input.verbose)
    Input.quiet = 0;

  /* To calculate radiances, we want to increase nstr to at least 16 */
  if (Input.rte.numu > 0){
  if (Input.rte.nstr<16) {
        if(!Input.quiet)
          fprintf (stderr, " ... setting nstr to 16 for radiance calculation!\n");
	Input.rte.nstr=16;
        }
    if(Input.rte.solver==POLRADTRAN){
        if(!Input.quiet){
        fprintf (stderr, " ... setting polradtran parameters:\n");
        fprintf (stderr, " ... polradtran_aziorder 4\n");
        fprintf (stderr, " ... polradtran_quad_type E\n");
        }
        Input.rte.pol_aziorder  = 4;
        Input.rte.pol_quad_type = "E";
      }
    }
  


  if (Input.rte.numu>0 && Input.rte.solver==DISORT) {
    fprintf (stderr, "\n");
    fprintf (stderr, "*** Warning, you want to calculate radiances with DISORT.\n");
    fprintf (stderr, "*** It is strongly recommended to use DISORT2 in this case because\n");
    fprintf (stderr, "*** the older DISORT version uses only the first NSTR moments of the\n");
    fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
    fprintf (stderr, "*** DISORT2 is set up to consider at least the first 1024 moments.\n");
    fprintf (stderr, "\n");
  }


  if (Input.atm.nzout == 0) {
    Input.atm.nzout    = 1;
    Input.atm.zout_sur = (float *) calloc (Input.atm.nzout, sizeof(float));

    if (Input.rte.solver == MONTECARLO)
      Input.atm.zout_sur[0] = -999.0;
    else 
      Input.atm.zout_sur[0] = 0.0;
  }


  /* surface temperature, this is == NOT_DEFINED_FLOAT, if not specified in the input file */
  Output.surface_temperature = Input.surface_temperature;


  /* define default output as a user defined output */
  if (Input.n_output_user < 0 ) {
    if (Input.heating == NO_HEAT) {
      switch (Input.rte.solver) {
      case TWOSTRPP:
      case TWOSTR:
        Input.n_output_user = 5;
        Input.output_user   = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        Input.output_user[1] = OUTPUT_USER_EDIR;
        Input.output_user[2] = OUTPUT_USER_EDN;
        Input.output_user[3] = OUTPUT_USER_EUP;
        Input.output_user[4] = OUTPUT_USER_U;
      break;
      case DISORT:
      case DISORT2:
      case SDISORT:
      case SPSDISORT:
      case QDISORT:
      case SUCORD:
      case MONTECARLO:
      case SSS:
      case TZS:
      case SSSI:
      case SSS2:
      case NULLSOLVER:
        Input.n_output_user = 7;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        Input.output_user[1] = OUTPUT_USER_EDIR;
        Input.output_user[2] = OUTPUT_USER_EDN;
        Input.output_user[3] = OUTPUT_USER_EUP;
        Input.output_user[4] = OUTPUT_USER_UDIR;
        Input.output_user[5] = OUTPUT_USER_UDN;
        Input.output_user[6] = OUTPUT_USER_UUP;
        break;
      case POLRADTRAN:
        /* polradtran has its special output function, do nothing here */
        break;
      default:
        fprintf (stderr, "Error, unknown rte_solver %d (in uvspec_lex.l)\n", Input.rte.solver);
        return -2;
      }
    }
    else { 

      /* default output for spectral heating rates, as a 2D matrix */
      if (Input.processing != PROCESS_INT && Input.processing != PROCESS_SUM) {

        Input.output_format = OUTPUT_FORMAT_MATRIX; 

        Input.n_output_user = 3;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        Input.output_user[0] = OUTPUT_USER_WAVE;
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
          Input.output_user[1] = OUTPUT_USER_ZOUT_SUR;
          break;
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          Input.output_user[1] = OUTPUT_USER_ZOUT_SEA;
          break;
        case OUTLEVEL_PRESS:
          Input.output_user[1] = OUTPUT_USER_P;
          break;
        default:
          fprintf (stderr, "Error, unknown zout marker %d when defining default output (uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        Input.output_user[2] = OUTPUT_USER_HEAT;
      }
      else { /* integrated output */
        Input.n_output_user = 2;
        Input.output_user = calloc (Input.n_output_user, sizeof (int));
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
          Input.output_user[0] = OUTPUT_USER_ZOUT_SUR;
          break;
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          Input.output_user[0] = OUTPUT_USER_ZOUT_SEA;
          break;
        case OUTLEVEL_PRESS:
          Input.output_user[0] = OUTPUT_USER_P;
          break;
        default:
          fprintf (stderr, "Error, unknown zout marker %d when defining default output (uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        Input.output_user[1] = OUTPUT_USER_HEAT;
      }
    }
  }

  /* if output_user requires heating_rate -> set heating_rate default calculation method */
  if (Input.heating == NO_HEAT) {
    for (io=0; io<Input.n_output_user; io++) {
      if (Input.output_user[io] == OUTPUT_USER_HEAT) { 
        Input.heating = HEAT_LAYER_CD;
        if (!Input.quiet)
          fprintf (stderr, " ... heating rates calculated with central differences\n");
      }
    }
  }

  /* if output_user any horizontal derivations -> atmosphere maps and wind file are mandatory */
  for (io=0; io<Input.n_output_user; io++) {
    if ( Input.output_user[io] == OUTPUT_USER_HEAT_AD_X || Input.output_user[io] == OUTPUT_USER_DTDX ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD_Y || Input.output_user[io] == OUTPUT_USER_DTDY ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD ) {
      if ( (strlen(Input.filename[FN_ECMWF]) == 0 && strlen(Input.filename[FN_ECHAM]) == 0) ) {
        fprintf (stderr, "Error. for output_user dTdx, dTdy, or any heat advection  \n");
        fprintf (stderr, "       An 3D atmosphere file in netCDF format are nessesary \n");
        return -1;
      }
    }
  }

  /* if output_user any horizontal derivations -> atmosphere maps and wind file are mandatory */
  for (io=0; io<Input.n_output_user; io++) {
    if ( Input.output_user[io] == OUTPUT_USER_HEAT_AD_X ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD_Y ||
         Input.output_user[io] == OUTPUT_USER_HEAT_AD ) {
      if ( strlen(Input.filename[FN_ECMWF_WIND_MAP]) == 0 ) {
        fprintf (stderr, "Error. for output_user heat_ad_x, heat_ad_y, or heat_ad  \n");
        fprintf (stderr, "       a wind file in netCDF format are nessesary \n");
        return -1;
      }
    }
  }

  /* activate z_interpolate for heating rate calculations */
  if (Input.heating != NO_HEAT && Input.atm.zout_interpolate == NO_ZOUT_INTERPOLATE) {
    Input.atm.zout_interpolate = ZOUT_INTERPOLATE;
    if (!Input.quiet)
      fprintf (stderr, " ... zout_interpolate is turned on by default for heating rates calculations\n");
  }

  /* check, if only time or day_of_year are used */
  if (Input.UTC.tm_yday > 0 && Input.UTC.tm_mday > 0) {
    fprintf (stderr, "Error. Either 'day_of_year' or 'time' can be specified  \n");
    fprintf (stderr, "       in the input file, but not both. Please remove one of these lines. \n");
    return -1; 
  }

  /* copy date to uninitialised UTC structure, if LAT input */
  if (Input.time_zone == TIME_LAT) {
    Input.UTC.tm_year = Input.LAT.tm_year;
    Input.UTC.tm_mon  = Input.LAT.tm_mon;
    Input.UTC.tm_mday = Input.LAT.tm_mday;
  }

  /* calculate day of year */
  if ( Input.UTC.tm_mday > 0. && (Input.UTC.tm_mon + 1) > 0. ) { /* +1 == C-standard Jan=0, Feb=1, ...*/
    if ((Input.UTC.tm_yday = day_of_year(Input.UTC.tm_mday, (Input.UTC.tm_mon+1))) < 0)  { /* +1 == C-standard Jan=0, Feb=1, ...*/
      fprintf (stderr, "\n*** Error, calculating day_of_year, (in uvspec_lex.l)");
      fprintf (stderr, "\n    Date out of range! day = %d, month = %d\n", Input.UTC.tm_mday, Input.UTC.tm_mon+1);
      return -1;
    }
  }
  Input.LAT.tm_yday = Input.UTC.tm_yday;
  

  /* convert local apperent time to UTC, needs correct day_of_year */
  switch (Input.time_zone) {
  case TIME_UTC:
    /* do nothing already UTC */
    /* fprintf (stderr, " *** UTC! ***  year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d\n", */
    /*                  Input.UTC.tm_year+1900, Input.UTC.tm_mon+1, Input.UTC.tm_mday, Input.UTC.tm_hour, Input.UTC.tm_min, Input.UTC.tm_sec ); */
    break;
  case TIME_LAT:

    if (Input.verbose) {
      fprintf (stderr, " ... Convert local time to UTC\n"); 
      fprintf (stderr, "     local apparent time:        year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d, longitude = %7.2f \n",
                     Input.LAT.tm_year+1900, Input.LAT.tm_mon+1, Input.LAT.tm_mday, Input.LAT.tm_hour, Input.LAT.tm_min, Input.LAT.tm_sec, Input.longitude );
    }

    time_LAT_in_s = (int) ( 3600*Input.LAT.tm_hour + 60*Input.LAT.tm_min + Input.LAT.tm_sec );

    /* convert LAT to UTC, last argument standard longitude of UTC, ATTENTION: for function LAT() POSITIVE longitude is WEST */
    time_UTC_in_s = standard_time ( time_LAT_in_s, Input.LAT.tm_yday, -Input.longitude, 0.0); 

    /* correct negative UTCs and UTCs larger than 24:00 */
    /* to get human-readable output                     */ 
    while (time_UTC_in_s<0)
      time_UTC_in_s += 86400; 
    while (time_UTC_in_s>86400)
      time_UTC_in_s -= 86400;

    Input.UTC.tm_hour =  time_UTC_in_s / 3600;
    Input.UTC.tm_min  = (time_UTC_in_s - Input.UTC.tm_hour*3600) / 60;
    Input.UTC.tm_sec  = (time_UTC_in_s - Input.UTC.tm_hour*3600 - Input.UTC.tm_min*60);
           
    if (Input.verbose) {
      fprintf (stderr, "     universal time correlated:  year=%4d, month=%2d, day=%2d, hour=%2d, min=%2d, sec=%2d\n",
                     Input.UTC.tm_year+1900, Input.UTC.tm_mon+1, Input.UTC.tm_mday, Input.UTC.tm_hour, Input.UTC.tm_min, Input.UTC.tm_sec );
    }

    break;
  default:
    fprintf (stderr, "Error, unknown time_zone %d!\n", Input.time_zone);
    fprintf (stderr, "This is a program bug, please contact the programmers!\n");
    return -1;
  }

  /* check that both satellite_pixel and satellite_geometry are activated */
  if (Input.sat_pixel_x > 0 && strlen(Input.filename[FN_SATGEOMETRY]) == 0) {
    fprintf (stderr, "Error, satellite_pixel is specified, but no satellite geometry is given\n");
    return -1; 
  }
  if (Input.sat_pixel_x < 0 && strlen(Input.filename[FN_SATGEOMETRY]) > 0) {
    fprintf (stderr, "Error, satellite geometry is given, but no satellite_pixel is specified\n");
    return -1; 
  }


  /* set default SATELLITE GEOMETRY map for MSG */
  if (strncasecmp("MSG",Input.filename[FN_SATGEOMETRY],3)==0) {
    strcpy (Input.filename[FN_SATGEOMETRY], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_SATGEOMETRY], "satellites/MSG_seviri/MSG_seviri_geometry.nc");
  }
  if (strlen(Input.filename[FN_SATGEOMETRY]) > 0 && Input.output_format == OUTPUT_FORMAT_NETCDF && !Input.quiet)
    fprintf (stderr, "*** Warning, using satellite geometry, but not 'output_format sat_picture'\n");

  /* initialise satellite geometry */
  if (strlen(Input.filename[FN_SATGEOMETRY]) > 0) {
    /* read satellite geometry: lat/lon/theta_sat/phi_sat */
    status = read_sat_geometry (Input.sat_pixel_x, Input.sat_pixel_y, Input.UTC, Input.filename[FN_SATGEOMETRY], 
                                &(Input.latitude), &(Input.longitude), 
                                &(Input.rte.numu),&(Input.rte.maxumu), &(Input.rte.umu),
                                &(Input.rte.nphi),&(Input.rte.maxphi), &(Input.rte.phi),
                                Input.rte.solver, Input.verbose, Input.quiet);    /* in ancillary.c */
    if (status!=0) {
      fprintf (stderr, "Error %d reading satellite geometry from %s in uvspec_lex.l\n", status, Input.filename[FN_SATGEOMETRY]);
      return status;
    }
    if (Input.latitude == 999) {
      /* if (!Input.quiet) */
      /*  fprintf (stderr, "Pixel outside Earth plate, skip this calculation\n"); */
      return 0;
      /* Input.rte.solver = NULLSOLVER; */
      /* Input.latitude  = 0.0; */
      /* Input.longitude = 0.0; */
    }
  }

  /* if no atmosphere file is given, check if we can find a appropriate atmosphere */
  if (strlen(Input.filename[FN_ATMOSPHERE]) == 0) {
    /* if lat, lon AND time is defined than search suitable atmosphere file */
    if ( ( fabs(Input.latitude  - NOT_DEFINED_FLOAT) > EPSILON ) &&
         ( fabs(Input.longitude - NOT_DEFINED_FLOAT) > EPSILON ) &&
         ( Input.UTC.tm_mday > 0 ) ) {

      if ( fabs (Input.latitude) < 23.0 )
        strcpy (Input.filename[FN_ATMOSPHERE], "tropics");
      else if ( fabs (Input.latitude) < 67.0 )
        strcpy (Input.filename[FN_ATMOSPHERE],"midlatitude");
      else 
        strcpy (Input.filename[FN_ATMOSPHERE],"subarctic");

      if ( fabs (Input.latitude) >= 23.0 ) { 
        if ( ( Input.latitude > 0 &&  (Input.UTC.tm_mon+1) >=  4 && (Input.UTC.tm_mon+1) <=  9  ) || 
             ( Input.latitude < 0 && ((Input.UTC.tm_mon+1) >= 10 || (Input.UTC.tm_mon+1) <=  3 )) )
          strcat (Input.filename[FN_ATMOSPHERE], "_summer");
        else 
          strcat (Input.filename[FN_ATMOSPHERE], "_winter");
      }
    }
    else /* not all nessesary information are given, choose standard atmosphere */
      strcpy (Input.filename[FN_ATMOSPHERE],"US-standard");

    if (Input.verbose)
      fprintf (stderr, " ... choosing atmosphere file %s\n",Input.filename[FN_ATMOSPHERE]);
  }
  


  /* replace keywords by atmosphere path + files */
  if      (strncasecmp("subarctic_winter",  Input.filename[FN_ATMOSPHERE],16)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglsw.dat");
  }
  else if (strncasecmp("subarctic_summer",  Input.filename[FN_ATMOSPHERE],16)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglss.dat");
  }
  else if (strncasecmp("midlatitude_summer",Input.filename[FN_ATMOSPHERE],18)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglms.dat");
  }
  else if (strncasecmp("midlatitude_winter",Input.filename[FN_ATMOSPHERE],18)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglmw.dat");
  }
  else if (strncasecmp("tropics",           Input.filename[FN_ATMOSPHERE], 7)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglt.dat");
  }
  else if (strncasecmp("US-standard",       Input.filename[FN_ATMOSPHERE],11)==0) {
    if (strlen(Input.filename[FN_PATH])>0)
      strcpy (Input.filename[FN_ATMOSPHERE],  Input.filename[FN_PATH]);
    else 
      strcpy (Input.filename[FN_ATMOSPHERE],  "../data/");
    strcat (Input.filename[FN_ATMOSPHERE], "atmmod/afglus.dat");
  }


  /* try to attach default path for solar_flux file, if file does not exit */
  if (strlen(Input.filename[FN_EXTRATERRESTRIAL]) > 0) {
    if ((f = fopen(Input.filename[FN_EXTRATERRESTRIAL], "r")) == NULL) {

      /* file does not exit, try to add an default path */
      if (strlen(Input.filename[FN_PATH])>0)
        strcpy (tmp_filename_solar,  Input.filename[FN_PATH]);
      else
        strcpy (tmp_filename_solar,  "../data/");
      strcat (tmp_filename_solar, "solar_flux/");
      strcat (tmp_filename_solar, Input.filename[FN_EXTRATERRESTRIAL]);

      /* try to open file with default path */
      if ((f = fopen(tmp_filename_solar, "r")) == NULL) {
        /* (default path + file) does not exits, also */
        /* final check if the solar spectrum is needed is done in */
        /* setup_extraterrestrial() (in extraterrestrial.c) */
      }
      else {
        /* (default path + file) does exit => copy filename */
        fclose(f);
        strcpy (Input.filename[FN_EXTRATERRESTRIAL],tmp_filename_solar);
      }
    }
    else
      fclose(f);
  }


  /* specify the individual interpolation methods for all gases*/
  /* !!! ATTENTION !!!: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR !!! */ 
  switch (Input.atm.z_interpolate) {
  case Z_INTERPOLATE_LOGLIN:    
    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_LINEAR;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_LINEAR; /* ozone is assumed to be not well-mixed which */
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)    
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LOG;    /* is certainly true for the stratosphere      */
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_LINEAR; /* NO2 is assumed to be not well-mixed which   */
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LOG;    /* is certainly true for the stratosphere      */ 
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LOG;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LOG;    
    break;
  case Z_INTERPOLATE_LINMIX:
    /* LINMIX means interpolation assuming linear mixing ratio for gases O3, ..., HCLO while       */
    /* air density is assumed to vary logarithically                                               */ 
    /* densities profiles interpolated             with LINMIX  */
    /* columns are summed up                       with LINMIX  */ 
    /* optical properties per layer are calculated with LINMIX  */
    /* in general: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR        */  

    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_LINEAR; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_LINEAR;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)    
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_LINMIX; 
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LINMIX;  
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LINMIX;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LINMIX;  
    break;
  case Z_INTERPOLATE_SPLINE:
    /*        CONTINUOUS FIRST DERIVATIVES, BUT COLUMNS AND OPTICAL PROPERTIES NOT CONSISTENT              */
    /* SPLINE means spline interpolation for additional levels zout and altitude levels as specified below */
    /* columns summed up  with LINMIX assumption (this might be improved, if nessesary)                    */   
    /* optical properties with LINMIX assumption (this might be improved, if nessesary)                    */
    /* in general: INTERP_METHOD_LINMIX not possible for press, temper, refind, rh, MOL_AIR                */

    if (Input.atm.interpol_method_press  < 0) 
        Input.atm.interpol_method_press  = INTERP_METHOD_LOG_SPLINE; /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_temper < 0)
        Input.atm.interpol_method_temper = INTERP_METHOD_SPLINE; 
    if (Input.atm.interpol_method_refind < 0)
        Input.atm.interpol_method_refind = INTERP_METHOD_SPLINE;

    if (Input.atm.interpol_method_gas[MOL_AIR]  < 0)
        Input.atm.interpol_method_gas[MOL_AIR]  = INTERP_METHOD_LOG_SPLINE;    /* do not make changes here, it will not affect all important parts*/
    if (Input.atm.interpol_method_gas[MOL_O3]   < 0)
        Input.atm.interpol_method_gas[MOL_O3]   = INTERP_METHOD_SPLINE; 
    if (Input.atm.interpol_method_gas[MOL_O2]   < 0)
        Input.atm.interpol_method_gas[MOL_O2]   = INTERP_METHOD_LOG_SPLINE; 
    if (Input.atm.interpol_method_gas[MOL_H2O]  < 0)
        Input.atm.interpol_method_gas[MOL_H2O]  = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_CO2]  < 0)
        Input.atm.interpol_method_gas[MOL_CO2]  = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_NO2]  < 0)
        Input.atm.interpol_method_gas[MOL_NO2]  = INTERP_METHOD_SPLINE;
    if (Input.atm.interpol_method_gas[MOL_BRO]  < 0)
        Input.atm.interpol_method_gas[MOL_BRO]  = INTERP_METHOD_LOG_SPLINE;  
    if (Input.atm.interpol_method_gas[MOL_OCLO] < 0)
        Input.atm.interpol_method_gas[MOL_OCLO] = INTERP_METHOD_LOG_SPLINE;    
    if (Input.atm.interpol_method_gas[MOL_HCHO] < 0)
        Input.atm.interpol_method_gas[MOL_HCHO] = INTERP_METHOD_LOG_SPLINE;  
    break;
  default:
    fprintf (stderr, "Error, unknown interpolation method z_interpolate %d!\n", Input.atm.z_interpolate);
    fprintf (stderr, "This is a program bug, please contact the programmers!\n");
    return -1;
    break;
  }

  if (Input.atm.interpol_method_temper != INTERP_METHOD_LINEAR &&
      Input.atm.interpol_method_temper != INTERP_METHOD_LOG && 
      Input.atm.interpol_method_temper != INTERP_METHOD_SPLINE) {
    fprintf (stderr, "Error, interpolation method %d not possible\n", Input.atm.interpol_method_temper);
    fprintf (stderr, "for vertical interpolation of temperature.\n");
    return -1;
  }

  /* check if EITHER one of o2_mixing_ratio OR dens_column[O2] is given in input file */
  if (Input.atm.column[MOL_O2] >= 0.0 && Input.mixing_ratio[MX_O2] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column O2' or 'o2_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1;
  }

  /* check if EITHER one of precip_water OR dens_column[H2O] is given in input file */
  if (Input.atm.column[MOL_H2O] >= 0.0 && Input.precip_water >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column H2O' or 'h2o_precip' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1; 
  }

  /* check if EITHER one of h2o_mixing_ratio OR dens_column[HO2] is given in input file */
  if (Input.atm.column[MOL_H2O] >= 0.0 && Input.mixing_ratio[MX_H2O] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column H2O' or 'h2o_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1;
  }

  /* check if EITHER one of co2_mixing_ratio OR dens_column[CO2] is given in input file */
  if (Input.atm.column[MOL_CO2] >= 0.0 && Input.mixing_ratio[MX_CO2] >= 0.0) {
    fprintf (stderr, "Error, either 'dens_column CO2' or 'co2_mixing_ratio' can be specified\n");
    fprintf (stderr, "in the input file but not both. Please remove one of these lines!\n");
    return -1; 
  }

  /* no user-defined transmittance grid if correlated-k */
  if ((Input.ck_scheme!=CK_NONE && Input.ck_scheme!=CK_LOWTRAN) && strlen(Input.filename[FN_WLTRANS])>0) {
    strcpy (Input.filename[FN_WLTRANS], "");
    fprintf (stderr, " ... option transmittance_wl_file ignored\n");
    fprintf (stderr, " ... (does not make sense with correlated-k)\n");
  }

  if (Input.alb.source == NOT_DEFINED_INTEGER) {
    /* default setting */
    Input.alb.source = ALBEDO_CONSTANT;
  }

  /* set default SURFACE TYPE map */
  if (strncasecmp("IGBP",Input.filename[FN_SURFACE_TYPE_MAP],4)==0) {
    strcpy (Input.filename[FN_SURFACE_TYPE_MAP], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_SURFACE_TYPE_MAP], "albedo/IGBP_map/SURFACE_TYPE_IGBP_10min.cdf");

    if (Input.alb.source != ALBEDO_IGBP_LIBRARY && !Input.quiet)
      fprintf (stderr, "*** Warning, using IGBP map, but not 'albedo_library IGBP'\n");
  }

  /* message to the user because of name change */
  if (strncasecmp("IGBP",Input.filename[FN_ALBEDO_MAP],4)==0) {
    fprintf (stderr, "\n*** ERROR, starting from libRadtran version 1.2 \n");
    fprintf (stderr,   "           'albedo_map' contains broadband albedo values \n");
    fprintf (stderr,   "           and not surface_type information any more. \n");
    fprintf (stderr,   "           Sorry for the inconvenience! \n");
    fprintf (stderr,   "           please use instead: \n");
    fprintf (stderr,   "           surface_type_map IGBP \n");
    return -1;
  }

  /* set default SURFACE TYPE netCDF variable name */
  if ( strlen(Input.alb.netCDF_surf_name) == 0 )
    strcpy (Input.alb.netCDF_surf_name, "surface_type");

  /* set default ALBEDO/EMISSIVITY netCDF variable name */
  if ( strlen(Input.alb.netCDF_alb_name) == 0 ) {
    if (Input.alb.source == ALBEDO_FROM_EMISSIVITY_MAP) 
      strcpy (Input.alb.netCDF_alb_name, "EMIS");
    else
      strcpy (Input.alb.netCDF_alb_name, "AL");
  }

  /* set default RPV Polder library path  */
  if (Input.rpv.source == RPV_IGBP_LIBRARY) {

    strcpy (Input.filename[FN_RPV_LIB_PATH], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_RPV_LIB_PATH], "albedo/IGBP_rpv_polder/");

  }

  /* set default ALTITUDE map */
  if (strncasecmp("GTOPO",Input.filename[FN_ALTITUDE_MAP],5)==0) {
    strcpy (Input.filename[FN_ALTITUDE_MAP], Input.filename[FN_PATH]);
    strcat (Input.filename[FN_ALTITUDE_MAP], "altitude/ELEVATION_GTOPO_10min.cdf");
  }


  /* check format of the ECMWF file */
  if (strlen(Input.filename[FN_ECMWF])>0) {

#if HAVE_LIBNETCDF

    /* open netcdf file */
    status = nc_open (Input.filename[FN_ECMWF], NC_NOWRITE, &ncid);
    if (status!=NC_NOERR) {
      fprintf (stderr, "Error %d opening netCDF file %s\n", status, Input.filename[FN_ECMWF]);
      return status;
    }
    status = nc_inq_varid (ncid, "z", &id_var_test);
    if (status==NC_NOERR) {
      Input.atm.ECMWF_new_format=TRUE;
    }
    else {
      status = nc_inq_varid (ncid, "Z", &id_var_test);
      if (status==NC_NOERR) {
        Input.atm.ECMWF_new_format=FALSE;
      }
      else {
        fprintf (stderr, "Error, unknown format of the ECMWF atmosphere file %s\n", Input.filename[FN_ECMWF]);
        return status;
      }
    }
    nc_close (ncid);

#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the ECMWF_atmosphere_file option. Please get netcdf and rebuild.*\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

  }
  

  /* set default ALTITUDE netCDF variable name */
  if ( strlen(Input.alt.netCDF_alt_name) == 0 ) {
    if ( Input.atm.ECMWF_new_format == TRUE && strlen(Input.filename[FN_ECMWF])>0 )
      strcpy (Input.alt.netCDF_alt_name, "z");
    else
      strcpy (Input.alt.netCDF_alt_name, "Z");
  }

  /* set default surface temperature netCDF variable name */
  if ( strlen(Input.netCDF_name_surf_T) == 0 ) {
    if ( Input.atm.ECMWF_new_format == TRUE && strlen(Input.filename[FN_ECMWF])>0 )
      strcpy (Input.netCDF_name_surf_T, "skt");
    else
      strcpy (Input.netCDF_name_surf_T, "SKT");
  }


  /* Default MYSTIC output filename */
  if (strlen(Input.rte.mc.filename[FN_MC_BASENAME]) == 0)
    strcpy (Input.rte.mc.filename[FN_MC_BASENAME], "mc");
  

  /* MYSTIC 3D fields passed back to uvspec ? */
  if (Input.atm.nzout>0 && Input.rte.mc.Nx_sample>0 && Input.rte.mc.Ny_sample>0 
      && Input.rte.solver==MONTECARLO)
    Input.rte.mc.passback3D=1;


  /* CLOUD default values */
  if ((strlen(Input.filename[FN_WCLOUD]) || Input.rte.solver==MONTECARLO) > 0 && Input.wc.properties == WCPROP_NONE)  {
    Input.wc.properties = WCPROP_HU;

    if (!Input.quiet)
      fprintf (stderr, " ... using default water cloud properties, Hu and Stamnes (1993)\n");
  }
  
  if (strlen(Input.filename[FN_ICLOUD]) > 0 && Input.ic.properties == ICPROP_NONE)  {
    Input.ic.properties = ICPROP_FU;

    if (!Input.quiet)
      fprintf (stderr, " ... using default ice cloud properties, Fu et al. (1996/98)\n");
  }

  if ( Input.wc.source == CLOUDS_FROM_ECMWF && Input.wc.layer != 1 ) {
    Input.wc.layer=1;
    if (Input.verbose)
      fprintf (stderr, " ... switch on wc_layer, as ECMWF cloud data is representative for layers \n");
  }

  if ( Input.ic.source == CLOUDS_FROM_ECMWF && Input.ic.layer != 1 ) {
    Input.ic.layer=1;
    if (Input.verbose)
      fprintf (stderr, " ... switch on ic_layer, as ECMWF cloud data is representative for layers \n");
  }


  /* CLOUD warnings */
  switch (Input.wc.properties) {
  case WCPROP_MIE:
  case WCPROP_FILE:
  case WCPROP_EXPLICIT:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** Warning, you specified a tabulated phase function for water clouds.\n");
      fprintf (stderr, "*** It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "*** the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
      fprintf (stderr, "*** DISORT2 consideres all moments correctly.\n");
      fprintf (stderr, "\n\n");
    }
    break;
  default:
    break;
  }

  /* something special: the new Mayer/Key/Yang parameterization is not yet */
  /* included in the distribution; if it is not found, Key et al is used   */
  /* instead.                                                              */
  #if HAVE_YANG
  #else
  if (Input.ic.properties == ICPROP_YANG)
    Input.ic.properties = ICPROP_KEY;
  #endif



  switch (Input.ic.properties) {
  case ICPROP_MIE:
  case ICPROP_BAUM:
  case ICPROP_BAUMDETAILED:
  case ICPROP_FILE:
  case ICPROP_EXPLICIT:
    if (Input.rte.solver==DISORT) {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** Warning, you specified a tabulated phase function for ice clouds.\n");
      fprintf (stderr, "*** It is strongly recommended to use DISORT2 in this case because\n");
      fprintf (stderr, "*** the older DISORT version uses only the first NSTR moments of the\n");
      fprintf (stderr, "*** phase function which might result in really bad distortions.\n");
      fprintf (stderr, "*** DISORT2 consideres all moments correctly");
      fprintf (stderr, "\n\n");
    }
    break;
  case ICPROP_YANG:
#if HAVE_YANG
#else
    fprintf (stderr, "***********************************************************************\n");
    fprintf (stderr, "* Error, you have built uvspec without Yang/Key/Mayer support and     *\n");
    fprintf (stderr, "* hence cannot use 'ic_properties yang'.                              *\n");
    fprintf (stderr, "***********************************************************************\n");
    return -1;
#endif
    break;

  case ICPROP_KEY:
#if HAVE_KEY56
#else
    fprintf (stderr, "***********************************************************************\n");
    fprintf (stderr, "* Error, you have built uvspec without Key et al. [2002] support and  *\n");
    fprintf (stderr, "* hence cannot use 'ic_properties key'.                               *\n");
    fprintf (stderr, "***********************************************************************\n");
    return -1;
#endif
    break;
  case ICPROP_NONE:
  case ICPROP_FU:
  case ICPROP_ECHAM4:
    break;
  default:
    fprintf (stderr, "Error, unknown ice cloud properties %d\n", Input.ic.properties);
    return -1;
  }


  if (strlen(Input.filename[FN_ICLOUD])>0 && Input.ic.properties==ICPROP_FU) {
    if (Input.ic.scale_gg >= -1  || Input.ic.gg  >= -1 ||
	Input.ic.scale_ssa >= 0  || Input.ic.ssa >= 0 ||
	Input.ic.tau > 0         || Input.ic.tau550 > 0) {
      fprintf (stderr, "*** Warning, you define an ice cloud using the parameterization by Fu (1996/98) *\n");
      fprintf (stderr, "*** and explicitely modify one of the optical properties, optical thickness,    *\n"); 
      fprintf (stderr, "*** asymmetry parameter, or single scattering albedo. Please make sure that you *\n");
      fprintf (stderr, "*** understand the discussion of delta-scaling in context with the Fu (1996/98) *\n");
      fprintf (stderr, "*** parameterization in the libRadtran manual; see the description of           *\n");
      fprintf (stderr, "*** ic_properties, ic_fu_tau, and ic_fu_reff!                                   *\n");
    }
  }    


  /* water cloud properties */
  if (Input.wc.properties == WCPROP_MIE) {
    switch (Input.ck_scheme) {
    case CK_KATO:
    case CK_KATO2:
    case CK_KATO2_96:
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k KATO\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;	    
		    
    case CK_FU:	    
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k FU\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;
      
    case CK_AVHRR_KRATZ:
      if (Input.wc.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"wc_properties mie\" together with\n");
        fprintf (stderr, "* wc_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'wc_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_WCLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      break;

    case CK_NONE:
    case CK_LOWTRAN:
      if (!Input.wc.interpolate)  {
        fprintf (stderr, "* 'wc_properties mie' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
	fprintf (stderr, "* option, please specify explicitely 'wc_properties_interpolate'\n");
	fprintf (stderr, "* in the uvspec input!\n");
	return -1;
      }
      else {
        strcpy (Input.filename[FN_WCLOUD_PROP], Input.filename[FN_PATH]);
        strcat (Input.filename[FN_WCLOUD_PROP], "/wc/mie/");
        strcat (Input.filename[FN_WCLOUD_PROP], "wc.");
      }
      break;

    case CK_GENERIC:
      fprintf (stderr, "Error, 'wc_properties mie' not implemented for a generic\n");
      fprintf (stderr, "correlated-k calculation\n");
      return -1;

    default:
      fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }
  }

  if (strlen(Input.filename[FN_WCLOUD]) > 0 &&
      strlen(Input.filename[FN_WCLOUD_FILES]) > 0)  {
    fprintf (stderr, "Error, it does not make sense to define both wc_file and wc_files.\n");
    fprintf (stderr, "Remove either of them and retry!\n");
    return -1;
  }

  /* ice cloud properties */
  if (Input.ic.properties == ICPROP_MIE) {

    switch (Input.ck_scheme) {
    case CK_KATO:
    case CK_KATO2:
    case CK_KATO2_96:
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k KATO\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kato/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;	    
		    
    case CK_FU:	    
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k FU\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/fu/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;
      
    case CK_AVHRR_KRATZ:
      if (Input.ic.interpolate)  {
        fprintf (stderr, "* It does not make sense to use \"ic_properties mie\" together with\n");
        fprintf (stderr, "* ic_properties_interpolate for \"correlated_k AVHRR_KRATZ\" because\n");
	fprintf (stderr, "* in the latter case the cloud properties are pre-calculated\n");
	fprintf (stderr, "* for the required wavelength and interpolation is not useful.\n");
	fprintf (stderr, "* Please remove 'ic_properties_interpolate' from the uvspec input.\n");
	return -1;
      }

      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
      strcat (Input.filename[FN_ICLOUD_PROP], "/correlated_k/kratz/");
      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      break;

    case CK_NONE:
    case CK_LOWTRAN:
      if (!Input.ic.interpolate)  {
        fprintf (stderr, "* 'ic_properties mie' for a spectral calculation requires that the\n");
        fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
	fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
	fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
	fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate'\n");
	fprintf (stderr, "* in the uvspec input!\n");
	return -1;
      }
      else {
        strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);
        strcat (Input.filename[FN_ICLOUD_PROP], "/ic/mie/");
        strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
      }
      break;

    case CK_GENERIC:
      fprintf (stderr, "Error, 'ic_properties mie' not implemented for a generic\n");
      fprintf (stderr, "correlated-k calculation\n");
      return -1;

    default:
      fprintf (stderr, "Error, unsupported correlated_k scheme %d\n", Input.ck_scheme);
      return -1;
    }

    fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
	     Input.filename[FN_ICLOUD_PROP]);
  }
  

  /* <<(CE:) */
  /* Baum ice cloud properties */ 
  if (Input.ic.properties == ICPROP_BAUM || Input.ic.properties == ICPROP_BAUMDETAILED) {

    if (!Input.ic.interpolate)  {
      fprintf (stderr, "* 'ic_properties baum' for a spectral calculation requires that the\n");
      fprintf (stderr, "* cloud optical properties are interpolated to the internal wavelength\n");
      fprintf (stderr, "* grid. Depending on the number of grid points this may require huge\n");
      fprintf (stderr, "* amounts of memory and some additional computational time. To use this\n");
      fprintf (stderr, "* option, please specify explicitely 'ic_properties_interpolate'\n");
      fprintf (stderr, "* in the uvspec input!\n");
      
      return -1;
    }
    else {
      strcpy (Input.filename[FN_ICLOUD_PROP], Input.filename[FN_PATH]);

      if (Input.ic.properties == ICPROP_BAUMDETAILED)
        strcat (Input.filename[FN_ICLOUD_PROP], "ic/baum/");

      if (Input.ic.properties == ICPROP_BAUM)
        strcat (Input.filename[FN_ICLOUD_PROP], "ic/baum_hufit/");

      strcat (Input.filename[FN_ICLOUD_PROP], "ic.");
    }

    fprintf (stderr, " ... reading ice cloud optical properties from %s*\n", 
	     Input.filename[FN_ICLOUD_PROP]);

  }


  /* check IPA options */
  if (Input.wc_cloudcover>0 && strlen(Input.filename[FN_ICLOUD_IPA])>0)  {
    fprintf (stderr, "Error, wc_cloudcover does not work together with ic_ipa_files.\n");
    fprintf (stderr, "If you need both water and ice clouds together in independent\n");
    fprintf (stderr, "pixel approximation, please use wc_ipa_files and ic_ipa_files!\n");
    return -1;
  }

  if (Input.ic_cloudcover>0 && strlen(Input.filename[FN_WCLOUD_IPA])>0)  {
    fprintf (stderr, "Error, ic_cloudcover does not work together with wc_ipa_files.\n");
    fprintf (stderr, "If you need both water and ice clouds together in independent\n");
    fprintf (stderr, "pixel approximation, please use wc_ipa_files and ic_ipa_files!\n");
    return -1;
  }

  if (Input.wc_cloudcover>0 && strlen(Input.filename[FN_WCLOUD_IPA])>0)  {
    fprintf (stderr, "Error, it does not make sense to specify both wc_cloudcover and wc_ipa_files!\n");
    return -1;
  }

  if (Input.ic_cloudcover>0 && strlen(Input.filename[FN_ICLOUD_IPA])>0)  {
    fprintf (stderr, "Error, it does not make sense to specify both ic_cloudcover and ic_ipa_files!\n");
    return -1;
  }


  /* aerosol properties */
  if (Input.aer.seasn>0 && Input.aer.vulcan>0 && Input.aer.haze>0 && Input.aer.visibility>0)
    Input.aer.standard = 1;


  /* BRDR stuff */
  brdftype=0;

  /******************************************/
  /* First, RPV (Rahman, Pinty, Verstraete) */
  /******************************************/

  /* homogeneous RPV */
  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0)
    brdftype++;

  /* homogeneous, wavelength-dependent RPV */
  if (strlen(Input.filename[FN_RPV])>0)
    brdftype++;

  /* 2D RPV */
  if (strlen(Input.rte.mc.filename[FN_MC_RPV])>0)  {
    brdftype++;

    Input.brdf = BRDF_RPV2D;

    if (!Input.quiet)
      fprintf (stderr, " ... using 2D RPV parameterization of the BRDF\n");
  }

  if (Input.rpv.k!=0 || Input.rpv.rho0!=0 || Input.rpv.theta!=0 || 
      strlen(Input.filename[FN_RPV])>0) {
    Input.brdf = BRDF_RPV;

    if (!Input.quiet)
      fprintf (stderr, " ... using RPV parameterization of the BRDF\n");
  }

  
  /******************************************/
  /* Second, Cox and Munk                   */
  /******************************************/

  if (Input.cm.u10 >= 0 || Input.cm.pcl >= 0 || Input.cm.sal >= 0 || Input.brdf == BRDF_CM) {
    brdftype++;

    Input.brdf = BRDF_CM;

    if (!Input.quiet)
      fprintf (stderr, " ... using Cox and Munk parameterization of the BRDF\n");
  }

  /* set default values for windspeed, pigment concentration, and salinity */
  if (Input.brdf == BRDF_CM)  {

    if (Input.cm.u10<0)  {
      if (strlen(Input.filename[FN_U10_MAP])==0) {
        Input.cm.u10 = 0;
      }
      else {
        /* read wind speed from netCDF wind map */
        status = read_u10_from_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate, 
                                    Input.filename[FN_U10_MAP],
                                    &Input.cm.u10, Input.verbose, Input.quiet);
        if (status!=0) {
          fprintf (stderr, "Error %d during read_u10_from_map in uvspec_lex.l \n", status);
          return status;
        }
      }
      if (Input.verbose)
        fprintf (stderr, " ... setting wind speed for Cox and Munk to %f m/s\n", Input.cm.u10);
    }

    if (Input.cm.pcl<0)  {

      if (strlen(Input.filename[FN_SALINITY_MAP])==0) {
        Input.cm.pcl = 0.01;
      }
      else {

        if ( strlen(Input.cm.pcl_netCDF_name) == 0 )
          strcpy (Input.cm.pcl_netCDF_name, "chlorophyll");

        /* read ocean water pigment concentration from map */
        status = get_number_from_netCDF_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate,
                                             Input.filename[FN_PIGMENTS_MAP], &(Input.cm.pcl), TYPE_FLOAT, Input.cm.pcl_netCDF_name,
                                             Input.verbose, Input.quiet);
        if (status!=0) {
          if ( status == ERROR_READ_MISSING_VALUE )
            fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                "read missing value", "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name ); 
          else { 
#if HAVE_LIBNETCDF
            fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                nc_strerror(status), "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name );
#else
            fprintf (stderr, "     Error %d reading '%s' from %s in %s (%s) \n", 
                                            status , "chlorophyll", Input.filename[FN_PIGMENTS_MAP], function_name, file_name );
#endif
          }
          return status;
        }
      }

      /* take care of user scale value */
      Input.cm.pcl *= Input.cm.pcl_scale_factor;

      /* automatic check for plausible salinity range */
      if ( (Input.cm.pcl < 0.0 || 100.0 < Input.cm.pcl) && !Input.quiet ) {
        fprintf (stderr, " *** Warning: ocean pigments value is not plausible: %f \n", Input.cm.pcl  );
        fprintf (stderr, "              value must be given in mg/m3\n" );
      }
      if ( Input.cm.pcl < 0.0 ) return -1;

      if (Input.verbose)
        fprintf (stderr, " ... setting pigment concentration for Cox and Munk to %f mg/m3\n", Input.cm.pcl);
    }

    if (Input.cm.sal<0)  {

      if (strlen(Input.filename[FN_SALINITY_MAP])==0) {
        Input.cm.sal = 34.3;
      }
      else {

        if ( strlen(Input.cm.sal_netCDF_name) == 0 )
          strcpy (Input.cm.sal_netCDF_name, "salinity");

        /* read salinity from map */
        status = get_number_from_netCDF_map (Input.latitude, Input.longitude, Input.UTC, Input.atm.time_interpolate,
                                             Input.filename[FN_SALINITY_MAP], &(Input.cm.sal), TYPE_FLOAT, Input.cm.sal_netCDF_name,
                                             Input.verbose, Input.quiet);
        if ( status == ERROR_READ_MISSING_VALUE ) {
          Input.cm.sal = 34.3;
          if ( !Input.quiet ) {
            fprintf (stderr, " *** Warning: read missing value from map. \n" );
            fprintf (stderr, "              set salinity to default value of %f \n", Input.cm.sal );
          }
          status=0;
        }

        if (status!=0) {
#if HAVE_LIBNETCDF
          fprintf (stderr, "     Error '%s' reading '%s' from %s in %s (%s) \n", 
                                nc_strerror(status), "SALT", Input.filename[FN_SALINITY_MAP], function_name, file_name );
#else
          fprintf (stderr, "     Error %d reading '%s' from %s in %s (%s) \n", 
                                            status , "SALT", Input.filename[FN_SALINITY_MAP], function_name, file_name );
#endif
          return status;
        }
      }

      /* take care of user scale value */
      Input.cm.sal *= Input.cm.sal_scale_factor;

      /* automatic check for plausible salinity range */
      if ( (Input.cm.sal < 0.0 || 55.0 < Input.cm.sal) && !Input.quiet ) {
        fprintf (stderr, " *** Warning: salinity value is not plausible: %f \n", Input.cm.sal  );
        fprintf (stderr, "              value must be given in ppt (=PSU) \n" );
      }
      if ( Input.cm.sal < 0.0 ) return -1;

      if (Input.verbose)
        fprintf (stderr, " ... setting salinity for Cox and Munk to %f ppm\n", Input.cm.sal);
    }
    
    /* we don't want wind speeds smaller than 1 m/s because the   */
    /* BRDF is expressed as a series of nstr Legendre polynomials */
    /* which causes unwanted structures not only in the BRDF but  */
    /* also in the albedo                                         */
    if (Input.cm.u10<1)  {
    Input.cm.u10 = 1;
      fprintf (stderr, " ... increasing wind speed for Cox and Munk to %.2f m/s\n", Input.cm.u10);
    }
    
    /* we also don't want nstr < 16 for the same reason, even for */
    /* the calculation of fluxes                                  */

    if (Input.rte.nstr<16)  {
      fprintf (stderr, " ... increasing nstr to 16 because of Cox and Munk!\n");
      Input.rte.nstr=16;
    }
  }


  /* Check if only one or more types of BRDF have been defined */
  if (brdftype > 1)  {
    fprintf (stderr, "Error, it does not make sense to define more than one BRDF type!\n");
    return -1;
  }
  
  
  /* Number of phase function moments equals the number of streams  */
  /* per default, but may be changed later in the code;             */
  /* in case of disort2 we increase nmom to at least 1024 to assure */
  /* correct representation of the phase function with only little  */
  /* computational cost                                             */
  Output.atm.nmom = Input.rte.nstr;      
  if (Input.rte.solver == DISORT2 || Input.rte.solver == SSS || Input.rte.solver == SSSI)
    Output.atm.nmom = (Input.rte.nstr>1024 ? Input.rte.nstr : 1024);

  if (Input.rte.solver == SDISORT)
    if (Input.rte.nscat != 1 && Input.rte.nscat != 2)
       Input.rte.nscat = 2;

  Output.atm.molabs   = Input.molabs;
  Output.atm.rayleigh = Input.rayleigh;

  if (Input.rte.nphi <= 0) {
    switch (Input.rte.solver) {
    case SDISORT:
    case SPSDISORT:
    case DISORT:
    case DISORT2:
    case QDISORT:
      Input.rte.maxphi = 3;  /* Minimum number required by disort. */
      Input.rte.nphi   = 1;
      Input.rte.phi    = (float *) calloc (Input.rte.maxphi, sizeof(float));
      Output.print_phi = 0;  /* Do not print radiances if user does not want them */
      break;
    case POLRADTRAN:
      if (Input.rte.pol_aziorder == 0) 
	Input.rte.nphi = 0;
      break; 
    }
  }

  if (Input.rte.numu <= 0) {
    Input.rte.maxumu = Input.rte.nstr;
    Input.rte.umu    = (float *) calloc (Input.rte.maxumu, sizeof(float));
  }
  

  /* Output processing: integrated values for brightness temperature */
  if (Input.calibration == OUTCAL_BRIGHTNESS) {
    if(Input.processing != PROCESS_SUM && Input.processing != PROCESS_INT) {
      fprintf (stderr, "Error, brightness temperatures only implemented for integrated quantities!\n");
      fprintf (stderr, "Please select 'output sum' or 'output integrate'!\n");
      return -1;
    }
  }

  /* Output processing: thermal source for brightness temperature */
  if (Input.calibration == OUTCAL_BRIGHTNESS && Input.source != SRC_THERMAL) {
    fprintf (stderr, "Error, brightness temperatures does only make sense for thermal sources!\n");
    return -1;
  }

  /* Output processing: no transmittance for thermal calculations */
  if (Input.calibration == OUTCAL_TRANSMITTANCE && Input.source == SRC_THERMAL) {
    fprintf (stderr, "Error, transmittance does not make sense for thermal sources!\n");
    return -1;
  }

  /* if thermal source, and no brightness temperature requested, use thermal calibration */
  if (Input.source == SRC_THERMAL && Input.calibration == OUTCAL_ABSOLUTE) {
    Input.calibration = OUTCAL_THERMAL;
  }


  if (Input.wl.start > Input.wl.end) {
    if (!Input.quiet) 
      fprintf(stderr," ... swap first and second wavelength.\n");
    tmpfloat = Input.wl.start;
    Input.wl.start = Input.wl.end;
    Input.wl.end = tmpfloat;
  }

  if (Input.wl.start <= 0 && Input.wl.start != -999) {
    fprintf (stderr, "Error, start wavelength %f is negative.\n", Input.wl.start);
    return -1;
  }

  Output.wl.start = Input.wl.start;
  Output.wl.end   = Input.wl.end;

  if (Input.wl.start_index > Input.wl.end_index) {
    if (!Input.quiet) 
      fprintf(stderr," ... swap first and second wavelength index.\n");
    tmpint = Input.wl.start_index;
    Input.wl.start_index = Input.wl.end_index;
    Input.wl.end_index = tmpint;
  }

  if (Input.wl.start_index <= 0 && Input.wl.start_index != -999) {
    fprintf (stderr, "Error, start wavelength_index %d is negative.\n", Input.wl.start_index);
    return -1;
  }

  /* check the wavelength range */
  if (Input.wl.end>850 && (Input.ck_scheme==CK_NONE && strlen(Input.filename[FN_MOLABS])==0))  {
    status=-1;
    fprintf (stderr, "Error %d, you want to do a spectral calculation for wavelengths larger than 850 nm. While uvspec\n", status);
    fprintf (stderr, "    treats ozone absorption correctly, molecular absorption is NOT considered in monochromatic\n");
    fprintf (stderr, "    uvspec calculations, as absorption cross-section are highly variable with wavelength.\n");
    fprintf (stderr, "    To consider molecular absorption other than ozone you have two choices \n");
    fprintf (stderr, "    with uvspec:\n");
    fprintf (stderr, "     (1) Do a line-by-line calculation using 'molecular_tau_file' to specify\n");
    fprintf (stderr, "         the wavelength-dependent absorption profile; to calculate the\n");
    fprintf (stderr, "         latter, you need something like David Edwards' genln2.\n");
    fprintf (stderr, "         ATTENTION: line-by-line calculations are very time-consuming!\n");
    fprintf (stderr, "     (2) Use the correlated-k approximation which is the most accurate\n");
    fprintf (stderr, "         solution after the line-by-line calculation; use either the\n");
    fprintf (stderr, "         pre-defined parameterization that come with libRadtran or provide\n");
    fprintf (stderr, "         your own; both options are selected with 'correlated_k ...'\n");
    fprintf (stderr, "\n");
    return -1;
  }

  if (Input.wl.end>500 && (Input.ck_scheme==CK_NONE && strlen(Input.filename[FN_MOLABS])==0))  {
    if (!Input.quiet)  {
      fprintf (stderr, "\n");
      fprintf (stderr, "*** WARNING, you want to do a spectral calculation for wavelengths larger\n");
      fprintf (stderr, "*** than 500 nm. Be warned that from about 500 nm upwards there are highly\n");
      fprintf (stderr, "*** structured molecular absorption bands which are NOT considered in the\n");
      fprintf (stderr, "*** spectral uvspec calculation. While uvspec treats ozone absorption\n");
      fprintf (stderr, "*** correctly, the water vapour absorption bands around 510nm, 545nm,\n");
      fprintf (stderr, "*** 595nm, 655nm, 725nm, 825nm, 935nm, and higher, as well as the\n");
      fprintf (stderr, "*** oxygen absorption at 630nm, 690nm, and 765nm are not implemented.\n");
      fprintf (stderr, "*** Below around 600nm these may usually be neglected; above 600nm it\n");
      fprintf (stderr, "*** starts getting more serious, and above 800nm it does not make any\n");
      fprintf (stderr, "*** sense to do the RTE calculation without considering molecular\n");
      fprintf (stderr, "*** absorption.\n");
      fprintf (stderr, "*** \n");
      fprintf (stderr, "*** To consider molecular absorption other than ozone you have two choices\n");
      fprintf (stderr, "*** with uvspec:\n");
      fprintf (stderr, "***  (1) Do a line-by-line calculation using 'molecular_tau_file' to specify\n");
      fprintf (stderr, "***      the wavelength-dependent absorption profile; to calculate the\n");
      fprintf (stderr, "***      latter, you need something like David Edwards' genln2.\n");
      fprintf (stderr, "***      ATTENTION: line-by-line calculations are very time-consuming!\n");
      fprintf (stderr, "***  (2) Use the correlated-k approximation which is the most accurate\n");
      fprintf (stderr, "***      solution after the line-by-line calculation; use either the\n");
      fprintf (stderr, "***      pre-defined parameterization that come with libRadtran or provide\n");
      fprintf (stderr, "***      your own; both options are selected with 'correlated_k ...'\n");
      fprintf (stderr, "\n");
    }
  }


  /**************************************/
  /* initialize random number generator */
  /**************************************/
  /* initialize random number generator */

  if (Input.rte.mc.randomseed>0)
    rseed = Input.rte.mc.randomseed;
  else
    rseed = (int) time(NULL) + (int) getpid();
    
  #if HAVE_LIBGSL 
    T = gsl_rng_mt19937;
    gsl_rng_default_seed = rseed;
    uvspecrng = gsl_rng_alloc (T);
    if (!Input.quiet && Input.rte.solver == MONTECARLO)
      fprintf (stderr, " ... using %s random number generator\n", gsl_rng_name (uvspecrng));
  #else
    r250_init (rseed);   /* sophisticated random number generator */
    if(!Input.quiet && Input.rte.solver == MONTECARLO) {
      fprintf (stderr, " ... using r250() random number generator\n");
      fprintf (stderr, " ... CAREFUL! r250() is good but has shown problems in MYSTIC.\n");
      fprintf (stderr, " ... Install GSL to be on the safe side!\n");
    }
  #endif

  /* store random seed for debugging purposes */
  if ((randomseedfile = fopen(randomseedfilename, "w")) != NULL) {
    fprintf (randomseedfile, "%d\n", rseed);
    (void) fclose (randomseedfile);
  }
    

  /********************************/
  /* Do the radiation calculation */
  /********************************/
  

  status = uvspec(Input,&Output);
  if (status!=0) {
    fprintf (stderr, "Error %d during execution of uvspec\n", status);
    return status;
  }

  
#ifndef UVSPEC_FUNCTION
  print_output(Input, &Output); 
#else
  print_output(Input, &Output, outfilename); 
#endif
  if (status!=0) {
    fprintf (stderr, "Error %d during print_output \n", status);
    return status;
  }

  free_output(Input, &Output);
  return status;

}

  /*************************/
  /* Output of the results */
  /*************************/

#ifndef UVSPEC_FUNCTION
static int print_output(input_struct Input, output_struct *output) 
#else 
static int print_output(input_struct Input, output_struct *output, char *outfilename) 
#endif
{
  int status=0;

#ifndef UVSPEC_FUNCTION
  FILE *f = stdout;
#else
  FILE *f = fopen (outfilename, "w");
#endif

  switch(Input.output_format) {

  case OUTPUT_FORMAT_FLEXSTOR:
    write_flexstor_output(Input, output, f);
    break;
  case OUTPUT_FORMAT_ASCII:
    
    if (Input.header)
      write_header(Input, output, f);

    if (Input.rte.solver != POLRADTRAN)
      /* write user defined output to file with pointer f */
      write_user_defined_output(Input, output, f);
    else { /* -> Input.rte.solver == POLRADTRAN) */
      /* write output in a special Polradtran format */
      write_polradtran_output(Input, output, f);
    }
    break;
  case OUTPUT_FORMAT_MATRIX:

    /* special output for spectral heating rates as 2D matrix */
    write_spectral_heat (Input, output, f);

    break; 
  case OUTPUT_FORMAT_NETCDF:
  case OUTPUT_FORMAT_SATPIC:

#if HAVE_LIBNETCDF

    status = write_netCDF_output(Input, output);
    if (status != NC_NOERR) {
      fprintf (stderr, "Error %d during write_netCDF_output() to %s (uvspec_lex.l) \n", status, Input.filename[FN_NETCDF_OUTFILE]);
      return -1;
    }
#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the netCDF output option. Please get netcdf and rebuild.        *\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

    break;
  default:
    fprintf(stderr, "Error, unknown output format %d in print_output()\n", Input.output_format);
    return -1; 
  }


  fflush(f);

  if (f!=stdout)
    fclose(f);

  return 0;
}  


void write_flexstor_output(input_struct Input, output_struct *output, FILE *f)
{

  int lev=0;
  int iv=0;

  fprintf (f, "flexstor 1 0 suvdama 1 1\n");
  fprintf (f, "# This flexstor output has not been checked by flxcheck.\n");
  fprintf (f, "# Hence it may not follow flexstor/suvdama rules. This will\n");
  fprintf (f, "# be improved soon.\n");
  fprintf (f, "model_name() uvspec, version %s\n", VERSION);
  if (fabs(Input.aer.beta-0.0) > 0.0) {
    fprintf (f, "angstrom_alpha()  %f\n", Input.aer.alpha);
    fprintf (f, "angstrom_beta()  %f\n",  Input.aer.beta);
  }
  fprintf (f, "ozone_column(DU) %f\n", output->column[MOL_O3]);
  fprintf (f, "pressure(mb) %f\n", Input.pressure);
  fprintf (f, "self_day_of_year() %d\n", Input.UTC.tm_yday);
  
  /*
  fprintf (f, "self_zout(km)");
  for (lev=0;lev<output->atm.nzoutatm.nzout;lev++)
    fprintf (f, "  %f", Input.atm.zout[lev]);
  fprintf (f, "\n");
  */

  fprintf (f, "self_temper(K)");
  for (lev=0;lev<output->atm.nzout;lev++)
    fprintf (f, "  %f", output->atm.temper_out[lev]);
  
  fprintf (f, "\n");
  fprintf (f, "self_day_of_year() %d\n", Input.UTC.tm_yday);
  fprintf (f, "self_eccentricity_factor() %f\n", output->eccentricity_factor);
  fprintf (f, "begin table data\n");
  for (lev=0;lev<output->atm.nzout;lev++) {
    fprintf (f, "WL(nm) SZA(deg.dec) self_rfldir(mW m-2 nm-2) self_rfldn(mW m-2 nm-2) self_rflup(mW m-2 nm-2)");
    fprintf (f, " self_uavgso(mW m-2 nm-2) self_uavgdn(mW m-2 nm-2) self_uavgup(mW m-2 nm-2) ");
  }
  fprintf (f, "\n");
  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {
      if (Input.rte.solver == TWOSTR) {
        fprintf (f, "%7.3f %13.6e %13.6e %13.6e %13.6e ", 
        	 output->wl.lambda_h[iv], 
		 output->rfldir[lev][iv], 
	         output->rfldn[lev][iv], 
	         output->flup[lev][iv],
	         output->uavg[lev][iv]);
      }
      else {
        fprintf (f, "%7.3f %7.3f %8.5f %13.6e %13.6e %13.6e %13.6e %13.6e %13.6e ",
      	           output->wl.lambda_h[iv], 
		   output->sza_h[iv], 
		   output->atm.zout_sur[lev],
		   output->rfldir[lev][iv], 
		   output->rfldn[lev][iv], 
		   output->flup[lev][iv],
		   output->uavgso[lev][iv], 
		   output->uavgdn[lev][iv], 
		   output->uavgup[lev][iv]);
      }
      fprintf (f, "\n");
    }
  }
  fprintf (f, "end table data\n");
}


void write_header (input_struct Input, output_struct *output, FILE *f)
{
  int lev=0;

  fprintf (f, "#aerosol_haze                 : %16d\n",   Input.aer.haze);
  fprintf (f, "#aerosol_season               : %16d\n",   Input.aer.seasn);
  fprintf (f, "#aerosol_set_gg               : %16.3f\n", Input.aer.set_gg);
  fprintf (f, "#aerosol_set_ssa              : %16.3f\n", Input.aer.set_ssa);
  fprintf (f, "#aerosol_tau_file             : %s\n",     Input.aer.filename[FN_AER_TAU]);
  fprintf (f, "#aerosol_ssa_file             : %s\n",     Input.aer.filename[FN_AER_SSA]);
  fprintf (f, "#aerosol_gg_file              : %s\n",     Input.aer.filename[FN_AER_GG]);
  fprintf (f, "#aerosol_moments_file         : %s\n",     Input.aer.filename[FN_AER_MOM]);
  fprintf (f, "#aerosol_visibility           : %16.3f\n", Input.aer.visibility);
  fprintf (f, "#albedo                       : %16.3f\n", Input.alb.albedo);
  fprintf (f, "#angstrom                     : %16.3f  %16.3f\n", Input.aer.alpha, Input.aer.beta);
  fprintf (f, "#atmosphere_file              : %s\n",     Input.filename[FN_ATMOSPHERE]);
  fprintf (f, "#day_of_year                  : %16d\n",   Input.UTC.tm_yday);

  switch (Input.rte.deltam) {
  case 0:
    printf("#deltam                       : off\n");
    break;
  case 1:
    printf("#deltam                       : on\n");
    break;
  default:
    printf("#deltam                       : unknown\n");
    break;
  }

  switch (Input.ozone_spectrum) {
  case O3_BASSPAUR:
    printf("#o3_crs                       : Bass_and_Paur\n");
    break;
  case O3_MOLINA:
    printf("#o3_crs                       : Molina\n");
    break;
  case O3_DAUMONT:
    printf("#o3_crs                       : Daumont\n");
    break;
  default:
    printf("#o3_crs                       : unknown\n");
    break;
  }

  fprintf (f, "#ozone_column                 : %16.3f\n", Output.column[MOL_O3]);

  switch (Input.rte.solver) {
  case SDISORT:
    printf("#rte_solver                   : sdisort\n");
    break;
  case SPSDISORT:
    printf("#rte_solver                   : spsdisort\n");
    break;
  case DISORT:
    printf("#rte_solver                   : disort\n");
    break;
  case DISORT2:
    printf("#rte_solver                   : disort2\n");
    break;
  case MONTECARLO:
    printf("#rte_solver                   : mystic\n");
    break;
  case POLRADTRAN:
    printf("#rte_solver                   : polradtran\n");
    break;
  case SUCORD:
    printf("#rte_solver                   : sucorcsca\n");
    break;
  case TZS: 
    printf("#rte_solver                   : tzs\n");
    break;
  case SSS: 
    printf("#rte_solver                   : sss\n");
    break;
  case SSSI: 
    printf("#rte_solver                   : sss\n");
    break;
  case TWOSTR: 
    printf("#rte_solver                   : twostr\n");
    break;
  default:
    printf("#rte_solver                   : unknown\n");
    break;
  }

  fprintf (f, "#slit_function_file           : %s\n", 
           Input.filename[FN_SLITFUNCTION]);
  fprintf (f, "#solar_file                   : %s\n", 
           Input.filename[FN_EXTRATERRESTRIAL]);
  fprintf (f, "#wavelength                          : %16.3f  %16.3f\n", 
           Input.wl.start, Input.wl.end);
  fprintf (f, "#zout                         : ");
  for (lev=0;lev<output->atm.nzout;lev++)
    fprintf (f, " %16.3f", Input.atm.zout_sur[lev]);
  fprintf (f, "\n");
}


void write_polradtran_output (input_struct Input, output_struct *output, FILE *f)
{

  int iv=0;
  int lev=0;
  int is=0;  /* counter for stokes element (1...4) */
  int iu=0;  /* counter for cos(SZA) */

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {

      fprintf (f, "%7.3f ", output->wl.lambda_h[iv]);

      for (is=0;is<Input.rte.pol_nstokes;is++)
        fprintf (f, "%13.6e %13.6e ",
                 output->down_flux[lev][is][iv], 
	         output->up_flux[lev][is][iv]);

      fprintf (f, "\n");

      if (output->print_phi && Input.rte.nphi > 0) {
        fprintf (f, "                      ");
        for (j=0; j<Input.rte.nphi; j++)
          fprintf (f, " %13.3f ", Input.rte.phi[j]);
        fprintf (f, "\n");
        for (is=0; is<Input.rte.pol_nstokes; is++) {
          fprintf (f, "Stokes vector %c\n", Input.rte.pol_stokes[is]);
          for (iu=0; iu<Input.rte.numu; iu++) {
            /* printf("%7.4f ", output->mu_values[iu+Input.rte.nstr/2]);*/
	    printf("%7.4f ", Input.rte.umu[iu]);
	    printf("%13.6e ", 0.0);
	    for (j=0; j<Input.rte.nphi; j++) {
	      if (Input.rte.umu[iu] <= 0)
	        fprintf (f, " %13.6e ", output->down_rad[lev][j][iu][is][iv]);
              else
	        fprintf(f, " %13.6e ", output->up_rad[lev][j][iu][is][iv]);}
	    printf("\n");
	  }
        }
      }
    }
  }
}

void write_spectral_heat (input_struct Input, output_struct *output, FILE *f)
{
  int iv=0;
  int lev=0;
  int lev2=0;

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {

      /* spectral output */
      if (iv == 0 && lev == 0) {
        fprintf (f, "%9.3e ", 0.0);
        for (lev2=0;lev2<output->atm.nzout;lev2++)
          fprintf (f, "%13.6f ", Input.atm.zout_sur[lev2]);
        fprintf (f, "\n");
      }
      if (lev == 0)
        fprintf (f, "%9.3f ", output->wl.lambda_h[iv]);
      fprintf (f, "%13.6e ", output->heat[lev][iv]);
      if (lev == output->atm.nzout-1)
        fprintf (f, "\n");
    }
  }
}


void write_user_defined_output(input_struct Input, output_struct *output, FILE *f)
{
  int io    = NOT_DEFINED_INTEGER;
  int i_gas = NOT_DEFINED_INTEGER;
  int iv    = NOT_DEFINED_INTEGER;
  int lev   = NOT_DEFINED_INTEGER;
  int iu    = NOT_DEFINED_INTEGER;

  for (iv=0;iv<output->wl.nlambda_h;iv++) {
    for (lev=0;lev<output->atm.nzout;lev++) {

      for (io=0; io<Input.n_output_user; io++) {
        switch (Input.output_user[io]) {
        case OUTPUT_USER_WAVE:
          fprintf (f, "%9.3f ", output->wl.lambda_h[iv]);
          break;
        case OUTPUT_USER_WAVE_MAX:
          fprintf (f, "%9.3f ", 1.0e7/output->wl.wvnmlo_h[iv]); 
          break;
        case OUTPUT_USER_WAVE_MIN:
          fprintf (f, "%9.3f ", 1.0e7/output->wl.wvnmhi_h[iv]); 
          break;
        case OUTPUT_USER_WAVENUMBER:
          fprintf (f, "%9.3f ", 1E7/output->wl.lambda_h[iv]);
          break;
        case OUTPUT_USER_WAVENUMBER_MAX:
          fprintf (f, "%9.3f ", output->wl.wvnmhi_h[iv]);        
          break;
        case OUTPUT_USER_WAVENUMBER_MIN:
          fprintf (f, "%9.3f ", output->wl.wvnmlo_h[iv]);        
          break;
        case OUTPUT_USER_ZOUT_SUR:
          fprintf (f, "%11.6f ", output->atm.zout_sur[lev]);
          break;
        case OUTPUT_USER_ZOUT_SEA:
          fprintf (f, "%11.6f ", output->atm.zout_sea[lev]);
          break;
        case OUTPUT_USER_EDIR:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]);
          break;
        case OUTPUT_USER_EGLO:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]);
          break;
        case OUTPUT_USER_EDN:
          fprintf (f, "%13.6e ", output->rfldn[lev][iv]);
          break;
        case OUTPUT_USER_EUP:
          fprintf (f, "%13.6e ", output->flup[lev][iv]);
          break;
        case OUTPUT_USER_ENET:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]-output->flup[lev][iv]);
          break;
        case OUTPUT_USER_ESUM:
          fprintf (f, "%13.6e ", output->rfldir[lev][iv]+output->rfldn[lev][iv]+output->flup[lev][iv]);
          break;
        case OUTPUT_USER_FDIR:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgso[lev][iv]);
          break;
        case OUTPUT_USER_FGLO:
          fprintf (f, "%13.6e ", 4.0*PI*(output->uavgso[lev][iv]+output->uavgdn[lev][iv]));
          break;
        case OUTPUT_USER_FDN:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_FUP:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavgup[lev][iv]);
          break;
        case OUTPUT_USER_F:
          fprintf (f, "%13.6e ", 4.0*PI*output->uavg[lev][iv]);
          break;
        case OUTPUT_USER_UDIR:
          fprintf (f, "%13.6e ", output->uavgso[lev][iv]);
          break;
        case OUTPUT_USER_UGLO:
          fprintf (f, "%13.6e ", output->uavgso[lev][iv]+output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_UDN:
          fprintf (f, "%13.6e ", output->uavgdn[lev][iv]);
          break;
        case OUTPUT_USER_UUP:
          fprintf (f, "%13.6e ", output->uavgup[lev][iv]);
          break;
        case OUTPUT_USER_U:
          fprintf (f, "%13.6e ", output->uavg[lev][iv]);
          break;
        case OUTPUT_USER_ALB:
          fprintf (f, "%13.6e ", output->flup[lev][iv]/(output->rfldir[lev][iv]+output->rfldn[lev][iv]));
          break;
        case OUTPUT_USER_SZA:
          fprintf (f, "%9.3f ", output->sza_h[iv]);
          break;
        case OUTPUT_USER_C_P:
          fprintf (f, "%13.6e ", output->atm.microphys.c_p[lev]);
          break;
        case OUTPUT_USER_HEAT:
          fprintf (f, "%13.6e ", output->heat[lev][iv]);
          break;
        case OUTPUT_USER_W_RAD:
          fprintf (f, "%13.6e ", output->w_zout[lev][iv]/3600.0); /* m/day -> m/s */
          break;
        case OUTPUT_USER_M_RAD:
          fprintf (f, "%13.6e ", output->w_zout[lev][iv] * output->atm.microphys.dens_zout[MOL_AIR][lev] 
                                 * 1.e+6 * 1.e-3 * Input.atm.mol_mass[MOL_AIR] / avogadro );   /* kg/ (m2 day) */
          break;
        case OUTPUT_USER_P:
          fprintf (f, "%11.6f ", output->atm.microphys.press_zout[lev]);
          break;
        case OUTPUT_USER_T:
          fprintf (f, "%11.6f ", output->atm.microphys.temper_zout[lev]);
          break;
        case OUTPUT_USER_T_D:
          fprintf (f, "%11.6f ", output->atm.microphys.temper_d_zout[lev]);
          break;
        case OUTPUT_USER_THETA:
          fprintf (f, "%11.6f ", output->atm.microphys.theta_zout[lev]);
          break;
        case OUTPUT_USER_THETA_E:
          fprintf (f, "%11.6f ", output->atm.microphys.theta_e_zout[lev]);
          break;
        case OUTPUT_USER_CLWC:
          fprintf (f, "%11.6f ", output->wc.microphys.lwc_zout[lev]);
          break;
        case OUTPUT_USER_CIWC:
          fprintf (f, "%11.6f ", output->ic.microphys.lwc_zout[lev]);
          break;
        case OUTPUT_USER_N:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev]);
          break;
        case OUTPUT_USER_RHO:   /* 1.e+6: convert from cm-3 to m-3; 1.e-3: convert g -> kg */
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev] 
                                 * 1.e+6 * 1.e-3 * Input.atm.mol_mass[i_gas] / avogadro);                   
          break;
        case OUTPUT_USER_VMR:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[i_gas][lev]/
                                 output->atm.microphys.dens_zout[MOL_AIR][lev]);
          break;
        case OUTPUT_USER_MMR:
          i_gas = Input.output_user_gas[io];
          fprintf (f, "%13.6e ", (output->atm.microphys.dens_zout[i_gas][lev]*
                                  Input.atm.mol_mass[i_gas]) /
                                  (output->atm.microphys.dens_zout[MOL_AIR][lev]*
                                  Input.atm.mol_mass[MOL_AIR]));
          break;
        case OUTPUT_USER_RH:
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[MOL_H2O][lev]/
                                 vapor_pressure(output->atm.microphys.temper_zout[lev]) * 100.0 );
          break;
        case OUTPUT_USER_RH_ICE:
          fprintf (f, "%13.6e ", output->atm.microphys.dens_zout[MOL_H2O][lev]/
                                 vapor_pressure_over_ice(output->atm.microphys.temper_zout[lev]) * 100.0 );
          break;
        case OUTPUT_USER_WIND_U:
          fprintf (f, "%13.6e ", output->wind.u_zout[lev] );  /* in m/s */
          break;
        case OUTPUT_USER_WIND_V:
          fprintf (f, "%13.6e ", output->wind.v_zout[lev] );  /* in m/s */
          break;
        case OUTPUT_USER_WIND_W:
          fprintf (f, "%13.6e ", output->wind.w_zout[lev] );  /* in m/s */
          break;
        case OUTPUT_USER_DTDX:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dx[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_DTDY:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dy[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_DTDZ:
          fprintf (f, "%13.6e ", output->atm.microphys.dtheta_dz[lev] );  /* in K/m */
          break;
        case OUTPUT_USER_HEAT_AD_X:
          fprintf (f, "%13.6e ", output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD_Y:
          fprintf (f, "%13.6e ", output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD_Z:
          fprintf (f, "%13.6e ", output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev] *24*60*60 );     /* in K/day */
          break;
        case OUTPUT_USER_HEAT_AD:
          fprintf (f, "%13.6e ", (output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] + 
                                  output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] +
                                  output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev]  ) *24*60*60 ); /* in K/day */
          break;
        default:
          fprintf (stderr, "Error, unknown user output %d\n", Input.output_user[io]);
        }
      }
      if (Input.n_output_user>0)
        fprintf (f, "\n");


      /* radiance output */
      if (Input.rte.numu > 0 && !output->print_phi)
        for (iu=0;iu<Input.rte.numu;iu++)
          fprintf (f, "%7.4f %16.6e \n", Input.rte.umu[iu], output->u0u[lev][iu][iv]);

      if (output->print_phi && (Input.rte.solver == DISORT || 
                                Input.rte.solver == DISORT2 ||
                                Input.rte.solver == SDISORT ||
		                Input.rte.solver == SPSDISORT ||
		                Input.rte.solver == SSS ||
		                Input.rte.solver == SSSI ||
		                Input.rte.solver == TZS)
	                && Input.rte.numu> 0) {
        fprintf (f, "                      ");
        for (j=0;j<Input.rte.nphi;j++)
          fprintf (f, " %16.3f ", Input.rte.phi[j]);
        fprintf (f, "\n");
        for (iu=0;iu<Input.rte.numu;iu++) {
          fprintf (f, "%7.4f %16.9e ", Input.rte.umu[iu], output->u0u[lev][iu][iv]);
          for (j=0;j<Input.rte.nphi;j++)
            fprintf (f, " %16.9e ", output->uu[lev][j][iu][iv]);
          fprintf (f, "\n");
        }
      } /* radiance output */

    } /* for (lev=0;lev<output->atm.nzout;lev++) */
  } /* for (iv=0;iv<output->wl.nlambda_h;iv++) */ 

}



int write_netCDF_output (input_struct Input, output_struct *output)
{

#if HAVE_LIBNETCDF

  int status=0;
  int io=0, iv=0, lev=0, i_gas=0;

  int ncid=0;

  int *id_dim  = NULL;
  int id_nx      = 0;
  int id_ny      = 0;
  int id_ntime   = 0;

  char dim_x_name[10];
  char dim_y_name[10];
  char x_name[10];
  char y_name[10];

  int periodic_x = NOT_DEFINED_INTEGER;
  int periodic_y = NOT_DEFINED_INTEGER;

  double x = NOT_DEFINED_FLOAT;
  double y = NOT_DEFINED_FLOAT;

  int id_nzout   = 0;
  int id_nlambda = 0;

  int *id_data  = NULL;
  int id_levels = 0;
  int id_lambda = 0;

  int id_phi  = 0;
  int id_nphi = 0;
  int id_umu  = 0;
  int id_numu = 0;

  size_t nx   = NOT_DEFINED_INTEGER;
  size_t ny   = NOT_DEFINED_INTEGER;
  float dt = 0.0;

  double *x_grid         = NULL;
  double *y_grid         = NULL;

  int itime1=0, itime2=0, nt=0;

  int ix  = NOT_DEFINED_INTEGER;
  int iy  = NOT_DEFINED_INTEGER;
  int itime = NOT_DEFINED_INTEGER;

  size_t index_3D[3] = {0,0,0};
  size_t index_4D[4] = {0,0,0,0};
  size_t index_5D[5] = {0,0,0,0,0};
  size_t index_6D[6] = {0,0,0,0,0,0};
  size_t index_7D[7] = {0,0,0,0,0,0,0};

  size_t n_4D[4] = {0,0,0,0};
  size_t n_5D[5] = {0,0,0,0,0};
  size_t n_6D[6] = {0,0,0,0,0,0};
  size_t n_7D[7] = {0,0,0,0,0,0,0};

  int id_dim_array3[3];
  int id_dim_array4[4]; 
  int id_dim_array5[5];
  int id_dim_array6[6];
  int id_dim_array7[7];
  const int dim3=3, dim4=4, dim5=5;

  int *valid_result=NULL;    /* marker, if the RTE solver can calculate the disired quantity */
  int simulation_done=TRUE; /* marker for valid (not NaN and not Infinity) results          */

  float  *tmp_data_1D = NULL;
  float **tmp_data_3D = NULL;

  char *gas      = NULL;
  char var_name[20];
  char long_name[40];
  char irradiance_units[40];
  char radiance_units[40];

  if (Input.verbose)
    fprintf (stderr, " ... start writing result to netCDF file %s\n", Input.filename[FN_NETCDF_OUTFILE]);

  /* open netcdf file */
  status = nc_open (Input.filename[FN_NETCDF_OUTFILE], NC_WRITE, &ncid); /* read and write access */
  if (status != NC_NOERR) {
    fprintf (stderr, "Error %d opening netCDF file %s\n", status, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }

  /* ----------------------------------------------------------------------- */

  /* get identification number and read data */

  /* get id for dimension "time" (the dimension, not the variable) */
  if      (Input.time_zone==TIME_UTC)
    status = nc_inq_dimid (ncid, "time", &(id_ntime));
  else if (Input.time_zone==TIME_LAT)
    status = nc_inq_dimid (ncid, "local_apparent_time", &(id_ntime)); 
  else {
    fprintf (stderr, "Error, unknown time_zone %d in write_netCDF_output \n", Input.time_zone);
    return status;
  }
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating 'ntime' in %s \n", status, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }


  switch (Input.output_format)  {
  case OUTPUT_FORMAT_NETCDF:
    strcpy (dim_x_name, "lon");
    strcpy (dim_y_name, "lat");
    strcpy (x_name, "lon");
    strcpy (y_name, "lat");
    x = Input.longitude;
    y = Input.latitude;
    periodic_x = FALSE;
    periodic_y = TRUE;
    break;
  case OUTPUT_FORMAT_SATPIC: 
    strcpy (dim_x_name, "pixel_x");
    strcpy (dim_y_name, "pixel_y");
    strcpy (x_name, "pixel_x");
    strcpy (y_name, "pixel_y");
    x = Input.sat_pixel_x;
    y = Input.sat_pixel_y;
    periodic_x = FALSE;
    periodic_y = FALSE;
    break;
  default:
    fprintf (stderr, "Error, unknown output format %d in print_output (uvspec_lex.l)\n", Input.output_format);
    return (-1);
  }

  /* get id for dimension "lat/pixel_x" (the dimension, not the variable) */
  status = nc_inq_dimid (ncid, dim_x_name, &(id_nx));
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating '%s' in %s \n", status, dim_x_name, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }

  /* read latitude array (must already be stored in the file) */
  status = alloc_and_read_netCDF_1D_double(ncid, dim_x_name, &nx, x_name, &(x_grid));
  if (status != 0) {
    fprintf (stderr, "Error %d reading %s from %s\n", status, x_name, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }

  /* get id for dimension "lon/pixel_y" (the dimension, not the variable) */
  status = nc_inq_dimid (ncid, dim_y_name, &(id_ny));
  if (status!=NC_NOERR) {
    fprintf (stderr, "Error %d locating '%s' in %s \n", status, dim_y_name, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }

  /* read longitude array (must already be stored in the file) */
  status = alloc_and_read_netCDF_1D_double( ncid, dim_y_name, &ny, y_name, &(y_grid) );
  if (status != 0) {
    fprintf (stderr, "Error %d reading %s from %s\n", status, y_name, Input.filename[FN_NETCDF_OUTFILE]);
    return status;
  }


  /* ----------------------------------------------------------------------- */

  /* search index */

  if (Input.time_zone==TIME_UTC)
    status = get_time_index (ncid, Input.UTC, TIME_NEAREST_DATE, 
                             &(nt), &(itime1), &(itime2), &(dt),
                             Input.verbose, Input.quiet);
  else if (Input.time_zone==TIME_LAT)
    status = get_local_apparent_time_index (ncid, Input.LAT, TIME_NEAREST_DATE, 
                                            &(nt), &(itime1), &(itime2), &(dt),
                                            Input.verbose, Input.quiet);
  else {
    fprintf (stderr, "Error, unknown time_zone %d in write_netCDF_output \n", Input.time_zone);
    return status;
  }


  if (status != 0) {
    fprintf (stderr, "Error %d, during get_time_index in write_netCDF_output (uvspec_lex.l)\n", status);
    return status;
  }
  itime = itime1;


  /* get lat/pixel_x-index in the result grid, where to write data */
  ix = get_grid_index(x, x_grid, nx, periodic_x);
  if (ix < 0) {
    fprintf (stderr, "Error %d, while searching index in %s_grid \n", ix, x_name);
    return ix;
  }

  /* get lon/pixel_y-index in the result grid, where to write data */
  iy = get_grid_index(y, y_grid, ny, periodic_y);
  if (iy < 0) {
    fprintf (stderr, "Error %d, while searching index in %s_grid \n", iy, y_name);
    return iy;
  }



  /* ----------------------------------------------------------------------- */

  /* check if lambda exists, (write lambda if not), and get dimension id for nlambda */
  write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                         "lambda", &id_lambda, "wavelength", "nm", output->wl.lambda_h, 
                         "lambda", &id_nlambda, output->wl.nlambda_h);

  /* check if zout exists, (write zout if not), and get dimension id for zout */
  switch (Input.atm.zout_source) {
  case OUTLEVEL_ZOUT_ABOVE_SUR:
    write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "zout_sur", &id_levels, "altitude above surface", "km", output->atm.zout_sur_org, 
                           "zout_sur", &id_nzout, output->atm.nzout_org );
    break;
  case OUTLEVEL_ZOUT_ABOVE_SEA:
    write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "zout_sea", &id_levels, "altitude above sea level", "km", output->atm.zout_sea_org, 
                           "zout_sea", &id_nzout, output->atm.nzout_org );
    break;
  case OUTLEVEL_ATM_LEVELS:
  case OUTLEVEL_ALL_LEVELS:
    write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "zout_sea", &id_levels, "altitude above sea level", "km", output->atm.zout_sea_org, 
                           "zout_sea", &id_nzout, output->atm.nzout_org ); /* Input.atm.nzout was not initialised! */
    break;
  case OUTLEVEL_PRESS:
    write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "p", &id_levels, "pressure", "hPa", Input.atm.press_zout, 
                           "p", &id_nzout, Input.atm.nzout);
    break;
  default:
    fprintf (stderr,"Error, unknown zout_source %d in write_netCDF_output (in uvspec_lex.l)\n", Input.atm.zout_source);
    return -2;
  }

  /* ----------------------------------------------------------------------- */


  /* initialisation of the index, where to write */
  id_dim_array3[POSITION_TIME_3D] = id_ntime;  
  id_dim_array3[POSITION_LAT_3D ] = id_ny;     /* one before last dimension is default y-axis in ncview */
  id_dim_array3[POSITION_LON_3D ] = id_nx;     /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_3D[POSITION_TIME_3D] = itime;
  index_3D[POSITION_LAT_3D]  = iy;               /* latitude or x-pixel */
  index_3D[POSITION_LON_3D]  = ix;               /* longitude or y-pixel */
 
  /* order of dimensions */
  id_dim_array4[POSITION_TIME_4D] = id_ntime;
  id_dim_array4[POSITION_ZOUT_4D] = id_nzout;
  id_dim_array4[POSITION_LAT_4D ] = id_ny;     /* one before last dimension is default y-axis in ncview */
  id_dim_array4[POSITION_LON_4D ] = id_nx;     /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_4D[POSITION_TIME_4D] = itime;
  n_4D[POSITION_ZOUT_4D]     = output->atm.nzout; /* loop over zout */
  index_4D[POSITION_LAT_4D]  = iy;                /* latitude or x-pixel */
  index_4D[POSITION_LON_4D]  = ix;                /* longitude or y-pixel */

  /* order of dimensions */
  id_dim_array5[POSITION_TIME_5D  ] = id_ntime;
  id_dim_array5[POSITION_LAMBDA_5D] = id_nlambda;
  id_dim_array5[POSITION_ZOUT_5D  ] = id_nzout;
  id_dim_array5[POSITION_LAT_5D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
  id_dim_array5[POSITION_LON_5D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

  /* initialisation of the index, where to write */
  index_5D[POSITION_TIME_5D] = itime;
  n_5D[POSITION_LAMBDA_5D]   = output->wl.nlambda_h; /* loop over lambda */    
  n_5D[POSITION_ZOUT_5D]     = output->atm.nzout;    /* loop over zout */ 
  index_5D[POSITION_LAT_5D]  = iy;                   /* latitude or x-pixel */
  index_5D[POSITION_LON_5D]  = ix;                   /* longitude or y-pixel */

  /* ----------------------------------------------------------------------- */

  /* allocate memory for id_dimensions */
  if ((id_dim  = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for id_dim  in write_netCDF_output (in uvspec_lex.l)\n");
    return -1;
  }

  /* allocate memory for id_variables */
  if ((id_data = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for id_data in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }

  /* allocate memory for marker, if result can be calculated by the solver */
  if ((valid_result = (int *) calloc (Input.n_output_user, sizeof (int *))) == NULL) {
    fprintf (stderr,"Error, allocating memory for valid_result in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }

  /* check if result exist for the specific solver */
  for (io=0; io<Input.n_output_user; io++)
    status += check_if_result_exists(Input.output_user, Input.n_output_user, Input.rte.solver, 
              &(valid_result), Input.quiet);
  if (status != 0) {
    fprintf (stderr,"Error, during check_if_result_exists in write_netCDF_output (in uvspec_lex.l)\n");
    return -2;
  }


  switch(Input.processing) {
  case PROCESS_NONE:
    switch(Input.output_unit) {
    case UNIT_NOT_DEFINED:
      strcpy (irradiance_units, "unknown units");
      strcpy (radiance_units,  "unknown units");
      break;
    case UNIT_PER_NM:
      strcpy (irradiance_units, "(W or mW)/(m^2 nm)");
      strcpy (radiance_units,  "(W or mW)/(m^2 nm sr)");
      break;
    case UNIT_PER_CMm1:
      strcpy (irradiance_units, "(W or mW)/(m^2 cm^-1)");
      strcpy (radiance_units,  "(W or mW)/(m^2 cm^-1 sr)");
      break;
    case UNIT_PER_CK_BAND:
      strcpy (irradiance_units, "(W or mW)/(m^2 ck-band)");
      strcpy (radiance_units,  "(W or mW)/(m^2 ck-band sr)");
      break;
      break;
    default:
      fprintf (stderr, "Error, unknown output units %d in write_netCDF_output (in uvspec_lex.l)\n", Input.output_unit);
      return -1;
    }
    break;
  case PROCESS_INT:
  case PROCESS_SUM:
    strcpy (irradiance_units, "(W or mW)/m^2");
    strcpy (radiance_units,  "(W or mW)/(m^2 sr)");
    break;

  case PROCESS_RGBNORM:
    strcpy (irradiance_units, "(red, green, blue) (normalised)");
    strcpy (radiance_units,   "(red, green, blue) (normalised)");
    break;

  case PROCESS_RGB:
    strcpy (irradiance_units, "(red, green, blue)");
    strcpy (radiance_units,   "(red, green, blue)");
    break;

  default:
    fprintf (stderr, "Error, unknown output processing %d in write_netCDF_output (in uvspec_lex.l)\n", Input.processing);
    return -1;
  }

  for (io=0; io<Input.n_output_user; io++) {

    if (valid_result[io] == TRUE) {

      switch (Input.output_user[io]) {

      case OUTPUT_USER_WAVE:
        /* is done in each case, already done above, do nothing here */
      break;

      case OUTPUT_USER_ZOUT_SUR:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          /* is done in each case, already done above, do nothing here  */
          break;
        case OUTLEVEL_PRESS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                          "zout_sur", "altitude above surface", "km", output->atm.zout_sur, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
          break;
        default:
          fprintf (stderr,"Error, unknown zout_source %d in write_netCDF_output (in uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        break;

      case OUTPUT_USER_ZOUT_SEA:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          /* is done in each case, already done above, do nothing here  */
          break;
        case OUTLEVEL_PRESS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                          "zout_sea", "altitude above sealevel", "km", output->atm.zout_sea, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
          break;
        default:
          fprintf (stderr,"Error, unknown zout_source %d in write_netCDF_output (in uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        break;

      case OUTPUT_USER_WAVENUMBER:
       /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->wl.nlambda_h, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          tmp_data_1D[iv] = 1.0E7/output->wl.lambda_h[iv];  /* convert from nm to cm-1 */

        status = write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "wavenumber", &(id_data[io]), "wavenumber", "cm-1", output->wl.lambda_h, 
                                        "wavenumber", &(id_dim[io]),  output->wl.nlambda_h);
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_EDIR:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "edir", "direct irradiance", irradiance_units, output->rfldir, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_EGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, Input.n_output_user, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++)
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "eglo", "global ( = direct + diffuse downward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
  
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;
  
      case OUTPUT_USER_EDN:

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "edn", "diffuse downward irradiance", irradiance_units, output->rfldn, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;
  
      case OUTPUT_USER_EUP:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "eup", "diffuse upward irradiance", irradiance_units, output->flup, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_ENET:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv]-output->flup[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "enet", "net ( = global - upward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_ESUM:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->rfldir[lev][iv]+output->rfldn[lev][iv]+output->flup[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "esum", "total ( = global + upward ) irradiance", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FDIR:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgso[lev][iv];
  
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "fdir", "direct actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*(output->uavgso[lev][iv]+output->uavgdn[lev][iv]);

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "fglo", "global ( = direct + diffuse downward ) actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FDN:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgdn[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "fdn", "diffuse downward actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_FUP:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavgup[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "fup", "diffuse upward actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_F:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = 4.0*PI*output->uavg[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "f", "total actinic flux", irradiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_UDIR:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "uavgdir", "direct mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgso, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_UGLO:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->uavgso[lev][iv]+output->uavgdn[lev][iv];

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "uavgglo", "global mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_UDN:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "uavgdn", "diffusive downward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgdn, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_UUP:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "uavgup", "diffusive upward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavgup, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_U:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "uavg", "diffusive upward mean radiance ( = actinic flux/ (4 pi) )", 
                                        radiance_units, output->uavg,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_ALB:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for eglo in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->flup[lev][iv]/(output->rfldir[lev][iv]+output->rfldn[lev][iv]);

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "albedo", "albedo", " ", tmp_data_3D, 
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));

        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_HEAT:
        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "heat", "heating rate", "K/day", output->heat,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        break;

      case OUTPUT_USER_W_RAD:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for tmp_data in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->w_zout[lev][iv]/3600.0; /* m/day -> m/s */

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "w_rad", "radiative vertical wind", "m/s", tmp_data_3D,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_M_RAD:
        if ((status = ASCII_calloc_float (&tmp_data_3D, output->atm.nzout, output->wl.nlambda_h)) !=0 ) {
          fprintf (stderr,"Error, allocating memory for tmp_data in write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        for (iv=0;iv<output->wl.nlambda_h;iv++)
          for (lev=0;lev<output->atm.nzout;lev++) 
            tmp_data_3D[lev][iv] = output->w_zout[lev][iv] * output->atm.microphys.dens_zout[MOL_AIR][lev] 
                                   * 1.0e+6 * 1.0e-3 * Input.atm.mol_mass[MOL_AIR] / avogadro;   /* kg/ (m2 day) */

        status = write_netCDF_5D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "m_rad", "radiative vertical mass flux", "kg/(m2 day)", tmp_data_3D,
                                        id_dim_array5, index_5D, dim5, n_5D, output->atm.zout_index, &(simulation_done));
        ASCII_free_float(tmp_data_3D, output->wl.nlambda_h);
        break;

      case OUTPUT_USER_P:
        switch (Input.atm.zout_source) {
        case OUTLEVEL_ZOUT_ABOVE_SUR:
        case OUTLEVEL_ZOUT_ABOVE_SEA:
        case OUTLEVEL_ATM_LEVELS:
        case OUTLEVEL_ALL_LEVELS:
          status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                          "p", "pressure", "hPa", output->atm.microphys.press_zout, 
                                          id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
          break;
        case OUTLEVEL_PRESS:
          /* is done in each case, already done above in 1D, do nothing here  */
          break;
        default:
          fprintf (stderr,"Error, unknown zout_source %d in write_netCDF_output (in uvspec_lex.l)\n", Input.atm.zout_source);
          return -2;
        }
        break;

      case OUTPUT_USER_T:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "T", "temperature", "K", output->atm.microphys.temper_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_T_D:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "T_d", "dew point temperature", "K", output->atm.microphys.temper_d_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;

      case OUTPUT_USER_THETA:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "theta", "potential temperature", "K", 
                                        output->atm.microphys.theta_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;

      case OUTPUT_USER_THETA_E:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "theta_e", "equivalent potential temperature", "K", 
                                        output->atm.microphys.theta_e_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;

      case OUTPUT_USER_CLWC:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "CLWC", "cloud liquid water content", "g/m3", output->wc.microphys.lwc_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;

      case OUTPUT_USER_CIWC:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "CIWC", "cloud ice water content",    "g/m3", output->ic.microphys.lwc_zout, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;

      case OUTPUT_USER_N:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "n_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "number density of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        var_name, long_name, "1/cm^3", output->atm.microphys.dens_zout[i_gas], 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        break;
  
      case OUTPUT_USER_RHO:   
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "rho_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "mass density of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */  /* 1.e+6: convert from cm-3 to m-3; 1.e-3: convert g -> kg */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[i_gas][lev] 
                               * 1.e+6 * 1.e-3 * Input.atm.mol_mass[i_gas] / avogadro ;  

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        var_name, long_name, "kg/m^3", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_VMR:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "vmr_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces before and behind text */

        strcpy (long_name, "volume mixing ratio of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[i_gas]  [lev]/
                             output->atm.microphys.dens_zout[MOL_AIR][lev];  

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        var_name, long_name, " ", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_MMR:
        i_gas = Input.output_user_gas[io];
        /* create variable name */
        gas = gas_number2string(i_gas);
        if (strcmp(" -1 ",gas)==0) {
          fprintf (stderr, "Error wrong gas identifier during write_netCDF_output (in uvspec_lex.l)\n");
          return -1;
        }
        strcpy (var_name, "mmr_");
        strcat (var_name, gas);
        strcpy (strtrim(var_name," "),var_name); /* get rid of spaces */

        strcpy (long_name, "mass mixing ratio of ");
        strcat (long_name, gas);
        strcpy (strtrim(long_name," "), long_name); /* get rid of spaces before and behind text */

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = (output->atm.microphys.dens_zout[i_gas]  [lev] * Input.atm.mol_mass[i_gas]  ) /
                             (output->atm.microphys.dens_zout[MOL_AIR][lev] * Input.atm.mol_mass[MOL_AIR]);

        /* write data */
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        var_name, long_name, "kg/kg", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_RH:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[MOL_H2O][lev]/
                             vapor_pressure(output->atm.microphys.temper_zout[lev]) * 100.0;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "rh", "relative humidity over water", "per cent", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_RH_ICE:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->atm.microphys.dens_zout[MOL_H2O][lev]/
                             vapor_pressure_over_ice(output->atm.microphys.temper_zout[lev]) * 100.0;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "rh_ice", "relative humidity over ice", "per cent", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);
        break;

      case OUTPUT_USER_WIND_U:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "u", "zonal wind", "m/s", output->wind.u, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_WIND_V:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "v", "meridional wind", "m/s", output->wind.v, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_WIND_W:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "w", "vertical wind", "m/s", output->wind.w, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_DTDX:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "dtheta_dx", "zonal gradiant of potential temperature", "K/m",    
                                        output->atm.microphys.dtheta_dx, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_DTDY:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "dtheta_dy", "merional gradiant of potential temperature", "K/m", 
                                        output->atm.microphys.dtheta_dy, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;

      case OUTPUT_USER_DTDZ:
        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                       "dtheta_dz", "vertical gradiant of potential temperature", "K/m", 
                                        output->atm.microphys.dtheta_dz, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);
       break;


      case OUTPUT_USER_HEAT_AD_X:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] *24*60*60; /* K/day */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "heat_ad_x", "zonal heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD_Y:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] *24*60*60; /* K/day */

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "heat_ad_y", "meridional heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD_Z:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev] *24*60*60;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "heat_ad_z", "vertical heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);

       break;

     case OUTPUT_USER_HEAT_AD:

        /* callocate data */
        if ((tmp_data_1D  = (float *) calloc (output->atm.nzout, sizeof (float))) == NULL) {
          fprintf (stderr,"Error, allocating memory for %s in write_netCDF_output (in uvspec_lex.l)\n", var_name);
          return -1;
        }

        /* calculate data */
        for (lev=0;lev<output->atm.nzout;lev++) 
          tmp_data_1D[lev] = (output->wind.u_zout[lev] * output->atm.microphys.dtheta_dx[lev] + 
                              output->wind.v_zout[lev] * output->atm.microphys.dtheta_dy[lev] +
                              output->wind.w_zout[lev] * output->atm.microphys.dtheta_dz[lev]  ) *24*60*60;

        status = write_netCDF_4D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                        "heat_ad", "heat advection", "K/day", tmp_data_1D, 
                                        id_dim_array4, index_4D, dim4, n_4D, output->atm.zout_index);

        /* free memory */
        free(tmp_data_1D);

       break;

      default:
        fprintf (stderr, "Error, unknown user output %d in write_netCDF_output (uvspec_lex.l) \n", Input.output_user[io]);
        return -1;
      }
    }
  }

  if (Input.rte.numu > 0 && Input.rte.nphi > 0) {
    /* write phi array */
    status = write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "phi", &id_phi, "radiance azimuth angle", "degrees", Input.rte.phi, 
                           "phi", &id_nphi, Input.rte.nphi );
    /* write umu array */
    status = write_netCDF_1D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                           "umu", &id_umu, "radiance zenith angle", "degrees", Input.rte.umu, 
                           "umu", &id_numu, Input.rte.numu );

    /* order of dimensions */
    id_dim_array6[POSITION_TIME_6D  ] = id_ntime;
    id_dim_array6[POSITION_LAMBDA_6D] = id_nlambda;
    id_dim_array6[POSITION_ZOUT_6D  ] = id_nzout;
    id_dim_array6[POSITION_UMU_6D   ] = id_numu;
    id_dim_array6[POSITION_LAT_6D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
    id_dim_array6[POSITION_LON_6D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

    /* initialisation of the index, where to write */
    index_6D[POSITION_TIME_6D]   = itime;
    n_6D    [POSITION_LAMBDA_6D] = output->wl.nlambda_h; /* loop over lambda */    
    n_6D    [POSITION_ZOUT_6D]   = output->atm.nzout;      /* loop over zout   */ 
    n_6D    [POSITION_UMU_6D]    = Input.rte.numu;       /* loop over umu    */ 
    index_6D[POSITION_LAT_6D]    = iy;                   /* latitude or x-pixel */
    index_6D[POSITION_LON_6D]    = ix;                   /* longitude or y-pixel */

    /* order of dimensions */
    id_dim_array7[POSITION_TIME_7D  ] = id_ntime;
    id_dim_array7[POSITION_LAMBDA_7D] = id_nlambda;
    id_dim_array7[POSITION_ZOUT_7D  ] = id_nzout;
    id_dim_array7[POSITION_UMU_7D   ] = id_nphi;
    id_dim_array7[POSITION_PHI_7D   ] = id_numu;
    id_dim_array7[POSITION_LAT_7D   ] = id_ny;  /* one before last dimension is default y-axis in ncview */
    id_dim_array7[POSITION_LON_7D   ] = id_nx;  /*            last dimension is default x-axis in ncview */

    /* initialisation of the index, where to write */
    index_7D[POSITION_TIME_7D]   = itime;
    n_7D    [POSITION_LAMBDA_7D] = output->wl.nlambda_h; /* loop over lambda */    
    n_7D    [POSITION_ZOUT_7D]   = output->atm.nzout;      /* loop over zout   */ 
    n_7D    [POSITION_UMU_7D]    = Input.rte.numu;       /* loop over umu    */ 
    n_7D    [POSITION_PHI_7D]    = Input.rte.nphi;       /* loop over umu    */ 
    index_7D[POSITION_LAT_7D]    = iy;                   /* latitude  or x-pixel */
    index_7D[POSITION_LON_7D]    = ix;                   /* longitude or y-pixel */

    /* write horizontal averaged radiance u0u */
    status = write_netCDF_6D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                    "u0u", "horizontally averaged radiance", radiance_units, output->u0u, 
                                    id_dim_array6, index_6D, 6, n_6D, output->atm.zout_index, &(simulation_done));

    /* write radiance */
    status = write_netCDF_7D_float (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                    "uu", "radiance", radiance_units, output->uu, 
                                    id_dim_array7, index_7D, 7, n_7D, output->atm.zout_index, &(simulation_done));
  }

  status = write_netCDF_3D_byte (ncid, Input.filename[FN_NETCDF_OUTFILE], 
                                 "done", "0=not jet calculated, 1=calculation done", " ", simulation_done, 
                                 id_dim_array3, index_3D, dim3);

  if (status!=0) {
  
  }

  nc_close (ncid);
  return status;

#else
    fprintf (stderr, " ***********************************************************************\n");
    fprintf (stderr, " * You have built uvspec without libnetcdf and hence cannot            *\n");
    fprintf (stderr, " * use the netCDF output option. Please get netcdf and rebuild.        *\n");
    fprintf (stderr, " ***********************************************************************\n");
    return -1;
#endif

}


int check_if_result_exists (int *output_user, int n_output_user, int solver, int **valid_result, int quiet)
{
  int status=0;
  int io=0;

  for ( io=0; io< n_output_user; io++ ) {
    switch (solver) {
    case TWOSTRPP:
    case TWOSTR:
      /* no dir, dn up parts of the actinic flux and radiance */

      switch (Input.output_user[io]) {
      case OUTPUT_USER_FDIR:
      case OUTPUT_USER_FGLO:
      case OUTPUT_USER_FDN:
      case OUTPUT_USER_FUP:
      case OUTPUT_USER_UDIR:
      case OUTPUT_USER_UGLO:
      case OUTPUT_USER_UDN:
        (*valid_result)[io] = FALSE;
        if (!quiet) {
          fprintf (stderr, "*** Warning: twostream solver does not calculated upward and \n");
          fprintf (stderr, "             downward part of the radiance/actinic flux. THIS OUTPUT IS SKIPPED. \n");
        }
        break;
      case OUTPUT_USER_WAVE:
      case OUTPUT_USER_WAVENUMBER:
      case OUTPUT_USER_ZOUT_SUR:
      case OUTPUT_USER_ZOUT_SEA:
      case OUTPUT_USER_EDIR:
      case OUTPUT_USER_EGLO:
      case OUTPUT_USER_EDN:
      case OUTPUT_USER_EUP:
      case OUTPUT_USER_ENET:
      case OUTPUT_USER_ESUM:
      case OUTPUT_USER_F:
      case OUTPUT_USER_U:
      case OUTPUT_USER_ALB:
      case OUTPUT_USER_HEAT:
      case OUTPUT_USER_W_RAD:
      case OUTPUT_USER_M_RAD:
      case OUTPUT_USER_P:
      case OUTPUT_USER_T:
      case OUTPUT_USER_T_D:
      case OUTPUT_USER_THETA:
      case OUTPUT_USER_THETA_E:
      case OUTPUT_USER_N:
      case OUTPUT_USER_RHO:   
      case OUTPUT_USER_VMR:
      case OUTPUT_USER_MMR:
      case OUTPUT_USER_CLWC:
      case OUTPUT_USER_CIWC:
      case OUTPUT_USER_RH:
      case OUTPUT_USER_RH_ICE:
      case OUTPUT_USER_WIND_U:
      case OUTPUT_USER_WIND_V:
      case OUTPUT_USER_WIND_W:
      case OUTPUT_USER_HEAT_AD_X:
      case OUTPUT_USER_HEAT_AD_Y:
      case OUTPUT_USER_HEAT_AD_Z:
      case OUTPUT_USER_HEAT_AD:
        (*valid_result)[io] = TRUE;
        break;
      default:
        fprintf (stderr, "Error, unknown user output %d in check_if_result_exists (uvspec_lex.l)\n", Input.output_user[io]);
        return -1;
      }
      break;      

    case DISORT:
    case DISORT2:
    case SDISORT:
    case SPSDISORT:
    case QDISORT:
    case MONTECARLO:
    case NULLSOLVER:
    case SUCORD:
      /* everything possible */
      (*valid_result)[io] = TRUE;
      break;

    case POLRADTRAN:
      fprintf (stderr, "Error, netCDF for POLRADTRAN not yet implemented in check_if_result_exists (uvspec_lex.l)\n");
      return -1;
      break;

    case SSS:
    case SSSI:
    case SSS2:
    case TZS:
      /* only radiance in a direction (no fluxes, no actinic fluxes, no averaged fluxes) */
      fprintf (stderr, "Error, netCDF for 'solar single scattering' and 'thermal_zero_scattering'\n");
      fprintf (stderr, "       not yet implemented (in uvspec_lex.l)\n");
      return -2;
      break;

    default:
      fprintf (stderr, "Error, unknown rte_solver %d in check_if_result_exists (in uvspec_lex.l)\n", solver);
      return -3;
    }
  }

  return status;
}




static void free_output(input_struct Input, output_struct *output) 
{

  if (output->down_flux != NULL)
    ASCII_free_float_3D (output->down_flux, 
	                 output->atm.nzout, Input.rte.pol_nstokes);

  if (output->down_flux_t != NULL)
    ASCII_free_float_3D (output->down_flux_t, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux != NULL)
    ASCII_free_float_3D (output->up_flux, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->up_flux_t != NULL)
    ASCII_free_float_3D (output->up_flux_t, 
			 output->atm.nzout, Input.rte.pol_nstokes);
  
  if (output->down_rad != NULL)
    ASCII_free_float_5D (output->down_rad, 
			 output->atm.nzout, Input.rte.nphi, 
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->down_rad_t != NULL)
    ASCII_free_float_5D (output->down_rad_t, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->up_rad != NULL)
    ASCII_free_float_5D (output->up_rad, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);
  
  if (output->up_rad_t != NULL)
    ASCII_free_float_5D (output->up_rad_t, 
			 output->atm.nzout, Input.rte.nphi,
			 Input.rte.numu, Input.rte.pol_nstokes);

  if (output->mc.dtaer != NULL) free(output->mc.dtaer);
  if (output->mc.dtair != NULL) free(output->mc.dtair);
  if (output->mc.dtcld != NULL) free(output->mc.dtcld);
  if (output->mc.dtice != NULL) free(output->mc.dtice);
  if (output->mc.dtmol != NULL) free(output->mc.dtmol);
  if (output->mc.omaer != NULL) free(output->mc.omaer);
  if (output->mc.omcld != NULL) free(output->mc.omcld);
  if (output->mc.omice != NULL) free(output->mc.omice);

  if (output->dtauc     != NULL) free(output->dtauc);
  if (output->dtauc_md  != NULL) free(output->dtauc_md);
  if (output->sza_h     != NULL) free(output->sza_h);
  if (output->sza_s     != NULL) free(output->sza_s);
  if (output->mu_values != NULL) free(output->mu_values);

}


static float yytext2float(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return (float) dnumber;
}


static double yytext2double(char* yytext, int which_token, int max_token) {
  double dnumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);
  
  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }
  
  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);
  
  dnumber = atof(s);
  return dnumber;
}


static float* yytext2floats(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;
  char *tmpstring=NULL;
  
  /* Fill up ary */
  float *ary = (float *) calloc (nary, sizeof(float));
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);
  
  p = strtok(tmpstring,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atof(p);
  
  return ary;
}


static int* yytext2integers(char* yytext, int nary) {
  int i=0;
  char token[]=" ";
  char *p=NULL;
  char *tmpstring=NULL;
  
  /* Fill up ary */
  int *ary    = (int *) calloc (nary, sizeof(int));

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);


  p = strtok(tmpstring,token);
  i = 0;

  while ((p = strtok(NULL,token)) != NULL)
    ary[i++] = atoi(p);

  return ary;
}


static int yytext2ntokens(char* yytext) {
  int nary=0;
  char token[]=" ";
  char *p=NULL;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  /* Get number of ary elements */
  nary = 0;
  p = strtok(tmpstring,token);

  while ((p = strtok(NULL,token)) != NULL) 
    nary++;

  return nary;
}


static int yytext2int(char* yytext, int which_token, int max_token) {
  int inumber=0;
  char token[]=" ";
  char *p=NULL, *s=NULL;
  int count=0;
  char *tmpstring=NULL;
  
  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  inumber = atoi(s);
  
  return inumber;
}


static char* yytext2string(char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL, *string=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  string = (char *) calloc (strlen(s)+1, sizeof (char));
  strcpy (string, s);
  return string;
}


/* like yytext2string, but string is copied to a pre-allocated string dest */
static int yytextCstring(char *dest, char* yytext, int which_token, int max_token) {
  char token[]=" ";
  char *p=NULL, *s=NULL, *tmpstring=NULL;
  int count=0;

  tmpstring = (char *) calloc (strlen(yytext)+1, sizeof (char));
  strcpy (tmpstring, yytext);

  p = strtok(tmpstring,token);
  count = 1;
  while ((p = strtok(NULL,token)) != NULL) {
    count++;
    if (count == which_token)
      s = p;
  }

  if ((count != which_token) && (count>max_token))
    fprintf(stderr,"Wrong number of arguments on line %d\n", line_number+1);

  strcpy (dest, s);
  return 0;
}
